@@ -22,24 +22,18 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+
+#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-//#include <assert.h>
 #include "types.h"
 #include "common.h"
-#include "fnctdsk.h"
 #include "analyse.h"
-#include "intrf.h"
-#include "intrfn.h"
-#include "savehdr.h"
-#include "lang.h"
 #include "bfs.h"
 #include "bsd.h"
 #include "cramfs.h"
+#include "exfat.h"
 #include "ext2.h"
 #include "fat.h"
 #include "fatx.h"
@@ -47,6 +41,7 @@
 #include "hfsp.h"
 #include "jfs_superblock.h"
 #include "jfs.h"
+#include "hpfs.h"
 #include "luks.h"
 #include "lvm.h"
 #include "md.h"
@@ -63,7 +58,7 @@
 int search_NTFS_backup(unsigned char *buffer, disk_t *disk_car,partition_t *partition, const int verbose, const int dump_ind)
 {
 //  assert(sizeof(struct ntfs_boot_sector)<=DEFAULT_SECTOR_SIZE);
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
     return -1;
   /* NTFS recovery using backup sector */
   if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)buffer,partition,verbose,dump_ind,1)==0)
@@ -75,7 +70,7 @@ int search_HFS_backup(unsigned char *buffer, disk_t *disk_car,partition_t *parti
 {
 //  assert(sizeof(hfs_mdb_t)<=0x400);
 //  assert(sizeof(struct hfsp_vh)==0x200);
-  if(disk_car->read(disk_car,0x400, buffer, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, buffer, 0x400, partition->part_offset) != 0x400)
     return -1;
   /* HFS recovery using backup sector */
   if(recover_HFS(disk_car,(const hfs_mdb_t*)buffer,partition,verbose,dump_ind,1)==0)
@@ -91,10 +86,25 @@ int search_HFS_backup(unsigned char *buffer, disk_t *disk_car,partition_t *parti
   return 0;
 }
 
+int search_EXFAT_backup(unsigned char *buffer, disk_t *disk, partition_t *partition)
+{
+  if(disk->pread(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
+    return -1;
+  /* EXFAT recovery using backup sector */
+  if(recover_EXFAT(disk, (const struct exfat_super_block *)buffer, partition)==0)
+  {
+    strncpy(partition->info,"EXFAT found using backup sector!",sizeof(partition->info));
+    partition->sb_offset=6*512;
+    partition->part_offset-=partition->sb_offset;  /* backup sector */
+    return 1;
+  }
+  return 0;
+}
+
 int search_FAT_backup(unsigned char *buffer, disk_t *disk_car,partition_t *partition, const int verbose, const int dump_ind)
 {
 //  assert(sizeof(struct fat_boot_sector)==DEFAULT_SECTOR_SIZE);
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
     return -1;
   /* FAT32 recovery using backup sector */
   if(recover_FAT(disk_car,(const struct fat_boot_sector*)buffer,partition,verbose,dump_ind,1)==0)
@@ -118,17 +128,18 @@ int search_type_0(unsigned char *buffer,disk_t *disk_car,partition_t *partition,
   {
     log_trace("search_type_0 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, buffer, 8 * DEFAULT_SECTOR_SIZE, partition->part_offset) != 8 * DEFAULT_SECTOR_SIZE)
     return -1;
-  if(recover_Linux_SWAP(disk_car,(const union swap_header *)buffer,partition,verbose,dump_ind)==0) return 1;
+  if(recover_Linux_SWAP((const union swap_header *)buffer, partition)==0) return 1;
   if(recover_LVM(disk_car,(const pv_disk_t*)buffer,partition,verbose,dump_ind)==0) return 1;
   if(recover_FAT(disk_car,(const struct fat_boot_sector*)buffer,partition,verbose,dump_ind,0)==0) return 1;
-  if(recover_HPFS(disk_car,(const struct fat_boot_sector*)buffer,partition,verbose,dump_ind)==0) return 1;
+  if(recover_EXFAT(disk_car, (const struct exfat_super_block*)buffer, partition)==0) return 1;
+  if(recover_HPFS(disk_car,(const struct fat_boot_sector*)buffer,partition,verbose)==0) return 1;
   if(recover_OS2MB(disk_car,(const struct fat_boot_sector*)buffer,partition,verbose,dump_ind)==0) return 1;
   if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)buffer,partition,verbose,dump_ind,0)==0) return 1;
   if(recover_netware(disk_car,(const struct disk_netware *)buffer,partition)==0) return 1;
   if(recover_xfs(disk_car,(const struct xfs_sb*)buffer,partition,verbose,dump_ind)==0) return 1;
-  if(recover_FATX(disk_car,(const struct disk_fatx*)buffer,partition,verbose,dump_ind)==0) return 1;
+  if(recover_FATX((const struct disk_fatx*)buffer, partition)==0) return 1;
   if(recover_LUKS(disk_car,(const struct luks_phdr*)buffer,partition,verbose,dump_ind)==0) return 1;
   { /* MD 1.1 */
     const struct mdp_superblock_1 *sb1=(const struct mdp_superblock_1 *)buffer;
@@ -153,10 +164,10 @@ int search_type_1(unsigned char *buffer, disk_t *disk_car,partition_t *partition
   {
     log_trace("search_type_1 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, buffer, 8 * DEFAULT_SECTOR_SIZE, partition->part_offset) != 8 * DEFAULT_SECTOR_SIZE)
     return -1;
   if(recover_BSD(disk_car,(const struct disklabel *)(buffer+0x200),partition,verbose,dump_ind)==0) return 1;
-  if(recover_BeFS(disk_car,(const struct disk_super_block *)(buffer+0x200),partition,verbose,dump_ind)==0) return 1;
+  if(recover_BeFS(disk_car, (const struct disk_super_block *)(buffer+0x200), partition, dump_ind)==0) return 1;
   if(recover_cramfs(disk_car,(const struct cramfs_super*)(buffer+0x200),partition,verbose,dump_ind)==0) return 1;
   if(recover_sysv(disk_car,(const struct sysv4_super_block*)(buffer+0x200),partition,verbose,dump_ind)==0) return 1;
   if(recover_LVM2(disk_car,(const unsigned char*)(buffer+0x200),partition,verbose,dump_ind)==0) return 1;
@@ -173,7 +184,7 @@ int search_type_2(unsigned char *buffer, disk_t *disk_car,partition_t *partition
   {
     log_trace("search_type_2 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,1024, (buffer+0x400), partition->part_offset+1024)!=0)
+  if(disk_car->pread(disk_car, (buffer + 0x400), 1024, partition->part_offset + 1024) != 1024)
     return -1;
   if(recover_EXT2(disk_car,(const struct ext2_super_block*)(buffer+0x400),partition,verbose,dump_ind)==0) return 1;
   if(recover_HFS(disk_car,(const hfs_mdb_t*)(buffer+0x400),partition,verbose,dump_ind,0)==0) return 1;
@@ -187,7 +198,7 @@ int search_type_8(unsigned char *buffer, disk_t *disk_car,partition_t *partition
   {
     log_trace("search_type_8 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,4096, buffer, partition->part_offset+4096)!=0)
+  if(disk_car->pread(disk_car, buffer, 4096, partition->part_offset + 4096) != 4096)
     return -1;
   { /* MD 1.2 */
     const struct mdp_superblock_1 *sb1=(const struct mdp_superblock_1 *)buffer;
@@ -208,7 +219,7 @@ int search_type_16(unsigned char *buffer, disk_t *disk_car,partition_t *partitio
   {
     log_trace("search_type_16 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,3*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+16*512)!=0) /* 8k offset */
+  if(disk_car->pread(disk_car, buffer, 3 * DEFAULT_SECTOR_SIZE, partition->part_offset + 16 * 512) != 3 * DEFAULT_SECTOR_SIZE) /* 8k offset */
     return -1;
   /* Test UFS */
   if(recover_ufs(disk_car,(const struct ufs_super_block*)buffer,partition,verbose,dump_ind)==0) return 1;
@@ -223,15 +234,9 @@ int search_type_64(unsigned char *buffer, disk_t *disk_car,partition_t *partitio
     log_trace("search_type_64 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
   /* Test JFS */
-#if 0
-  if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+64*512)!=0) /* 32k offset */
-    return -1;
-  if(recover_JFS(disk_car,(const struct jfs_superblock*)buffer,partition,verbose,dump_ind)==0) return 1;
-#else
-  if(disk_car->read(disk_car,3*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+63*512)!=0) /* 32k offset */
+  if(disk_car->pread(disk_car, buffer, 3 * DEFAULT_SECTOR_SIZE, partition->part_offset + 63 * 512) != 3 * DEFAULT_SECTOR_SIZE) /* 32k offset */
     return -1;
   if(recover_JFS(disk_car,(const struct jfs_superblock*)(buffer+0x200),partition,verbose,dump_ind)==0) return 1;
-#endif
   return 0;
 }
 
@@ -245,115 +250,11 @@ int search_type_128(unsigned char *buffer, disk_t *disk_car,partition_t *partiti
   {
     log_trace("search_type_128 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-#if 0
   /* Test ReiserFS */
-  if(disk_car->read(disk_car,9*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+128*512)!=0) /* 64k offset */
-    return -1;
-  if(recover_rfs(disk_car,(const struct reiserfs_super_block*)buffer,partition,verbose,dump_ind)==0) return 1;
-  /* Test UFS2 */
-  if(recover_ufs(disk_car,(const struct ufs_super_block*)buffer,partition,verbose,dump_ind)==0) return 1;
-#else
-  /* Test ReiserFS */
-  if(disk_car->read(disk_car,11*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+126*512)!=0) /* 64k offset */
+  if(disk_car->pread(disk_car, buffer, 11 * DEFAULT_SECTOR_SIZE, partition->part_offset + 126 * 512) != 11 * DEFAULT_SECTOR_SIZE) /* 64k offset */
     return -1;
   if(recover_rfs(disk_car,(const struct reiserfs_super_block*)(buffer+0x400),partition,verbose,dump_ind)==0) return 1;
   /* Test UFS2 */
   if(recover_ufs(disk_car,(const struct ufs_super_block*)(buffer+0x400),partition,verbose,dump_ind)==0) return 1;
-#endif
   return 0;
 }
-
-list_part_t *search_superblock(disk_t *disk_car, const partition_t *partition, const int verbose, const int dump_ind, const int interface)
-{
-  unsigned char *buffer=(unsigned char *)MALLOC(2*0x200);
-  uint64_t hd_offset;
-  int nbr_sb=0;
-  list_part_t *list_part=NULL;
-  int ind_stop=0;
-  unsigned long int old_percent=0;
-  struct ext2_super_block *sb=(struct ext2_super_block *)buffer;
-  partition_t *new_partition=partition_new(disk_car->arch);
-  log_trace("search_superblock\n");
-#ifdef HAVE_NCURSES
-  if(interface>0)
-  {
-    aff_copy(stdscr);
-    wmove(stdscr,4,0);
-    wprintw(stdscr,"%s",disk_car->description(disk_car));
-    mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
-    wmove(stdscr,6,0);
-    aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
-    wmove(stdscr,22,0);
-    wattrset(stdscr, A_REVERSE);
-    waddstr(stdscr,"  Stop  ");
-    wattroff(stdscr, A_REVERSE);
-  }
-#endif
-  for(hd_offset=0;hd_offset<partition->part_size && nbr_sb<10 && ind_stop==0;hd_offset+=DEFAULT_SECTOR_SIZE)
-  {
-#ifdef HAVE_NCURSES
-    unsigned long int percent;
-    percent=hd_offset*100/partition->part_size;
-    if(interface>0 && percent!=old_percent)
-    {
-      wmove(stdscr,9,0);
-      wclrtoeol(stdscr);
-      wprintw(stdscr,"Search ext2/ext3/ext4 superblock %10lu/%lu %lu%%", (long unsigned)(hd_offset/disk_car->sector_size),
-	  (long unsigned)(partition->part_size/disk_car->sector_size),percent);
-      wrefresh(stdscr);
-      ind_stop|=check_enter_key_or_s(stdscr);
-      old_percent=percent;
-    }
-#endif
-    /* ext2/ext3/ext4 */
-    if( hd_offset==(EXT2_MIN_BLOCK_SIZE<<0) ||
-	hd_offset==(EXT2_MIN_BLOCK_SIZE<<1) ||
-	hd_offset==(EXT2_MIN_BLOCK_SIZE<<2) ||
-      hd_offset==(1*(EXT2_MIN_BLOCK_SIZE<<0)*8*(EXT2_MIN_BLOCK_SIZE<<0)+2*512) ||
-      hd_offset==(1*(EXT2_MIN_BLOCK_SIZE<<1)*8*(EXT2_MIN_BLOCK_SIZE<<1)) ||
-      hd_offset==(1*(EXT2_MIN_BLOCK_SIZE<<2)*8*(EXT2_MIN_BLOCK_SIZE<<2)) ||
-      hd_offset%(3*(EXT2_MIN_BLOCK_SIZE<<0)*8*(EXT2_MIN_BLOCK_SIZE<<0)+2*512)==0 ||
-      hd_offset%(5*(EXT2_MIN_BLOCK_SIZE<<0)*8*(EXT2_MIN_BLOCK_SIZE<<0)+2*512)==0 ||
-      hd_offset%(7*(EXT2_MIN_BLOCK_SIZE<<0)*8*(EXT2_MIN_BLOCK_SIZE<<0)+2*512)==0 ||
-      hd_offset%(3*(EXT2_MIN_BLOCK_SIZE<<1)*8*(EXT2_MIN_BLOCK_SIZE<<1))==0 ||
-      hd_offset%(5*(EXT2_MIN_BLOCK_SIZE<<1)*8*(EXT2_MIN_BLOCK_SIZE<<1))==0 ||
-      hd_offset%(7*(EXT2_MIN_BLOCK_SIZE<<1)*8*(EXT2_MIN_BLOCK_SIZE<<1))==0 ||
-      hd_offset%(3*(EXT2_MIN_BLOCK_SIZE<<2)*8*(EXT2_MIN_BLOCK_SIZE<<2))==0 ||
-      hd_offset%(5*(EXT2_MIN_BLOCK_SIZE<<2)*8*(EXT2_MIN_BLOCK_SIZE<<2))==0 ||
-      hd_offset%(7*(EXT2_MIN_BLOCK_SIZE<<2)*8*(EXT2_MIN_BLOCK_SIZE<<2))==0)
-    {
-      if(disk_car->read(disk_car,1024, buffer, partition->part_offset+hd_offset)==0)
-      {
-	if(le16(sb->s_magic)==EXT2_SUPER_MAGIC)
-	{
-	  dup_partition_t(new_partition,partition);
-	  new_partition->part_offset+=hd_offset;
-	  if(recover_EXT2(disk_car,sb,new_partition,verbose,dump_ind)==0)
-	  {
-	    int insert_error=0;
-	    if(hd_offset<=(EXT2_MIN_BLOCK_SIZE<<2))
-	      new_partition->part_offset-=hd_offset;
-	    log_info("Ext2 superblock found at sector %llu (block=%llu, blocksize=%u)\n",
-		(long long unsigned) hd_offset/DEFAULT_SECTOR_SIZE,
-		(long long unsigned) hd_offset>>(EXT2_MIN_BLOCK_LOG_SIZE+le32(sb->s_log_block_size)),
-		EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size));
-#ifdef HAVE_NCURSES
-	    wmove(stdscr,10+nbr_sb,0);
-	    wprintw(stdscr,"Ext2 superblock found at sector %llu (block=%llu, blocksize=%u)        \n",
-		(long long unsigned) hd_offset/DEFAULT_SECTOR_SIZE,
-		(long long unsigned) hd_offset>>(EXT2_MIN_BLOCK_LOG_SIZE+le32(sb->s_log_block_size)),
-                EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size));
-#endif
-	    list_part=insert_new_partition(list_part, new_partition, 1, &insert_error);
-	    new_partition=partition_new(disk_car->arch);
-	    nbr_sb++;
-	  }
-	}
-      }
-    }
-  }
-  free(new_partition);
-  free(buffer);
-  return list_part;
-}
-
