@@ -2,7 +2,7 @@
 
     File: dir.h
 
-    Copyright (C) 2004-2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,20 +21,32 @@
  */
 #ifndef _DIR_H
 #define _DIR_H
-#include "list.h"
+#ifdef __cplusplus
+extern "C" {
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
-#define INTER_DIR 16
-#define DIR_NAME_LEN 255
+#include "list.h"
+#define DIR_NAME_LEN 1024
+#define FLAG_LIST_DELETED	1
+#define FLAG_LIST_MASK12	2
+#define FLAG_LIST_MASK16	4
+#define FLAG_LIST_PATHNAME	8
+/* capabilities */
+#define CAPA_LIST_DELETED	1
+
 typedef struct dir_data dir_data_t;
 typedef struct file_data file_data_t;
+
 struct dir_data
 {
   void *display;
   char current_directory[DIR_NAME_LEN];
   unsigned long int current_inode;
-  int debug;
+  int verbose;
+  unsigned int param;
+  unsigned int capabilities;
   file_data_t *(*get_dir)(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_inode);
   int (*copy_file)(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file);
   void (*close)(dir_data_t *dir_data);
@@ -42,26 +54,39 @@ struct dir_data
   void *private_dir_data;
 };
 
+#define	FILE_STATUS_DELETED	1
+#define	FILE_STATUS_MARKED	2
+
+/* TODO: add status to file_info and migrate file_data to file_info */
 struct file_data
 {
   file_data_t *prev;
   file_data_t *next;
   char	name[DIR_NAME_LEN];
-  struct stat filestat;
+  struct stat stat;
+  unsigned int status;
 };
 
-struct file_info {
+typedef struct 
+{
   struct td_list_head list;
   char name[4096];
   struct stat stat;
-};
+  unsigned int status;
+} file_info_t;
 
-int dir_aff_log(const disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const file_data_t*dir_list);
-void delete_list_file(file_data_t *file_list);
-int dir_partition_aff(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, char **current_cmd);
+int dir_aff_log(const dir_data_t *dir_data, const file_data_t*dir_list);
+int log_list_file(const disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list);
+unsigned int delete_list_file(file_data_t *file_list);
+void delete_list_file_info(struct td_list_head *list);
+int dir_whole_partition_log(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode);
 void mode_string (const unsigned int mode, char *str);
-FILE *create_file(const char *filename);
 int set_date(const char *pathname, time_t actime, time_t modtime);
+int set_mode(const char *pathname, unsigned int mode);
+FILE *fopen_local(char **localfilename, const char *localroot, const char *filename);
+char *gen_local_filename(const char *filename);
+char *mkdir_local(const char *localroot, const char *pathname);
+void mkdir_local_for_file(const char *filename);
 
 #define LINUX_S_IFMT  00170000
 #define LINUX_S_IFSOCK 0140000
@@ -105,4 +130,8 @@ int set_date(const char *pathname, time_t actime, time_t modtime);
 #define LINUX_S_ISFIFO(m)       (((m) & LINUX_S_IFMT) == LINUX_S_IFIFO)
 #define LINUX_S_ISSOCK(m)       (((m) & LINUX_S_IFMT) == LINUX_S_IFSOCK)
 
+int filesort(const struct td_list_head *a, const struct td_list_head *b);
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
 #endif
