@@ -2,7 +2,7 @@
 
     File: dirpart.c
 
-    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,21 +26,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
 #include "types.h"
 #include "common.h"
 #include "fat.h"
-#include "lang.h"
-#include "fnctdsk.h"
-#include "testdisk.h"
 #include "intrf.h"
 #ifdef HAVE_NCURSES
 #include "intrfn.h"
@@ -48,6 +36,7 @@
 #include <stdio.h>
 #endif
 #include "dir.h"
+#include "dirn.h"
 #include "ext2_dir.h"
 #include "fat_dir.h"
 #include "ntfs_dir.h"
@@ -56,6 +45,7 @@
 #include "ntfs.h"
 #include "adv.h"
 #include "log.h"
+#include "log_part.h"
 
 int dir_partition(disk_t *disk_car, const partition_t *partition, const int verbose, char **current_cmd)
 {
@@ -83,6 +73,7 @@ int dir_partition(disk_t *disk_car, const partition_t *partition, const int verb
       case UP_FAT32:
 	res=dir_partition_fat_init(disk_car,partition,&dir_data,verbose);
 	break;
+      case UP_EXT4:
       case UP_EXT3:
       case UP_EXT2:
 	res=dir_partition_ext2_init(disk_car,partition,&dir_data,verbose);
@@ -110,14 +101,14 @@ int dir_partition(disk_t *disk_car, const partition_t *partition, const int verb
   switch(res)
   {
     case -2:
-      aff_buffer(BUFFER_RESET,"Q");
+      screen_buffer_reset();
 #ifdef HAVE_NCURSES
       aff_copy(window);
       wmove(window,4,0);
       aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
 #endif
       log_partition(disk_car,partition);
-      aff_buffer(BUFFER_ADD,"Support for this filesystem hasn't been enable during compilation.\n");
+      screen_buffer_add("Support for this filesystem hasn't been enable during compilation.\n");
       screen_buffer_to_log();
       if(*current_cmd==NULL)
       {
@@ -127,14 +118,14 @@ int dir_partition(disk_t *disk_car, const partition_t *partition, const int verb
       }
       break;
     case -1:
-      aff_buffer(BUFFER_RESET,"Q");
+      screen_buffer_reset();
 #ifdef HAVE_NCURSES
       aff_copy(window);
       wmove(window,4,0);
       aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
 #endif
       log_partition(disk_car,partition);
-      aff_buffer(BUFFER_ADD,"Can't open filesystem. Filesystem seems damaged.\n");
+      screen_buffer_add("Can't open filesystem. Filesystem seems damaged.\n");
       screen_buffer_to_log();
       if(*current_cmd==NULL)
       {
@@ -144,8 +135,47 @@ int dir_partition(disk_t *disk_car, const partition_t *partition, const int verb
       }
       break;
     default:
-      dir_partition_aff(disk_car,partition,&dir_data,dir_data.current_inode,current_cmd);
-      dir_data.close(&dir_data);
+      {
+	int recursive=0;
+	if(*current_cmd!=NULL)
+	{
+	  int do_continue;
+	  do
+	  {
+	    do_continue=0;
+	    while(*current_cmd[0]==',')
+	      (*current_cmd)++;
+	    if(strncmp(*current_cmd,"recursive",9)==0)
+	    {
+	      (*current_cmd)+=9;
+	      recursive=1;
+	      do_continue=1;
+	    }
+	    else if(strncmp(*current_cmd,"fullpathname",12)==0)
+	    {
+	      (*current_cmd)+=12;
+	      dir_data.param|=FLAG_LIST_PATHNAME;
+	      do_continue=1;
+	    }
+	  } while(do_continue==1);
+	}
+	if(recursive>0)
+	  dir_whole_partition_log(disk_car,partition,&dir_data,dir_data.current_inode);
+	else
+	{
+#ifdef HAVE_NCURSES
+	  dir_partition_aff(disk_car, partition, &dir_data, dir_data.current_inode, current_cmd);
+#else
+	  {
+	    file_data_t *dir_list;
+	    dir_list=dir_data.get_dir(disk_car, partition, &dir_data, dir_data.current_inode);
+	    dir_aff_log(&dir_data, dir_list);
+	    delete_list_file(dir_list);
+	  }
+#endif
+	}
+	dir_data.close(&dir_data);
+      }
       break;
   }
 #ifdef HAVE_NCURSES
@@ -154,6 +184,7 @@ int dir_partition(disk_t *disk_car, const partition_t *partition, const int verb
 #ifdef HAVE_TOUCHWIN
   touchwin(stdscr);
 #endif
+  wrefresh(stdscr);
 #endif
   return res;
 }
