@@ -2,7 +2,7 @@
 
     File: fat.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
 
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #include <ctype.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -39,21 +40,40 @@
 #include "fat.h"
 #include "lang.h"
 #include "fnctdsk.h"
-#include "testdisk.h"
 #include "intrf.h"
-#ifdef HAVE_NCURSES
-#include "intrfn.h"
-#else
-#include <stdio.h>
-#endif
 #include "log.h"
-/* #include "guid_cmp.h" */
+#include "log_part.h"
 extern const arch_fnct_t arch_i386;
 extern const arch_fnct_t arch_mac;
 
-static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose);
+static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition);
 static void fat_set_part_name(partition_t *partition,const unsigned char *src,const int max_size);
+static int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct fat_boot_sector*fat_header);
 static int log_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size);
+static int test_OS2MB(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind);
+static int is_fat12(const partition_t *partition);
+static int is_fat16(const partition_t *partition);
+static int is_fat32(const partition_t *partition);
+
+#define DELETED_FLAG 0xe5 /* marks file as deleted when in name[0] */
+#define IS_FREE(n) (!*(n) || *(const unsigned char *) (n) == DELETED_FLAG)
+#define ATTR_RO      1  /* read-only */
+#define ATTR_HIDDEN  2  /* hidden */
+#define ATTR_SYS     4  /* system */
+#define ATTR_VOLUME  8  /* volume label */
+#define ATTR_DIR     16 /* directory */
+#define ATTR_ARCH    32 /* archived */
+
+#define ATTR_NONE    0 /* no attribute bits */
+#define ATTR_UNUSED  (ATTR_VOLUME | ATTR_ARCH | ATTR_SYS | ATTR_HIDDEN)
+	/* attribute bits that are copied "as is" */
+#define ATTR_EXT     (ATTR_RO | ATTR_HIDDEN | ATTR_SYS | ATTR_VOLUME)
+#define ATTR_EXT_MASK     (ATTR_RO | ATTR_HIDDEN | ATTR_SYS | ATTR_VOLUME | ATTR_DIR | ATTR_ARCH)
+	/* bits that are used by the Windows 95/Windows NT extended FAT */
+#define FAT12_BAD	0x0FF7
+#define FAT12_EOC	0x0FF8
+#define FAT16_BAD	0xFFF7
+#define FAT16_EOC	0xFFF8
 
 static void fat_set_part_name(partition_t *partition,const unsigned char *src,const int max_size)
 {
@@ -99,127 +119,6 @@ static int log_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upa
   return 0;
 }
 
-#ifdef HAVE_NCURSES
-static int dump_fat_info_ncurses(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size)
-{
-  switch(upart_type)
-  {
-    case UP_FAT12:
-      wprintw(stdscr,"FAT : 12\n");
-      break;
-    case UP_FAT16:
-      wprintw(stdscr,"FAT : 16\n");
-      break;
-    case UP_FAT32:
-      wprintw(stdscr,"FAT : 32\n");
-      break;
-    default:
-      wprintw(stdscr,"Not a FAT\n");
-      return 0;
-  }
-  wprintw(stdscr,"cluster_size %u\n", fh1->sectors_per_cluster);
-  wprintw(stdscr,"reserved     %u\n", le16(fh1->reserved));
-  if(sectors(fh1)!=0)
-    wprintw(stdscr,"sectors      %u\n", sectors(fh1));
-  if(le32(fh1->total_sect)!=0)
-    wprintw(stdscr,"total_sect   %u\n", (unsigned int)le32(fh1->total_sect));
-  if(upart_type==UP_FAT32)
-  {
-    wprintw(stdscr,"fat32_length %u\n", (unsigned int)le32(fh1->fat32_length));
-    wprintw(stdscr,"root_cluster %u\n", (unsigned int)le32(fh1->root_cluster));
-    wprintw(stdscr,"flags        %04X\n", le16(fh1->flags));
-    wprintw(stdscr,"version      %u.%u\n", fh1->version[0], fh1->version[1]);
-    wprintw(stdscr,"root_cluster %u\n", (unsigned int)le32(fh1->root_cluster));
-    wprintw(stdscr,"info_sector  %u\n", le16(fh1->info_sector));
-    wprintw(stdscr,"backup_boot  %u\n", le16(fh1->backup_boot));
-    if(fat32_get_free_count((const unsigned char*)fh1,sector_size)==0xFFFFFFFF)
-      wprintw(stdscr,"free_count   uninitialised\n");
-    else
-      wprintw(stdscr,"free_count   %lu\n",fat32_get_free_count((const unsigned char*)fh1,sector_size));
-    if(fat32_get_next_free((const unsigned char*)fh1,sector_size)==0xFFFFFFFF)
-      wprintw(stdscr,"next_free    uninitialised\n");
-    else
-      wprintw(stdscr,"next_free    %lu\n",fat32_get_next_free((const unsigned char*)fh1,sector_size));
-  } else {
-    wprintw(stdscr,"fat_length   %u\n", le16(fh1->fat_length));
-    wprintw(stdscr,"dir_entries  %u\n", get_dir_entries(fh1));
-  }
-  return 0;
-}
-#endif
-
-int dump_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size)
-{
-#ifdef HAVE_NCURSES
-  return dump_fat_info_ncurses(fh1, upart_type, sector_size);
-#else
-  return 0;
-#endif
-}
-
-#ifdef HAVE_NCURSES
-static int dump_2fat_info_ncurses(const struct fat_boot_sector*fh1, const struct fat_boot_sector*fh2, const upart_type_t upart_type, const unsigned int sector_size)
-{
-  switch(upart_type)
-  {
-    case UP_FAT12:
-      wprintw(stdscr,"FAT : 12\n");
-      break;
-    case UP_FAT16:
-      wprintw(stdscr,"FAT : 16\n");
-      break;
-    case UP_FAT32:
-      wprintw(stdscr,"FAT : 32\n");
-      break;
-    default:
-      wprintw(stdscr,"Not a FAT\n");
-      return 1;
-  }
-  wprintw(stdscr,"cluster_size %u %u\n", fh1->sectors_per_cluster, fh2->sectors_per_cluster);
-  wprintw(stdscr,"reserved     %u %u\n", le16(fh1->reserved),le16(fh2->reserved));
-  if(sectors(fh1)!=0 || sectors(fh2)!=0)
-    wprintw(stdscr,"sectors      %u %u\n", sectors(fh1), sectors(fh2));
-  if(le32(fh1->total_sect)!=0 || le32(fh2->total_sect)!=0)
-    wprintw(stdscr,"total_sect   %u %u\n", (unsigned int)le32(fh1->total_sect), (unsigned int)le32(fh2->total_sect));
-  if(upart_type==UP_FAT32)
-  {
-    wprintw(stdscr,"fat32_length %u %u\n", (unsigned int)le32(fh1->fat32_length), (unsigned int)le32(fh2->fat32_length));
-    wprintw(stdscr,"root_cluster %u %u\n", (unsigned int)le32(fh1->root_cluster), (unsigned int)le32(fh2->root_cluster));
-    wprintw(stdscr,"free_count   ");
-    if(fat32_get_free_count((const unsigned char*)fh1,sector_size)==0xFFFFFFFF)
-      wprintw(stdscr,"uninitialised ");
-    else
-      wprintw(stdscr,"%lu ",fat32_get_free_count((const unsigned char*)fh1,sector_size));
-    if(fat32_get_free_count((const unsigned char*)fh2,sector_size)==0xFFFFFFFF)
-      wprintw(stdscr,"uninitialised\n");
-    else
-      wprintw(stdscr,"%lu\n",fat32_get_free_count((const unsigned char*)fh2,sector_size));
-    wprintw(stdscr,"next_free    ");
-    if(fat32_get_next_free((const unsigned char*)fh1,sector_size)==0xFFFFFFFF)
-      wprintw(stdscr,"uninitialised ");
-    else
-      wprintw(stdscr,"%lu ",fat32_get_next_free((const unsigned char*)fh1,sector_size));
-    if(fat32_get_next_free((const unsigned char*)fh2,sector_size)==0xFFFFFFFF)
-      wprintw(stdscr,"uninitialised\n");
-    else
-      wprintw(stdscr,"%lu\n",fat32_get_next_free((const unsigned char*)fh2,sector_size));
-  } else {
-    wprintw(stdscr,"fat_length   %u %u\n", le16(fh1->fat_length), le16(fh2->fat_length));
-    wprintw(stdscr,"dir_entries  %u %u\n", get_dir_entries(fh1), get_dir_entries(fh2));
-  }
-  return 0;
-}
-#endif
-
-int dump_2fat_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector*fh2, const upart_type_t upart_type, const unsigned int sector_size)
-{
-#ifdef HAVE_NCURSES
-  return dump_2fat_info_ncurses(fh1, fh2, upart_type, sector_size);
-#else
-  return 0;
-#endif
-}
-
 int log_fat2_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector*fh2, const upart_type_t upart_type, const unsigned int sector_size)
 {
   switch(upart_type)
@@ -282,7 +181,7 @@ int check_FAT(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer;
   buffer=(unsigned char *)MALLOC(3*disk_car->sector_size);
-  if(disk_car->read(disk_car,3*disk_car->sector_size, buffer, partition->part_offset)!=0)
+  if((unsigned)disk_car->pread(disk_car, buffer, 3 * disk_car->sector_size, partition->part_offset) != 3 * disk_car->sector_size)
   {
     screen_buffer_add("check_FAT: can't read FAT boot sector\n");
     log_error("check_FAT: can't read FAT boot sector\n");
@@ -300,13 +199,13 @@ int check_FAT(disk_t *disk_car,partition_t *partition,const int verbose)
     free(buffer);
     return 1;
   }
-  set_FAT_info(disk_car,(const struct fat_boot_sector *)buffer,partition,verbose);
+  set_FAT_info(disk_car,(const struct fat_boot_sector *)buffer,partition);
   /*  screen_buffer_add("Ok\n"); */
   free(buffer);
   return 0;
 }
 
-static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose)
+static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_header, partition_t *partition)
 {
   const char*buffer=(const char*)fat_header;
   partition->fsname[0]='\0';
@@ -356,9 +255,12 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
       {
         offset_s=(cluster+cluster/2)/disk_car->sector_size;
         offset_o=(cluster+cluster/2)%disk_car->sector_size;
-        if(disk_car->read(disk_car,2*disk_car->sector_size, buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+        if((unsigned)disk_car->pread(disk_car, buffer, 2 * disk_car->sector_size,
+	      partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != 2 * disk_car->sector_size)
         {
-          log_error("get_next_cluster read error\n"); return 0;
+          log_error("get_next_cluster read error\n");
+	  free(buffer);
+	  return 0;
         }
         if((cluster&1)!=0)
           next_cluster=le16((*((uint16_t*)&buffer[offset_o])))>>4;
@@ -372,9 +274,12 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
         const uint16_t *p16=(const uint16_t*)buffer;
         offset_s=cluster/(disk_car->sector_size/2);
         offset_o=cluster%(disk_car->sector_size/2);
-        if(disk_car->read(disk_car,disk_car->sector_size, buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+        if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size,
+	      partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != disk_car->sector_size)
         {
-          log_error("get_next_cluster read error\n"); return 0;
+          log_error("get_next_cluster read error\n");
+	  free(buffer);
+	  return 0;
         }
         next_cluster=le16(p16[offset_o]);
         free(buffer);
@@ -385,9 +290,12 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
         const uint32_t *p32=(const uint32_t*)buffer;
         offset_s=cluster/(disk_car->sector_size/4);
         offset_o=cluster%(disk_car->sector_size/4);
-        if(disk_car->read(disk_car,disk_car->sector_size, buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+        if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size,
+	      partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != disk_car->sector_size)
         {
-          log_error("get_next_cluster read error\n"); return 0;
+          log_error("get_next_cluster read error\n");
+	  free(buffer);
+	  return 0;
         }
         /* FAT32 used 28 bits, the 4 high bits are reserved
          * 0x00000000: free cluster
@@ -432,7 +340,8 @@ int set_next_cluster(disk_t *disk_car,const partition_t *partition, const upart_
       free(buffer);
       return 1;
   }
-  if(disk_car->read(disk_car,buffer_size, buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+  if((unsigned)disk_car->pread(disk_car, buffer, buffer_size,
+	partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != buffer_size)
   {
     log_error("set_next_cluster read error\n");
     free(buffer);
@@ -466,9 +375,9 @@ int set_next_cluster(disk_t *disk_car,const partition_t *partition, const upart_
     default:	/* Avoid compiler warning */
       break;
   }
-  if(disk_car->write(disk_car,buffer_size, buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+  if((unsigned)disk_car->pwrite(disk_car, buffer, buffer_size, partition->part_offset + (uint64_t)(offset + offset_s) * disk_car->sector_size) != buffer_size)
   {
-    display_message("Write error: set_next_cluster write error\n");
+    log_error("Write error: set_next_cluster write error\n");
     free(buffer);
     return 1;
   }
@@ -486,12 +395,11 @@ unsigned int fat32_get_prev_cluster(disk_t *disk_car,const partition_t *partitio
 
   for(prev_cluster=2;prev_cluster<=no_of_cluster+1;prev_cluster++)
   {
-    unsigned int offset_s,offset_o;
-    offset_s=prev_cluster/(disk_car->sector_size/4);
+    unsigned int offset_o;
     offset_o=prev_cluster%(disk_car->sector_size/4);
     if((offset_o==0)||(prev_cluster==2))
     {
-      if(disk_car->read(disk_car,disk_car->sector_size, buffer, hd_offset)!=0)
+      if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, hd_offset) != disk_car->sector_size)
       {
         log_error("fat32_get_prev_cluster error\n"); return 0;
       }
@@ -522,7 +430,6 @@ static unsigned int get_prev_cluster(disk_t *disk_car,const partition_t *partiti
 
 int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind)
 {
-  upart_type_t upart_type=UP_UNK;
   uint64_t start_fat1,start_fat2,start_rootdir,start_data,part_size,end_data;
   unsigned long int no_of_cluster,fat_length,fat_length_calc;
   const char *buffer=(const char*)fat_header;
@@ -530,15 +437,13 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
         && (fat_header->ignored[0]==0xeb || fat_header->ignored[0]==0xe9)
         && (fat_header->fats==1 || fat_header->fats==2)))
     return 1;   /* Obviously not a FAT */
-  if(verbose>1)
+  if(verbose>1 || dump_ind!=0)
   {
     log_trace("test_FAT\n");
     log_partition(disk_car,partition);
   }
-#ifdef HAVE_NCURSES
   if(dump_ind!=0)
-    dump_ncurses(fat_header,DEFAULT_SECTOR_SIZE);
-#endif
+    dump_log(fat_header, DEFAULT_SECTOR_SIZE);
   if(!((fat_header->ignored[0]==0xeb && fat_header->ignored[2]==0x90)||fat_header->ignored[0]==0xe9))
   {
     screen_buffer_add(msg_CHKFAT_BAD_JUMP);
@@ -604,7 +509,6 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
   }
   if(no_of_cluster<4085)
   {
-    upart_type=UP_FAT12;
     if(verbose>0)
     {
       log_info("FAT12 at %u/%u/%u\n",
@@ -645,7 +549,6 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
   }
   else if(no_of_cluster<65525)
   {
-    upart_type=UP_FAT16;
     if(verbose>0)
     {
       log_info("FAT16 at %u/%u/%u\n",
@@ -681,7 +584,6 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
   }
   else
   {
-    upart_type=UP_FAT32;
     if(verbose>0)
     {
       log_info("FAT32 at %u/%u/%u\n",
@@ -761,20 +663,24 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
   }
   if(fat_header->fats>1)
     comp_FAT(disk_car,partition,fat_length,le16(fat_header->reserved));
-  if(le16(fat_header->heads)!=disk_car->CHS.head+1)
+  if(le16(fat_header->heads)!=disk_car->geom.heads_per_cylinder)
   {
-    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
-    log_warning("heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
+    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",
+	le16(fat_header->heads), disk_car->geom.heads_per_cylinder);
+    log_warning("heads/cylinder %u (FAT) != %u (HD)\n",
+	le16(fat_header->heads), disk_car->geom.heads_per_cylinder);
   }
-  if(le16(fat_header->secs_track)!=disk_car->CHS.sector)
+  if(le16(fat_header->secs_track)!=disk_car->geom.sectors_per_head)
   {
-    screen_buffer_add("Warning: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
-    log_warning("sect/track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
+    screen_buffer_add("Warning: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",
+	le16(fat_header->secs_track), disk_car->geom.sectors_per_head);
+    log_warning("sect/track %u (FAT) != %u (HD)\n",
+	le16(fat_header->secs_track), disk_car->geom.sectors_per_head);
   }
   return 0;
 }
 
-int comp_FAT(disk_t *disk_car,const partition_t *partition,const unsigned long int fat_size,const unsigned long int sect_res)
+int comp_FAT(disk_t *disk, const partition_t *partition, const unsigned long int fat_size, const unsigned long int sect_res)
 {
   /*
   return 0 if FATs match
@@ -782,37 +688,42 @@ int comp_FAT(disk_t *disk_car,const partition_t *partition,const unsigned long i
   unsigned int reste;
   uint64_t hd_offset, hd_offset2;
   unsigned char *buffer, *buffer2;
-  buffer=(unsigned char *)MALLOC(NBR_SECT*disk_car->sector_size);
-  buffer2=(unsigned char *)MALLOC(NBR_SECT*disk_car->sector_size);
-  hd_offset=partition->part_offset+(uint64_t)sect_res*disk_car->sector_size;
-  hd_offset2=hd_offset+(uint64_t)fat_size*disk_car->sector_size;
+  buffer=(unsigned char *)MALLOC(16*disk->sector_size);
+  buffer2=(unsigned char *)MALLOC(16*disk->sector_size);
+  hd_offset=partition->part_offset+(uint64_t)sect_res*disk->sector_size;
+  hd_offset2=hd_offset+(uint64_t)fat_size*disk->sector_size;
   reste=(fat_size>1000?1000:fat_size); /* Quick check ! */
+  reste*=disk->sector_size;
   while(reste>0)
   {
-    const unsigned int read_size=reste>NBR_SECT?NBR_SECT:reste;
+    const unsigned int read_size=(reste > 16 * disk->sector_size ? 16 * disk->sector_size :reste);
     reste-=read_size;
-    if(disk_car->read(disk_car,read_size*disk_car->sector_size, buffer, hd_offset))
+    if((unsigned)disk->pread(disk, buffer, read_size, hd_offset) != read_size)
     {
       log_error("comp_FAT: can't read FAT1\n");
+      free(buffer2);
+      free(buffer);
       return 1;
     }
-    if(disk_car->read(disk_car,read_size*disk_car->sector_size, buffer2, hd_offset2))
+    if((unsigned)disk->pread(disk, buffer2, read_size, hd_offset2) != read_size)
     {
       log_error("comp_FAT: can't read FAT2\n");
+      free(buffer2);
+      free(buffer);
       return 1;
     }
-    if(memcmp(buffer,buffer2,disk_car->sector_size*read_size)!=0)
+    if(memcmp(buffer, buffer2, read_size)!=0)
     {
       log_error("FAT differs, FAT sectors=%lu-%lu/%lu\n",
-          (unsigned long) ((hd_offset-partition->part_offset)/disk_car->sector_size-sect_res),
-          (unsigned long) ((hd_offset-partition->part_offset)/disk_car->sector_size-sect_res+read_size),
+          (unsigned long) ((hd_offset-partition->part_offset)/disk->sector_size-sect_res),
+          (unsigned long) ((hd_offset-partition->part_offset+read_size)/disk->sector_size-sect_res),
           fat_size); 
       free(buffer2);
       free(buffer);
       return 1;
     }
-    hd_offset+=read_size*disk_car->sector_size;
-    hd_offset2+=read_size*disk_car->sector_size;
+    hd_offset+=read_size;
+    hd_offset2+=read_size;
   }
   free(buffer2);
   free(buffer);
@@ -848,7 +759,7 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
   partition->sborg_offset=0;
   partition->sb_size=512;
   partition->sb_offset=0;
-  set_FAT_info(disk_car,fat_header,partition,verbose);
+  set_FAT_info(disk_car, fat_header, partition);
   switch(partition->upart_type)
   {
     case UP_FAT12:
@@ -908,17 +819,15 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
   return 0;
 }
 
-int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct fat_boot_sector*fat_header)
+static int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct fat_boot_sector*fat_header)
 {
   partition->fsname[0]='\0';
   if((fat_header->sectors_per_cluster>0)&&(fat_header->sectors_per_cluster<=128))
   {
     unsigned char *buffer=(unsigned char*)MALLOC(fat_header->sectors_per_cluster*disk_car->sector_size);
-    if(disk_car->read(disk_car,
-          fat_header->sectors_per_cluster*disk_car->sector_size,
-          buffer,
-          partition->part_offset +
-          (le16(fat_header->reserved)+fat_header->fats*le32(fat_header->fat32_length)+(uint64_t)(le32(fat_header->root_cluster)-2)*fat_header->sectors_per_cluster) * disk_car->sector_size))
+    if((unsigned)disk_car->pread(disk_car, buffer,
+	  fat_header->sectors_per_cluster * disk_car->sector_size,
+	  partition->part_offset + (le16(fat_header->reserved) + fat_header->fats * le32(fat_header->fat32_length) + (uint64_t)(le32(fat_header->root_cluster) - 2) * fat_header->sectors_per_cluster) * disk_car->sector_size) != fat_header->sectors_per_cluster * disk_car->sector_size)
     {
       log_error("fat32_set_part_name() cannot read FAT32 root cluster.\n");
     }
@@ -930,7 +839,6 @@ int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct f
       { /* Test attribut volume name and check if the volume name is erased or not */
         if(((buffer[i*0x20+0xB] & ATTR_EXT) !=ATTR_EXT) && ((buffer[i*0x20+0xB] & ATTR_VOLUME) !=0) && (buffer[i*0x20]!=0xE5))
         {
-          /*      dump_ncurses(&buffer[i*0x20],0x20); */
           fat_set_part_name(partition,&buffer[i*0x20],11);
           if(check_VFAT_volume_name(partition->fsname, 11))
             partition->fsname[0]='\0';
@@ -953,71 +861,10 @@ int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct f
   return 0;
 }
 
-int check_HPFS(disk_t *disk_car,partition_t *partition,const int verbose)
-{
-  unsigned char buffer[512];
-  if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset)!=0)
-  {
-    screen_buffer_add("check_HPFS: Read error\n");
-    log_error("check_HPFS: Read error\n");
-    return 1;
-  }
-  if(test_HPFS(disk_car,(const struct fat_boot_sector *)buffer,partition,verbose,0)!=0)
-  {
-    if(verbose>0)
-    {
-      log_info("\n\ntest_HPFS()\n");
-      log_partition(disk_car,partition);
-    }
-    return 1;
-  }
-  return 0;
-}
-
-int test_HPFS(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind)
-{
-  const char*buffer=(const char*)fat_header;
-  if(le16(fat_header->marker)==0xAA55)
-  {
-    if(memcmp(buffer+OS2_NAME,"IBM",3)==0)
-    {   /* D'apres une analyse de OS2 sur systeme FAT...
-           FAT_NAME1=FAT
-         */
-      if(verbose||dump_ind)
-      {
-        log_info("\nHPFS maybe at %u/%u/%u\n",
-            offset2cylinder(disk_car,partition->part_offset),
-            offset2head(disk_car,partition->part_offset),
-            offset2sector(disk_car,partition->part_offset));
-      }
-#ifdef HAVE_NCURSES
-      if(dump_ind)
-        dump_ncurses(buffer,DEFAULT_SECTOR_SIZE);
-#endif
-      partition->part_size=(uint64_t)(sectors(fat_header)>0?sectors(fat_header):le32(fat_header->total_sect)) *
-        fat_sector_size(fat_header);
-      partition->upart_type=UP_HPFS;
-      return 0;
-    }
-  }     /* fin marqueur de fin :)) */
-  return 1;
-}
-
-int recover_HPFS(disk_t *disk_car,const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind)
-{
-  if(test_HPFS(disk_car,fat_header,partition,verbose,0)!=0)
-    return 1;
-  partition->part_type_i386=P_HPFS;
-  partition->part_type_gpt=GPT_ENT_TYPE_MAC_HFS;
-  partition->fsname[0]='\0';
-  partition->info[0]='\0';
-  return 0;
-}
-
 int check_OS2MB(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char buffer[0x200];
-  if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset)!=0)
+  if((unsigned)disk_car->pread(disk_car, &buffer, disk_car->sector_size, partition->part_offset) != disk_car->sector_size)
   {
     screen_buffer_add("check_OS2MB: Read error\n");
     log_error("check_OS2MB: Read error\n");
@@ -1039,26 +886,25 @@ int recover_OS2MB(disk_t *disk_car, const struct fat_boot_sector*fat_header, par
 {
   if(test_OS2MB(disk_car, fat_header, partition, verbose, dump_ind))
     return 1;
-  partition->part_size=(uint64_t)(disk_car->CHS.head+1) * disk_car->CHS.sector*disk_car->sector_size; /* 1 cylinder */
+  /* 1 cylinder */
+  partition->part_size=(uint64_t)disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
   partition->part_type_i386=P_OS2MB;
   partition->fsname[0]='\0';
   partition->info[0]='\0';
   return 0;
 }
 
-int test_OS2MB(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind)
+static int test_OS2MB(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind)
 {
   const char*buffer=(const char*)fat_header;
   if(le16(fat_header->marker)==0xAA55 && memcmp(buffer+FAT_NAME1,"FAT     ",8)==0)
   {
     if(verbose||dump_ind)
     {
-      log_info("\nMarker (0xAA55) at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+      log_info("OS2MB at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
     }
-#ifdef HAVE_NCURSES
     if(dump_ind)
-      dump_ncurses(buffer,DEFAULT_SECTOR_SIZE);
-#endif
+      dump_log(buffer,DEFAULT_SECTOR_SIZE);
     partition->upart_type=UP_OS2MB;
     return 0;
   }
@@ -1098,7 +944,7 @@ int is_part_fat12(const partition_t *partition)
   return 0;
 }
 
-int is_fat12(const partition_t *partition)
+static int is_fat12(const partition_t *partition)
 {
   return (is_part_fat12(partition) || partition->upart_type==UP_FAT12);
 }
@@ -1130,7 +976,7 @@ int is_part_fat16(const partition_t *partition)
   return 0;
 }
 
-int is_fat16(const partition_t *partition)
+static int is_fat16(const partition_t *partition)
 {
   return (is_part_fat16(partition) || partition->upart_type==UP_FAT16);
 }
@@ -1165,7 +1011,7 @@ int is_part_fat32(const partition_t *partition)
   return 0;
 }
 
-int is_fat32(const partition_t *partition)
+static int is_fat32(const partition_t *partition)
 {
   return (is_part_fat32(partition) || partition->upart_type==UP_FAT32);
 }
@@ -1183,12 +1029,11 @@ int fat32_free_info(disk_t *disk_car,const partition_t *partition, const unsigne
   for(prev_cluster=2;prev_cluster<=no_of_cluster+1;prev_cluster++)
   {
     unsigned long int cluster;
-    unsigned int offset_s,offset_o;
-    offset_s=prev_cluster/(disk_car->sector_size/4);
+    unsigned int offset_o;
     offset_o=prev_cluster%(disk_car->sector_size/4);
     if((offset_o==0)||(prev_cluster==2))
     {
-      if(disk_car->read(disk_car,disk_car->sector_size, buffer, hd_offset)!=0)
+      if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, hd_offset) != disk_car->sector_size)
       {
         log_error("fat32_free_info read error\n");
         *next_free=0xFFFFFFFF;
