@@ -21,7 +21,15 @@
  */
 #ifndef _FAT_ADV_H
 #define _FAT_ADV_H
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int rebuild_FAT_BS(disk_t *disk_car,partition_t *partition, const int verbose, const int dump_ind,const int interface, const unsigned int expert, char**current_cmd);
 int FAT_init_rootdir(disk_t *disk_car, partition_t *partition, const int verbose);
 int repair_FAT_table(disk_t *disk_car, partition_t *partition, const int verbose);
+
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
 #endif
