@@ -23,9 +23,6 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
 #include <stdio.h>
 #include "types.h"
 #include "filegen.h"
