@@ -39,7 +39,7 @@ const file_hint_t file_hint_all= {
   .min_header_distance=0,
   .max_filesize=50*1024*1024,
   .recover=1,
-  .header_check=&header_check_all,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_all
 };
 
