@@ -46,12 +46,13 @@ static void register_header_check_indd(file_stat_t *file_stat);
 static int header_check_indd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void file_check_indd(file_recovery_t *file_recovery);
 
-file_hint_t file_hint_indd= {
+const file_hint_t file_hint_indd= {
   .extension="indd",
   .description="InDesign File",
   .min_header_distance=8192,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_indd
 };
 
