@@ -54,7 +54,9 @@ static void register_header_check_ogg(file_stat_t *file_stat)
 
 static int header_check_ogg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_ogg)
+  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_ogg &&
+      file_recovery->calculated_file_size == file_recovery->file_size)
     return 0;
   if(memcmp(buffer,ogg_header,sizeof(ogg_header))==0)
   {
@@ -74,7 +76,7 @@ static int data_check_ogg(const unsigned char *buffer, const unsigned int buffer
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 27 +255 < file_recovery->file_size + buffer_size/2)
   {
-    unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
+    const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
     if(memcmp(&buffer[i],ogg_header,sizeof(ogg_header))==0)
     {
       unsigned int number_page_segments;
