@@ -40,7 +40,7 @@ const file_hint_t file_hint_rns= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
-  .header_check=&header_check_rns,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_rns
 };
 
