@@ -39,7 +39,6 @@
 #include "filegen.h"
 #include "log.h"
 #include "memmem.h"
-#include "log.h"
 
 extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_jpg;
@@ -55,8 +54,9 @@ static void register_header_check_fasttxt(file_stat_t *file_stat);
 static int header_check_fasttxt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 static int data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-static void file_check_html(file_recovery_t *file_recovery);
 static void file_check_emlx(file_recovery_t *file_recovery);
+static void file_check_ers(file_recovery_t *file_recovery);
+static void file_check_html(file_recovery_t *file_recovery);
 static void file_check_xml(file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_fasttxt= {
@@ -71,7 +71,7 @@ const file_hint_t file_hint_fasttxt= {
 
 const file_hint_t file_hint_txt= {
   .extension="txt",
-  .description="Other text files: txt,html,asp,bat,C,jsp,perl,php,py... scripts",
+  .description="Other text files: txt,html,asp,bat,C,jsp,perl,php,py/emlx... scripts",
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
@@ -79,23 +79,32 @@ const file_hint_t file_hint_txt= {
   .register_header_check=&register_header_check_txt
 };
 
+static const unsigned char header_adr[25]	= "Opera Hotlist version 2.0";
+static const unsigned char header_bash[11]  	= "#!/bin/bash";
 static const unsigned char header_cls[24]	= {'V','E','R','S','I','O','N',' ','1','.','0',' ','C','L','A','S','S','\r','\n','B','E','G','I','N'};
+static const unsigned char header_cue[10]	= "REM GENRE ";
+static const unsigned char header_dc[6]		= "SC V10";
+static const unsigned char header_dif[12]	= { 'T', 'A', 'B', 'L', 'E', '\r', '\n', '0', ',', '1', '\r', '\n'};
+static const unsigned char header_ers[19]	= "DatasetHeader Begin";
+static const unsigned char header_ics[15]	= "BEGIN:VCALENDAR";
 static const unsigned char header_imm[13]	= {'M','I','M','E','-','V','e','r','s','i','o','n',':'};
-static const unsigned char header_imm2[13]	= {'R','e','t','u','r','n','-','P','a','t','h',':',' '};
+static const unsigned char header_lyx[7]	= {'#', 'L', 'y', 'X', ' ', '1', '.'};
+static const unsigned char header_m3u[7]	= {'#','E','X','T','M','3','U'};
 static const unsigned char header_mail[5]	= {'F','r','o','m',' '};
+static const unsigned char header_msf[19]	= "// <!-- <mdb:mork:z";
 static const unsigned char header_perlm[7] 	= "package";
-static const unsigned char header_rtf[5]	= { '{','\\','r','t','f'};
+static const unsigned char header_ram[7]	= "rtsp://";
+static const unsigned char header_ReceivedFrom[14]= {'R','e','c','e','i','v','e','d',':',' ','f','r','o','m'};
 static const unsigned char header_reg[8]  	= "REGEDIT4";
+static const unsigned char header_ReturnPath[13]= {'R','e','t','u','r','n','-','P','a','t','h',':',' '};
+static const unsigned char header_rpp[16]	= { '<', 'R', 'E', 'A', 'P', 'E', 'R', '_', 'P', 'R', 'O', 'J', 'E', 'C', 'T', ' '};
+static const unsigned char header_rtf[5]	= { '{','\\','r','t','f'};
 static const unsigned char header_sh[9]  	= "#!/bin/sh";
 static const unsigned char header_slk[10]  	= "ID;PSCALC3";
-static const unsigned char header_stp[13]  	= "ISO-10303-21;";
-static const unsigned char header_ram[7]	= "rtsp://";
-static const unsigned char header_xml[14]	= "<?xml version=";
-static const unsigned char header_dc[6]		= "SC V10";
-static const unsigned char header_ics[15]	= "BEGIN:VCALENDAR";
-static const unsigned char header_msf[19]	= "// <!-- <mdb:mork:z";
-static const unsigned char header_adr[25]	= "Opera Hotlist version 2.0";
 static const unsigned char header_stl[6]	= "solid ";
+static const unsigned char header_stp[13]  	= "ISO-10303-21;";
+static const unsigned char header_wpl[21]	= { '<', '?', 'w', 'p', 'l', ' ', 'v', 'e', 'r', 's', 'i', 'o', 'n', '=', '"', '1', '.', '0', '"', '?', '>' };
+static const unsigned char header_xml[14]	= "<?xml version=";
 
 static void register_header_check_txt(file_stat_t *file_stat)
 {
@@ -104,23 +113,31 @@ static void register_header_check_txt(file_stat_t *file_stat)
 
 static void register_header_check_fasttxt(file_stat_t *file_stat)
 {
+  register_header_check(0, header_adr, sizeof(header_adr), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_bash,sizeof(header_bash), &header_check_fasttxt, file_stat);
   register_header_check(0, header_cls,sizeof(header_cls), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_cue,sizeof(header_cue), &header_check_fasttxt, file_stat);
+  register_header_check(4, header_dc, sizeof(header_dc), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_dif, sizeof(header_dif), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_ers,sizeof(header_ers), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_ics, sizeof(header_ics), &header_check_fasttxt, file_stat);
   register_header_check(0, header_imm,sizeof(header_imm), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_imm2,sizeof(header_imm2), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_lyx,sizeof(header_lyx), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_m3u, sizeof(header_m3u), &header_check_fasttxt, file_stat);
   register_header_check(0, header_mail,sizeof(header_mail), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_msf, sizeof(header_msf), &header_check_fasttxt, file_stat);
   register_header_check(0, header_perlm,sizeof(header_perlm), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_rtf,sizeof(header_rtf), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_ram,sizeof(header_ram), &header_check_fasttxt, file_stat);
   register_header_check(0, header_reg,sizeof(header_reg), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_ReturnPath,sizeof(header_ReturnPath), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_rpp,sizeof(header_rpp), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_rtf,sizeof(header_rtf), &header_check_fasttxt, file_stat);
   register_header_check(0, header_sh,sizeof(header_sh), &header_check_fasttxt, file_stat);
   register_header_check(0, header_slk,sizeof(header_slk), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_stp,sizeof(header_stp), &header_check_fasttxt, file_stat);
   register_header_check(0, header_stl,sizeof(header_stl), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_ram,sizeof(header_ram), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_stp,sizeof(header_stp), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_wpl,sizeof(header_wpl), &header_check_fasttxt, file_stat);
   register_header_check(0, header_xml,sizeof(header_xml), &header_check_fasttxt, file_stat);
-  register_header_check(4, header_dc, sizeof(header_dc), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_ics, sizeof(header_ics), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_msf, sizeof(header_msf), &header_check_fasttxt, file_stat);
-  register_header_check(0, header_adr, sizeof(header_adr), &header_check_fasttxt, file_stat);
 }
 
 // #define DEBUG_FILETXT
@@ -143,6 +160,7 @@ static int filtre(unsigned char car)
     case 0xa8:
     case 0xa9:	/* '' */
     case 0xab:	/* '' */
+    case 0xae:	/* '' */
     case 0xb0:	/* '' */
     case 0xb4:  /* '' */
     case 0xb7:
@@ -329,7 +347,7 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
   }
   /* Incredimail has .imm extension but this extension isn't frequent */
   if(memcmp(buffer,header_imm,sizeof(header_imm))==0 ||
-      memcmp(buffer,header_imm2,sizeof(header_imm2))==0 ||
+      memcmp(buffer,header_ReturnPath,sizeof(header_ReturnPath))==0 ||
       memcmp(buffer,header_mail,sizeof(header_mail))==0)
   {
     if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
@@ -350,6 +368,15 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="pm";
     return 1;
   }
+  if(memcmp(buffer,header_rpp,sizeof(header_rpp))==0)
+  {
+    /* Reaper Project */
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="rpp";
+    return 1;
+  }
   if(memcmp(buffer,header_rtf,sizeof(header_rtf))==0)
   {
     reset_file_recovery(file_recovery_new);
@@ -366,7 +393,8 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="reg";
     return 1;
   }
-  if(memcmp(buffer,header_sh,sizeof(header_sh))==0)
+  if(memcmp(buffer,header_sh,sizeof(header_sh))==0 ||
+      memcmp(buffer,header_bash,sizeof(header_bash))==0)
   {
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
@@ -393,6 +421,15 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="stl";
     return 1;
   }
+  if(memcmp(buffer, header_ers, sizeof(header_ers))==0)
+  {
+    /* ER Mapper Rasters (ERS) */
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_ers;
+    file_recovery_new->extension="ers";
+    return 1;
+  }
   if(memcmp(buffer,header_stp,sizeof(header_stp))==0)
   {
     /* ISO 10303 is an ISO standard for the computer-interpretable
@@ -406,6 +443,15 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="stp";
     return 1;
   }
+  if(memcmp(buffer,header_wpl,sizeof(header_wpl))==0)
+  {
+    /* Windows Play List*/
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="wpl";
+    return 1;
+  }
   if(memcmp(buffer,header_ram,sizeof(header_ram))==0)
   {
     reset_file_recovery(file_recovery_new);
@@ -445,6 +491,16 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="dc";
     return 1;
   }
+  if(memcmp(buffer,header_dif,sizeof(header_dif))==0)
+  { /*
+       Lotus Data Interchange Format
+     */
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="dif";
+    return 1;
+  }
   if(memcmp(buffer, header_ics, sizeof(header_ics))==0)
   {
     const char *date_asc;
@@ -475,12 +531,30 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
       tm_time.tm_mon=(date_asc[4]-'0')*10+(date_asc[5]-'0')-1;	/* month 0-11 */
       tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
 	(date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;        	/* year */
-      tm_time.tm_isdst=-1;       /* unknown daylight saving time */
+      tm_time.tm_isdst = -1;		/* unknown daylight saving time */
       file_recovery_new->time=mktime(&tm_time);
     }
     free(buffer2);
     return 1;
   }
+  /* Lyx http://www.lyx.org */
+  if(memcmp(buffer, header_lyx, sizeof(header_lyx))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="lyx";
+    return 1;
+  }
+  /* Moving Picture Experts Group Audio Layer 3 Uniform Resource Locator */
+  if(memcmp(buffer, header_m3u, sizeof(header_m3u))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="m3u";
+    return 1;
+  }
   /* Mozilla, firefox, thunderbird msf (Mail Summary File) */
   if(memcmp(buffer, header_msf, sizeof(header_msf))==0)
   {
@@ -499,6 +573,16 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="adr";
     return 1;
   }
+  /* Cue sheet often begins by the music genre
+   * http://wiki.hydrogenaudio.org/index.php?title=Cue_sheet */
+  if(memcmp(buffer, header_cue, sizeof(header_cue))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="cue";
+    return 1;
+  }
   return 0;
 }
 
@@ -506,7 +590,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
 {
   static char *buffer_lower=NULL;
   static unsigned int buffer_lower_size=0;
-  unsigned int i;
+  unsigned int l=0;
   const unsigned char header_asp[22]	= "<%@ language=\"vbscript";
   const unsigned char header_bat[9]  	= "@echo off";
   const unsigned char header_vcf[11]	= "begin:vcard";
@@ -521,12 +605,16 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   const char sign_php[]			= "<?php";
   const char sign_tex[]			= "\\begin{";
   const char sign_html[]		= "<html";
+  const char sign_cue[6]		= { 'F', 'I', 'L', 'E', ' ', '"'};
   const unsigned int buffer_size_test=(buffer_size < 2048 ? buffer_size : 2048);
   {
+    unsigned int i;
     unsigned int tmp=0;
     for(i=0;i<10 && isdigit(buffer[i]);i++)
       tmp=tmp*10+buffer[i]-'0';
-    if(buffer[i]==0x0a && memcmp(buffer+i+1, header_imm2, sizeof(header_imm2))==0 &&
+    if(buffer[i]==0x0a &&
+      (memcmp(buffer+i+1, header_ReturnPath, sizeof(header_ReturnPath))==0 ||
+       memcmp(buffer+i+1, header_ReceivedFrom, sizeof(header_ReceivedFrom))==0) &&
         !(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
           file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
           strcmp(file_recovery->extension,"imm")==0))
@@ -539,6 +627,16 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       return 1;
     }
   }
+  /* Cue sheet can begin by the filename 
+   * http://wiki.hydrogenaudio.org/index.php?title=Cue_sheet */
+  if(memcmp(buffer, sign_cue, sizeof(sign_cue))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="cue";
+    return 1;
+  }
   if(buffer_lower_size<buffer_size_test+16)
   {
     free(buffer_lower);
@@ -550,7 +648,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     buffer_lower_size=buffer_size_test+16;
     buffer_lower=(char *)MALLOC(buffer_lower_size);
   }
-  i=UTF2Lat(buffer_lower,buffer,buffer_size_test);
+  l=UTF2Lat((unsigned char*)buffer_lower, buffer, buffer_size_test);
   /* strncasecmp */
   if(memcmp(buffer_lower,header_bat,sizeof(header_bat))==0)
   {
@@ -578,13 +676,13 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   if(buffer[0]=='#' && buffer[1]=='!')
   {
-    unsigned int l=i-2;
-    const unsigned char *haystack=buffer_lower+2;
+    unsigned int ll=l-2;
+    const unsigned char *haystack=(const unsigned char *)buffer_lower+2;
     const unsigned char *res;
-    res=memchr(haystack,'\n',l);
+    res=(const unsigned char *)memchr(haystack,'\n',ll);
     if(res!=NULL)
-      l=res-haystack;
-    if(td_memmem(haystack, l, header_sig_perl, sizeof(header_sig_perl)) != NULL)
+      ll=res-haystack;
+    if(td_memmem(haystack, ll, header_sig_perl, sizeof(header_sig_perl)) != NULL)
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
@@ -592,7 +690,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       file_recovery_new->extension="pl";
       return 1;
     }
-    if(td_memmem(haystack, l, header_sig_python, sizeof(header_sig_python)) != NULL)
+    if(td_memmem(haystack, ll, header_sig_python, sizeof(header_sig_python)) != NULL)
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
@@ -600,7 +698,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       file_recovery_new->extension="py";
       return 1;
     }
-    if(td_memmem(haystack, l, header_sig_ruby, sizeof(header_sig_ruby)) != NULL)
+    if(td_memmem(haystack, ll, header_sig_ruby, sizeof(header_sig_ruby)) != NULL)
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
@@ -625,31 +723,59 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     /* ind=~0: random
      * ind=~1: constant	*/
     double ind=1;
-    unsigned int nbr=0;
+    unsigned int nbrf=0;
+    unsigned int is_csv=1;
     /* Detect Fortran */
     {
       char *str=buffer_lower;
       while((str=strstr(str, "\n      "))!=NULL)
       {
-	nbr++;
+	nbrf++;
 	str++;
       }
     }
-    if(i>1)
+    /* Detect csv */
+    {
+      unsigned int csv_per_line_current=0;
+      unsigned int csv_per_line=0;
+      unsigned int line_nbr=0;
+      unsigned int i;
+      for(i=0;i<l && is_csv>0;i++)
+      {
+	if(buffer_lower[i]==';')
+	{
+	  csv_per_line_current++;
+	}
+	else if(buffer_lower[i]=='\n')
+	{
+	  if(line_nbr==0)
+	    csv_per_line=csv_per_line_current;
+	  if(csv_per_line_current!=csv_per_line)
+	    is_csv=0;
+	  line_nbr++;
+	  csv_per_line_current=0;
+	}
+      }
+      if(csv_per_line<1 || line_nbr<10)
+	is_csv=0;
+    }
+    if(l>1)
     {
       unsigned int stats[256];
-      unsigned int j;
+      unsigned int i;
       memset(&stats, 0, sizeof(stats));
-      for(j=0;j<i;j++)
-        stats[buffer[j]]++;
+      for(i=0;i<l;i++)
+        stats[(unsigned char)buffer_lower[i]]++;
       ind=0;
-      for(j=0;j<256;j++)
-        if(stats[j]>0)
-          ind+=stats[j]*(stats[j]-1);
-      ind=ind/i/(i-1);
+      for(i=0;i<256;i++)
+        if(stats[i]>0)
+          ind+=stats[i]*(stats[i]-1);
+      ind=ind/l/(l-1);
     }
-    if(nbr>10 && i<=0.90)
+    if(nbrf>10 && ind<0.9)
       ext="f";
+    else if(is_csv>0)
+      ext="csv";
     /* Detect LaTeX, C, PHP, JSP, ASP, HTML, C header */
     else if(strstr(buffer_lower, sign_tex)!=NULL)
       ext="tex";
@@ -665,9 +791,9 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       ext="asp";
     else if(strstr(buffer_lower, sign_html)!=NULL)
       ext="html";
-    else if(strstr(buffer_lower, sign_h)!=NULL && i>50)
+    else if(strstr(buffer_lower, sign_h)!=NULL && l>50)
       ext="h";
-    else if(i<100 || ind<0.03 || ind>0.90)
+    else if(l<100 || ind<0.03 || ind>0.90)
       ext=NULL;
     else
       ext=file_hint_txt.extension;
@@ -694,15 +820,15 @@ Doc: \r (0xD)
       if(file_recovery->file_stat->file_hint==&file_hint_doc &&
           strstr(file_recovery->filename,".doc")!=NULL)
       {
-        unsigned int j;
+        unsigned int i;
         unsigned int txt_nl=0;
-        for(j=0;j<i-1;j++)
-          if(buffer_lower[j]=='\r' && buffer_lower[j+1]!='\n')
+        for(i=0;i<l-1;i++)
+          if(buffer_lower[i]=='\r' && buffer_lower[i+1]!='\n')
           {
             return 0;
           }
-        for(j=0;j<i && j<512;j++)
-          if(buffer[j]=='\n')
+        for(i=0;i<l && i<512;i++)
+          if(buffer_lower[i]=='\n')
             txt_nl=1;
         if(txt_nl==1)
         {
@@ -752,7 +878,7 @@ static int data_check_txt(const unsigned char *buffer, const unsigned int buffer
 {
   unsigned int i;
   char *buffer_lower=(char *)MALLOC(buffer_size+16);
-  i=UTF2Lat(buffer_lower,&buffer[buffer_size/2],buffer_size/2);
+  i=UTF2Lat((unsigned char*)buffer_lower, &buffer[buffer_size/2], buffer_size/2);
   if(i<buffer_size/2)
   {
     const char sign_html_end[]	= "</html>";
@@ -825,3 +951,10 @@ static void file_check_xml(file_recovery_t *file_recovery)
   file_search_footer(file_recovery, xml_footer, sizeof(xml_footer));
   file_allow_nl(file_recovery, NL_BARENL|NL_CRLF|NL_BARECR);
 }
+
+static void file_check_ers(file_recovery_t *file_recovery)
+{
+  const unsigned char ers_footer[17]= "DatasetHeader End";
+  file_search_footer(file_recovery, ers_footer, sizeof(ers_footer));
+  file_allow_nl(file_recovery, NL_BARENL|NL_CRLF|NL_BARECR);
+}
