@@ -2,7 +2,7 @@
 
     File: fnctdsk.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005,2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,49 +34,46 @@
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
-#include "lang.h"
-#include "testdisk.h"
-#include "analyse.h"
 #include "log.h"
+#include "log_part.h"
 #include "guid_cpy.h"
 
 static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, const list_part_t *list_part, const int verbose);
+static list_part_t *element_new(partition_t *part);
 
 unsigned long int C_H_S2LBA(const disk_t *disk_car,const unsigned int C, const unsigned int H, const unsigned int S)
-{ return ((unsigned long int)C*(disk_car->CHS.head+1)+H)*disk_car->CHS.sector+S-1;
+{
+  return ((unsigned long int)C * disk_car->geom.heads_per_cylinder + H) * disk_car->geom.sectors_per_head + S - 1;
 }
 
 uint64_t CHS2offset(const disk_t *disk_car,const CHS_t*CHS)
-{ return (((uint64_t)CHS->cylinder*(disk_car->CHS.head+1)+CHS->head)*disk_car->CHS.sector+CHS->sector-1)*disk_car->sector_size;
-}
-
-uint64_t C_H_S2offset(const disk_t *disk_car,const unsigned int C, const unsigned int H, const unsigned int S)
-{ return (((uint64_t)C*(disk_car->CHS.head+1)+H)*disk_car->CHS.sector+S-1)*disk_car->sector_size;
+{
+  return (((uint64_t)CHS->cylinder * disk_car->geom.heads_per_cylinder + CHS->head) *
+      disk_car->geom.sectors_per_head + CHS->sector - 1) * disk_car->sector_size;
 }
 
 unsigned int offset2sector(const disk_t *disk_car, const uint64_t offset)
-{ return ((offset/disk_car->sector_size)%disk_car->CHS.sector)+1; }
+{
+  return ((offset / disk_car->sector_size) % disk_car->geom.sectors_per_head) + 1;
+}
 
 unsigned int offset2head(const disk_t *disk_car, const uint64_t offset)
-{ return ((offset/disk_car->sector_size)/disk_car->CHS.sector)%(disk_car->CHS.head+1); }
+{
+  return ((offset / disk_car->sector_size) / disk_car->geom.sectors_per_head) % disk_car->geom.heads_per_cylinder;
+}
 
 unsigned int offset2cylinder(const disk_t *disk_car, const uint64_t offset)
-{ return ((offset/disk_car->sector_size)/disk_car->CHS.sector)/(disk_car->CHS.head+1); }
+{
+  return ((offset / disk_car->sector_size) / disk_car->geom.sectors_per_head) / disk_car->geom.heads_per_cylinder;
+}
 
 void offset2CHS(const disk_t *disk_car,const uint64_t offset, CHS_t*CHS)
 {
   uint64_t pos=offset/disk_car->sector_size;
-  CHS->sector=(pos%disk_car->CHS.sector)+1;
-  pos/=disk_car->CHS.sector;
-  CHS->head=pos%(disk_car->CHS.head+1);
-  CHS->cylinder=pos/(disk_car->CHS.head+1);
-}
-
-void dup_CHS(CHS_t * CHS_dst, const CHS_t * CHS_source)
-{
-  CHS_dst->cylinder=CHS_source->cylinder;
-  CHS_dst->head=CHS_source->head;
-  CHS_dst->sector=CHS_source->sector;
+  CHS->sector=(pos%disk_car->geom.sectors_per_head)+1;
+  pos/=disk_car->geom.sectors_per_head;
+  CHS->head=pos%disk_car->geom.heads_per_cylinder;
+  CHS->cylinder=pos/disk_car->geom.heads_per_cylinder;
 }
 
 void dup_partition_t(partition_t *dst, const partition_t *src)
@@ -209,42 +206,11 @@ int delete_list_disk(list_disk_t *list_disk)
   return write_used;
 }
 
-int check_list_part(list_part_t *list_part)
-{
-  list_part_t *prev=NULL;
-  list_part_t *parts;
-  if((list_part!=NULL) && (list_part->prev!=NULL))
-  {
-    log_critical("\ncheck_list_part error: list_part->prev!=NULL\n");
-    exit(EXIT_FAILURE);
-  }
-  log_trace("check_list_part\n");
-  for(parts=list_part;parts!=NULL;parts=parts->next)
-  {
-    log_info("%p %p %p\n",parts->prev, parts, parts->next);
-    if(prev!=parts->prev)
-    {
-      log_critical("\ncheck_list_part error: prev!=parts->prev\n");
-      exit(EXIT_FAILURE);
-    }
-    prev=parts;
-  }
-  if((prev!=NULL) && (prev->next!=NULL))
-  {
-    log_critical("\ncheck_list_part error: prev->next!=NULL\n");
-    exit(EXIT_FAILURE);
-  }
-  return 0;
-}
-
 list_part_t *sort_partition_list(list_part_t *list_part)
 {
   list_part_t *new_list_part=NULL;
   list_part_t *element;
   list_part_t *next;
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
   for(element=list_part;element!=NULL;element=next)
   {
     int insert_error=0;
@@ -254,9 +220,6 @@ list_part_t *sort_partition_list(list_part_t *list_part)
       free(element->part);
     free(element);
   }
-#ifdef DEBUG
-  check_list_part(new_list_part);
-#endif
   return new_list_part;
 }
 
@@ -277,9 +240,6 @@ list_part_t *gen_sorted_partition_list(const list_part_t *list_part)
 void part_free_list(list_part_t *list_part)
 {
   list_part_t *element;
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
   element=list_part;
   while(element!=NULL)
   {
@@ -294,9 +254,6 @@ void part_free_list(list_part_t *list_part)
 void part_free_list_only(list_part_t *list_part)
 {
   list_part_t *element;
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
   element=list_part;
   while(element!=NULL)
   {
@@ -330,7 +287,6 @@ int is_part_overlapping(const list_part_t *list_part)
       return 1;
     element=next;
   }
-  return 0;
 }
 
 void  partition_reset(partition_t *partition, const arch_fnct_t *arch)
@@ -364,7 +320,7 @@ partition_t *partition_new(const arch_fnct_t *arch)
   return partition;
 }
 
-list_part_t *element_new(partition_t *part)
+static list_part_t *element_new(partition_t *part)
 {
   list_part_t *new_element=(list_part_t*)MALLOC(sizeof(*new_element));
   new_element->part=part;
@@ -386,7 +342,7 @@ static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, cons
     if(start.sector==1 && start.head<=1)
     {
       nbr++;
-      if(end.head==disk_car->CHS.head)
+      if(end.head==disk_car->geom.heads_per_cylinder-1)
       {
 	nbr++;
 	/* Doesn't check if end.sector==disk_car->CHS.sector */
@@ -395,7 +351,8 @@ static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, cons
   }
   if(nbr>0)
   {
-    log_info("get_geometry_from_list_part_aux head=%u nbr=%u\n",disk_car->CHS.head+1,nbr);
+    log_info("get_geometry_from_list_part_aux head=%u nbr=%u\n",
+	disk_car->geom.heads_per_cylinder, nbr);
     if(verbose>1)
     {
       for(element=list_part;element!=NULL;element=element->next)
@@ -404,7 +361,7 @@ static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, cons
 	CHS_t end;
 	offset2CHS(disk_car,element->part->part_offset,&start);
 	offset2CHS(disk_car,element->part->part_offset+element->part->part_size-1,&end);
-	if(start.sector==1 && start.head<=1 && end.head==disk_car->CHS.head)
+	if(start.sector==1 && start.head<=1 && end.head==disk_car->geom.heads_per_cylinder-1)
 	{
 	  log_partition(disk_car,element->part);
 	}
@@ -417,25 +374,25 @@ static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, cons
 unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part_t *list_part, const int verbose)
 {
   const unsigned int head_list[]={8,16,32,64,128,240,255,0};
-  unsigned int nbr_max;
-  unsigned int nbr;
-  unsigned int h_index=0;
-  unsigned int head_max=disk_car->CHS.head;
-  disk_t *new_disk_car=MALLOC(sizeof(*new_disk_car));
+  unsigned int best_score;
+  unsigned int score;
+  unsigned int i;
+  unsigned int heads_per_cylinder=disk_car->geom.heads_per_cylinder;
+  disk_t *new_disk_car=(disk_t *)MALLOC(sizeof(*new_disk_car));
   memcpy(new_disk_car,disk_car,sizeof(*new_disk_car));
-  nbr_max=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
-  for(h_index=0;head_list[h_index]!=0;h_index++)
+  best_score=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
+  for(i=0; head_list[i]!=0; i++)
   {
-    new_disk_car->CHS.head=head_list[h_index]-1;
-    nbr=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
-    if(nbr>=nbr_max)
+    new_disk_car->geom.heads_per_cylinder=head_list[i];
+    score=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
+    if(score >= best_score)
     {
-      nbr_max=nbr;
-      head_max=new_disk_car->CHS.head;
+      best_score=score;
+      heads_per_cylinder=new_disk_car->geom.heads_per_cylinder;
     }
   }
   free(new_disk_car);
-  return head_max;
+  return heads_per_cylinder;
 }
 
 const char *size_to_unit(uint64_t disk_size, char *buffer)
