@@ -2,7 +2,7 @@
 
     File: godmode.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,30 +23,30 @@
 #include <config.h>
 #endif
  
-#include <ctype.h>      /* tolower */
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
 #include "analyse.h"
 #include "lang.h"
 #include "godmode.h"
-#include "testdisk.h"
-#include "fat.h"
+#include "intrface.h"
 #include "ext2.h"
 #include "intrf.h"
 #include "intrfn.h"
-#include "intrface.h"
 #include "md.h"
-#include "adv.h"
 #include "ntfs.h"
 #include "next.h"
+#include "tpartwr.h"
 #include "log.h"
+#include "log_part.h"
+#include "fat32.h"
+#include "tntfs.h"
+#include "thfs.h"
+#include "partmacn.h"
 
 #define RO 1
 #define RW 0
@@ -65,6 +65,9 @@ static int interface_part_bad_log(disk_t *disk_car,list_part_t *list_part_bad);
 static int interface_part_bad_ncurses(disk_t *disk_car, list_part_t *list_part_bad);
 static void warning_geometry_ncurses(disk_t *disk_car, const unsigned int recommanded_heads_per_cylinder);
 static void ask_mbr_order_i386(disk_t *disk_car,list_part_t *list_part);
+#define ANALYSE_X	0
+#define ANALYSE_Y	5
+#define INTER_BAD_PART	10
 #endif
 static list_part_t *add_ext_part_i386(disk_t *disk_car, list_part_t *list_part, const int max_ext, const int align,const int verbose);
 static unsigned int tab_insert(uint64_t *tab, const uint64_t offset, unsigned int tab_nbr);
@@ -76,14 +79,15 @@ static inline void offset2CHS_inline(const disk_t *disk_car,const uint64_t offse
 static inline void offset2CHS_inline(const disk_t *disk_car,const uint64_t offset, CHS_t*CHS)
 {
   uint64_t pos=offset/disk_car->sector_size;
-  CHS->sector=(pos%disk_car->CHS.sector)+1;
-  pos/=disk_car->CHS.sector;
-  CHS->head=pos%(disk_car->CHS.head+1);
-  CHS->cylinder=pos/(disk_car->CHS.head+1);
+  CHS->sector=(pos%disk_car->geom.sectors_per_head)+1;
+  pos/=disk_car->geom.sectors_per_head;
+  CHS->head=pos%disk_car->geom.heads_per_cylinder;
+  CHS->cylinder=pos/disk_car->geom.heads_per_cylinder;
 }
 
 static inline uint64_t CHS2offset_inline(const disk_t *disk_car,const CHS_t*CHS)
-{ return (((uint64_t)CHS->cylinder*(disk_car->CHS.head+1)+CHS->head)*disk_car->CHS.sector+CHS->sector-1)*disk_car->sector_size;
+{
+  return (((uint64_t)CHS->cylinder*disk_car->geom.heads_per_cylinder+CHS->head)*disk_car->geom.sectors_per_head+CHS->sector-1)*disk_car->sector_size;
 }
 /* Optimization end */
 
@@ -174,7 +178,7 @@ static int interface_part_bad_ncurses(disk_t *disk_car, list_part_t *list_part)
 	wattrset(stdscr, A_REVERSE);
 	aff_part(stdscr, AFF_PART_BASE, disk_car, parts->part);
 	wattroff(stdscr, A_REVERSE);
-	wmove(stdscr,24,0);
+	wmove(stdscr,23,0);
 	wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
 	if(parts->part->info[0]!='\0')
 	{
@@ -201,56 +205,40 @@ static int interface_part_bad_ncurses(disk_t *disk_car, list_part_t *list_part)
 	quit=1;
 	break;
       case KEY_UP:
-	if(list_part!=NULL)
+	if(pos->prev!=NULL)
 	{
-	  if(pos->prev!=NULL)
-	  {
-	    pos=pos->prev;
-	    pos_num--;
-	  }
-	  if(pos_num<offset)
-	    offset--;
+	  pos=pos->prev;
+	  pos_num--;
 	}
 	break;
       case KEY_DOWN:
-	if(list_part!=NULL)
+	if(pos->next!=NULL)
 	{
-	  if(pos->next!=NULL)
-	  {
-	    pos=pos->next;
-	    pos_num++;
-	  }
-	  if(pos_num>=offset+INTER_BAD_PART)
-	    offset++;
+	  pos=pos->next;
+	  pos_num++;
 	}
 	break;
       case KEY_PPAGE:
-	if(list_part!=NULL)
+	for(i=0; i<INTER_BAD_PART && pos->prev!=NULL; i++)
 	{
-	  for(i=0;(i<INTER_BAD_PART) && (pos->prev!=NULL);i++)
-	  {
-	    pos=pos->prev;
-	    pos_num--;
-	    if(pos_num<offset)
-	      offset--;
-	  }
+	  pos=pos->prev;
+	  pos_num--;
 	}
 	break;
       case KEY_NPAGE:
-	if(list_part!=NULL)
+	for(i=0; i<INTER_BAD_PART && pos->next!=NULL; i++)
 	{
-	  for(i=0;(i<INTER_BAD_PART) && (pos->next!=NULL);i++)
-	  {
-	    pos=pos->next;
-	    pos_num++;
-	    if(pos_num>=offset+INTER_BAD_PART)
-	      offset++;
-	  }
+	  pos=pos->next;
+	  pos_num++;
 	}
 	break;
       default:
 	break;
     }
+    if(pos_num<offset)
+      offset=pos_num;
+    if(pos_num>=offset+INTER_BAD_PART)
+      offset=pos_num-INTER_BAD_PART+1;
   } while(quit==0);
   return 0;
 }
@@ -304,7 +292,8 @@ static void warning_geometry_ncurses(disk_t *disk_car, const unsigned int recomm
   wmove(stdscr,4,0);
   wprintw(stdscr,"%s",disk_car->description(disk_car));
   wmove(stdscr,6,0);
-  wprintw(stdscr,"Warning: the current number of heads per cylinder is %u",disk_car->CHS.head+1);
+  wprintw(stdscr, "Warning: the current number of heads per cylinder is %u",
+      disk_car->geom.heads_per_cylinder);
   wmove(stdscr,7,0);
   wprintw(stdscr,"but the correct value may be %u.",recommanded_heads_per_cylinder);
   wmove(stdscr,8,0);
@@ -379,18 +368,21 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
   uint64_t search_location;
   unsigned int try_offset_nbr=0;
   unsigned int try_offset_raid_nbr=0;
+#ifdef HAVE_NCURSES
   unsigned int old_cylinder=0;
+#endif
   unsigned int location_boundary;
   int ind_stop=0;
   list_part_t *list_part=NULL;
   list_part_t *list_part_bad=NULL;
   partition_t *partition;
   /* It's not a problem to read a little bit more than necessary */
-  const uint64_t search_location_max=td_max(disk_car->disk_size +
-      (disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size,
+  const uint64_t search_location_max=td_max((disk_car->disk_size /
+      ((uint64_t) disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size) + 1 ) *
+      ((uint64_t) disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size),
       disk_car->disk_real_size);
   const uint64_t max_disk_size_for_partition=td_max(disk_car->disk_size,
-      (uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size);
+      (uint64_t)disk_car->geom.cylinders*disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size);
   partition=partition_new(disk_car->arch);
   buffer_disk=(unsigned char*)MALLOC(16*DEFAULT_SECTOR_SIZE);
   {
@@ -428,21 +420,24 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
     /* sometimes users don't choose Vista by mistake */
     try_offset_nbr=tab_insert(try_offset, 2048*512, try_offset_nbr);
     /* try to deal with incorrect geometry */
+    /* 0/1/1 */
+    try_offset_nbr=tab_insert(try_offset, 32 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 63 * disk_car->sector_size, try_offset_nbr);
     /* 1/[01]/1 CHS x  16 63 */
     try_offset_nbr=tab_insert(try_offset, 16 * 63 * disk_car->sector_size, try_offset_nbr);
     try_offset_nbr=tab_insert(try_offset, 17 * 63 * disk_car->sector_size, try_offset_nbr);
-    try_offset_nbr=tab_insert(try_offset, 16 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
-    try_offset_nbr=tab_insert(try_offset, 17 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 16 * disk_car->geom.sectors_per_head * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 17 * disk_car->geom.sectors_per_head * disk_car->sector_size, try_offset_nbr);
     /* 1/[01]/1 CHS x 240 63 */
     try_offset_nbr=tab_insert(try_offset, 240 * 63 * disk_car->sector_size, try_offset_nbr);
     try_offset_nbr=tab_insert(try_offset, 241 * 63 * disk_car->sector_size, try_offset_nbr);
-    try_offset_nbr=tab_insert(try_offset, 240 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
-    try_offset_nbr=tab_insert(try_offset, 241 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 240 * disk_car->geom.sectors_per_head * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 241 * disk_car->geom.sectors_per_head * disk_car->sector_size, try_offset_nbr);
     /* 1/[01]/1 CHS x 255 63 */
     try_offset_nbr=tab_insert(try_offset, 255 * 63 * disk_car->sector_size, try_offset_nbr);
     try_offset_nbr=tab_insert(try_offset, 256 * 63 * disk_car->sector_size, try_offset_nbr);
-    try_offset_nbr=tab_insert(try_offset, 255 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
-    try_offset_nbr=tab_insert(try_offset, 256 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 255 * disk_car->geom.sectors_per_head * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 256 * disk_car->geom.sectors_per_head * disk_car->sector_size, try_offset_nbr);
   }
   else if(disk_car->arch==&arch_mac)
   {
@@ -453,8 +448,8 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
   }
   else if(disk_car->arch==&arch_sun)
   {
-    min_location=(disk_car->CHS.head+1) * disk_car->CHS.sector * disk_car->sector_size;
-    location_boundary=(disk_car->CHS.head+1) * disk_car->CHS.sector * disk_car->sector_size;
+    min_location=disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
+    location_boundary=disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
   }
   else if(disk_car->arch==&arch_xbox)
   {
@@ -476,12 +471,14 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
     static CHS_t start;
     offset2CHS_inline(disk_car,search_location,&start);
 #ifdef HAVE_NCURSES
-    if(old_cylinder!=start.cylinder && interface!=0 && (disk_car->CHS.head!=0 || (start.cylinder & 0xFFF)==0))
+    if(old_cylinder!=start.cylinder && interface!=0 && (disk_car->geom.heads_per_cylinder>0 || (start.cylinder & 0xFFF)==0))
     {
       old_cylinder=start.cylinder;
       wmove(stdscr,ANALYSE_Y,ANALYSE_X);
       wclrtoeol(stdscr);
-      wprintw(stdscr,"Analyse cylinder %5u/%u: %02u%%",start.cylinder,disk_car->CHS.cylinder,(unsigned int)((uint64_t)start.cylinder*100/(disk_car->CHS.cylinder+1)));
+      wprintw(stdscr,"Analyse cylinder %5u/%u: %02u%%",
+	  start.cylinder, disk_car->geom.cylinders-1,
+	  (unsigned int)((uint64_t)start.cylinder*100/disk_car->geom.cylinders));
       wrefresh(stdscr);
       ind_stop|=check_enter_key_or_s(stdscr);
     }
@@ -531,19 +528,29 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
         {
           if(search_now_raid>0 || fast_mode>1)
           { /* Search Linux software RAID */
-            if(disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer_disk, search_location)!=0)
+            if(disk_car->pread(disk_car, buffer_disk, 8 * DEFAULT_SECTOR_SIZE, search_location) != 8 * DEFAULT_SECTOR_SIZE)
             {
-              res=-1;
+              res = -1;
             }
             else
             {
               if(recover_MD(disk_car,(const struct mdp_superblock_s*)buffer_disk,partition,verbose,dump_ind)==0)
               {
                 const struct mdp_superblock_1 *sb1=(const struct mdp_superblock_1 *)buffer_disk;
-                if(le32(sb1->major_version)==0)
-                  partition->part_offset-=(uint64_t)MD_NEW_SIZE_SECTORS(partition->part_size/512)*512;
-                else
-                  partition->part_offset-=le64(sb1->super_offset)*512;
+		if(le32(sb1->md_magic)==(unsigned int)MD_SB_MAGIC)
+		{
+		  if(le32(sb1->major_version)==0)
+		    partition->part_offset-=(uint64_t)MD_NEW_SIZE_SECTORS(partition->part_size/512)*512;
+		  else
+		    partition->part_offset-=le64(sb1->super_offset)*512;
+		}
+		else
+		{
+		  if(be32(sb1->major_version)==0)
+		    partition->part_offset-=(uint64_t)MD_NEW_SIZE_SECTORS(partition->part_size/512)*512;
+		  else
+		    partition->part_offset-=be64(sb1->super_offset)*512;
+		}
                 res=1;
               }
               else
@@ -555,7 +562,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
         if(res<=0 && test_nbr==2)
         {
           if(fast_mode==0)
-            test_nbr=6;
+            test_nbr=7;
           else
           {
             if((disk_car->arch==&arch_i386 &&
@@ -567,26 +574,42 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
           }
         }
         if(res<=0 && test_nbr==3)
+        {
+          if(fast_mode==0)
+            test_nbr=7;
+          else
+          {
+            if((disk_car->arch==&arch_i386 &&
+                  ((start.sector==13 && (start.head<=2 || fast_mode>1)) ||
+                   (search_vista_part>0 && search_location%(2048*512)==(13-1)*512))) ||
+                (disk_car->arch!=&arch_i386 && (search_location%location_boundary==(13-1)*512)))
+              res=search_EXFAT_backup(buffer_disk, disk_car, partition);
+            test_nbr++;
+          }
+        }
+        if(res<=0 && test_nbr==4)
         {
           if((disk_car->arch==&arch_i386 &&
-                ((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1)) ||
+                ((start.sector==disk_car->geom.sectors_per_head &&
+		  (start.head==disk_car->geom.heads_per_cylinder-1 || fast_mode>1)) ||
                  (search_vista_part>0 && search_location%(2048*512)==(2048-1)*512))) ||
               (disk_car->arch!=&arch_i386 && search_location%location_boundary==(location_boundary-512) &&
                search_location>0))
             res=search_NTFS_backup(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(res<=0 && test_nbr==4)
+        if(res<=0 && test_nbr==5)
         {
           if((disk_car->arch==&arch_i386 &&
-                ((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1)) ||
+                ((start.sector==disk_car->geom.sectors_per_head &&
+		  (start.head==disk_car->geom.heads_per_cylinder-1 || fast_mode>1)) ||
                  (search_vista_part>0 && search_location%(2048*512)==(2048-1)*512))) ||
               (disk_car->arch!=&arch_i386 && search_location%location_boundary==(location_boundary-512) &&
                search_location>0))
             res=search_HFS_backup(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(res<=0 && test_nbr==5)
+        if(res<=0 && test_nbr==6)
         {
           int s_log_block_size;
           /* try backup superblock */
@@ -606,7 +629,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
               if((disk_car->arch==&arch_i386 && start_ext2.sector==1 &&  (start_ext2.head<=2 || fast_mode>1)) ||
                   (disk_car->arch!=&arch_i386 && search_location%location_boundary==0))
               {
-                if(disk_car->read(disk_car,1024, buffer_disk, search_location)==0)
+                if(disk_car->pread(disk_car, buffer_disk, 1024, search_location) == 1024)
                 {
                   const struct ext2_super_block *sb=(const struct ext2_super_block*)buffer_disk;
                   if(le16(sb->s_block_group_nr)>0)
@@ -617,14 +640,14 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
                 }
                 else
                 {
-                  res=-1;
+                  res = -1;
                 }
               }
             }
           }
           test_nbr++;
         }
-        if(res<=0 && test_nbr==6)
+        if(res<=0 && test_nbr==7)
         {
           if(search_now>0)
           {
@@ -632,38 +655,39 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
             test_nbr++;
           }
           else
-            test_nbr=12;
+            test_nbr=13;
         }
-        if(res<=0 && test_nbr==7)
+        if(res<=0 && test_nbr==8)
         {
           res=search_type_0(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(res<=0 && test_nbr==8)
+        if(res<=0 && test_nbr==9)
         {
           res=search_type_8(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(res<=0 && test_nbr==9)
+        if(res<=0 && test_nbr==10)
         {
           /* Try to catch disklabel before BSD FFS partition */
           res=search_type_16(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(res<=0 && test_nbr==10)
+        if(res<=0 && test_nbr==11)
         {
           res=search_type_64(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(res<=0 && test_nbr==11)
+        if(res<=0 && test_nbr==12)
         {
           /* read to fill the cache */
-          disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer_disk, partition->part_offset+(63+16)*512);
+          disk_car->pread(disk_car, buffer_disk, 8 * DEFAULT_SECTOR_SIZE,
+	      partition->part_offset + (63 + 16) * 512);
           /* Try to catch disklabel before BSD FFS partition */
           res=search_type_128(buffer_disk,disk_car,partition,verbose,dump_ind);
           test_nbr++;
         }
-        if(test_nbr==12)
+        if(test_nbr>=13)
         {
           sector_inc=1;
           test_nbr=0;
@@ -678,6 +702,9 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
             wprintw(stdscr,msg_READ_ERROR_AT, start.cylinder,start.head,start.sector,(unsigned long)(partition->part_offset/disk_car->sector_size));
           }
 #endif
+	  /* Stop reading after the end of the disk */
+	  if(search_location >= disk_car->disk_real_size)
+	    search_location = search_location_max;
         }
         else if(res>0)
         {
@@ -686,7 +713,9 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
           aff_part_buffer(AFF_PART_BASE, disk_car,partition);
           if(interface)
           {
+#ifdef HAVE_NCURSES
 	    screen_buffer_to_interface();
+#endif
           }
           if(disk_car->arch->is_part_known(partition)!=0 &&
               partition->part_size>1 &&
@@ -698,20 +727,24 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
             { /* Detect Linux md 0.9 software raid */
               unsigned int disk_factor;
               unsigned int help_factor;
+#if 0
               unsigned int help_factor_max;
+#endif
               const int align=2;
               CHS_t end;
               offset2CHS_inline(disk_car,partition->part_offset+partition->part_size-1,&end);
               if(align>0)
               {
-                end.sector=disk_car->CHS.sector;
+                end.sector=disk_car->geom.sectors_per_head;
                 if(align>1)
-                  end.head=disk_car->CHS.head;
+                  end.head=disk_car->geom.heads_per_cylinder-1;
               }
+#if 0
               help_factor_max=((uint64_t)CHS2offset_inline(disk_car, &end)-partition->part_offset+disk_car->sector_size-partition->part_size)/MD_RESERVED_BYTES;
               if(help_factor_max<3)
                 help_factor_max=3;
               help_factor_max+=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES;
+#endif
               for(disk_factor=6;disk_factor>=1 && ind_stop==0;disk_factor--)
               { /* disk_factor=1, detect Raid 0/1 */
                 /* disk_factor>1, detect Raid 5 */
@@ -736,7 +769,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
               }
               {
                 uint64_t next_part_offset=partition->part_offset+partition->part_size-1+1;
-                uint64_t head_size=disk_car->CHS.sector*disk_car->sector_size;
+                uint64_t head_size=disk_car->geom.sectors_per_head * disk_car->sector_size;
                 try_offset_nbr=tab_insert(try_offset,next_part_offset,try_offset_nbr);
                 try_offset_nbr=tab_insert(try_offset,next_part_offset+head_size,try_offset_nbr);
                 if(next_part_offset%head_size!=0)
@@ -810,7 +843,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
         {
           partition->part_size=(uint64_t)0;
           partition->part_offset=element->part->part_offset - i * disk_car->sector_size;
-          if(disk_car->read(disk_car, DEFAULT_SECTOR_SIZE, buffer_disk, partition->part_offset)==0)
+          if(disk_car->pread(disk_car, buffer_disk, DEFAULT_SECTOR_SIZE, partition->part_offset) == DEFAULT_SECTOR_SIZE)
           {
             if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)buffer_disk,partition,verbose,dump_ind,0)==0)
             {
@@ -1062,7 +1095,7 @@ static list_part_t *add_ext_part_i386(disk_t *disk_car, list_part_t *list_part,
 	offset2CHS_inline(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
 	start.sector=1;
 	start.head++;
-	if(start.head>=disk_car->CHS.head)
+	if(start.head >= disk_car->geom.heads_per_cylinder)
 	{
 	  start.cylinder++;
 	  start.head=0;
@@ -1075,25 +1108,25 @@ static list_part_t *add_ext_part_i386(disk_t *disk_car, list_part_t *list_part,
     }
     if(fin->next==NULL)
     {
-      end.cylinder=disk_car->CHS.cylinder;
-      end.head=disk_car->CHS.head;
-      end.sector=disk_car->CHS.sector;
+      end.cylinder=disk_car->geom.cylinders-1;
+      end.head=disk_car->geom.heads_per_cylinder-1;
+      end.sector=disk_car->geom.sectors_per_head;
     }
     else
     {
       end.cylinder=offset2cylinder(disk_car,fin->next->part->part_offset)-1; /* 8 october 2002 */
-      end.head=disk_car->CHS.head;
-      end.sector=disk_car->CHS.sector;
+      end.head=disk_car->geom.heads_per_cylinder-1;
+      end.sector=disk_car->geom.sectors_per_head;
       if(CHS2offset_inline(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
       {
 	offset2CHS_inline(disk_car,fin->next->part->part_offset-1,&end);
-	end.sector=disk_car->CHS.sector;
+	end.sector=disk_car->geom.sectors_per_head;
 	if(end.head>0)
 	  end.head--;
 	else
 	{
 	  end.cylinder--;
-	  end.head=disk_car->CHS.head;
+	  end.head=disk_car->geom.heads_per_cylinder-1;
 	}
 	if(CHS2offset_inline(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
 	{
@@ -1120,12 +1153,12 @@ static list_part_t *add_ext_part_i386(disk_t *disk_car, list_part_t *list_part,
       }
     }
     offset2CHS_inline(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
-    end.head=disk_car->CHS.head;
-    end.sector=disk_car->CHS.sector;
+    end.head=disk_car->geom.heads_per_cylinder-1;
+    end.sector=disk_car->geom.sectors_per_head;
     if(fin->next && CHS2offset_inline(disk_car,&end)>=fin->next->part->part_offset)
     {
       offset2CHS_inline(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
-      end.sector=disk_car->CHS.sector;
+      end.sector=disk_car->geom.sectors_per_head;
     }
     if(fin->next && CHS2offset_inline(disk_car,&end)>=fin->next->part->part_offset)
     {
@@ -1161,12 +1194,12 @@ static int use_backup(disk_t *disk_car, const list_part_t *list_part, const int
 	  fat32_boot_sector(disk_car, element->part, verbose, dump_ind, expert,current_cmd);
 	  break;
 	case UP_NTFS:
-	  ntfs_boot_sector(disk_car, element->part, verbose, dump_ind, expert, current_cmd);
+	  ntfs_boot_sector(disk_car, element->part, verbose, expert, current_cmd);
 	  break;
 	case UP_HFS:
 	case UP_HFSP:
 	case UP_HFSX:
-	  HFS_HFSP_boot_sector(disk_car, element->part, verbose, dump_ind, expert, current_cmd);
+	  HFS_HFSP_boot_sector(disk_car, element->part, verbose, current_cmd);
 	  break;
 	default:
 	  log_warning("Need to fix\n");
@@ -1199,16 +1232,16 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
     list_part=search_part(disk_car, list_part_org, verbose, dump_ind, fast_mode, 1, search_vista_part, current_cmd);
     if(list_part!=NULL && (disk_car->arch==&arch_i386 || disk_car->arch==&arch_sun))
     { /* Correct disk geometry is necessary for successfull Intel and Sun partition recovery */
-      unsigned int head_max;
-      head_max=get_geometry_from_list_part(disk_car, list_part, verbose);
-      if(disk_car->CHS.head!=head_max)
+      unsigned int heads_per_cylinder;
+      heads_per_cylinder=get_geometry_from_list_part(disk_car, list_part, verbose);
+      if(disk_car->geom.heads_per_cylinder!=heads_per_cylinder)
       {
 	log_warning("Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",
-	    disk_car->CHS.head+1, head_max+1);
+	    disk_car->geom.heads_per_cylinder, heads_per_cylinder);
 #ifdef HAVE_NCURSES
 	if(*current_cmd==NULL)
 	{
-	  warning_geometry_ncurses(disk_car, head_max+1);
+	  warning_geometry_ncurses(disk_car, heads_per_cylinder);
 	}
 #endif
       }
@@ -1235,9 +1268,9 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 	  if(align==0)
 	    location_boundary=disk_car->sector_size;
 	  else if(align==1)
-	    location_boundary=disk_car->CHS.sector * disk_car->sector_size;
+	    location_boundary=disk_car->geom.sectors_per_head * disk_car->sector_size;
 	  else
-	    location_boundary=(disk_car->CHS.head+1) * disk_car->CHS.sector * disk_car->sector_size;
+	    location_boundary=disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
 	}
       }
       else if(disk_car->arch==&arch_mac)
@@ -1246,7 +1279,7 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
       }
       else if(disk_car->arch==&arch_sun)
       {
-	location_boundary=(disk_car->CHS.head+1) * disk_car->CHS.sector * disk_car->sector_size;
+	location_boundary=disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
       }
       else
       {	/* arch_none, arch_xbox, arch_gpt */
@@ -1313,9 +1346,31 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 	switch(res_interface_write)
 	{
 	  case 'W':
-	    if(disk_car->arch->write_part!=NULL)
+	    if(disk_car->arch == &arch_mac)
 	    {
-	      if(no_confirm!=0 || ask_confirmation("Write partition table, confirm ? (Y/N)")!=0)
+#ifdef HAVE_NCURSES
+	      write_part_mac_warning_ncurses();
+#endif
+	    }
+	    else if(disk_car->arch == &arch_sun)
+	    {
+#ifdef HAVE_NCURSES
+	      not_implemented("write_part_sun");
+#endif
+	    }
+	    else if(disk_car->arch == &arch_xbox)
+	    {
+#ifdef HAVE_NCURSES
+	      not_implemented("write_part_xbox");
+#endif
+	    }
+	    else if(disk_car->arch->write_part!=NULL)
+	    {
+	      if(no_confirm!=0
+#ifdef HAVE_NCURSES
+		  || ask_confirmation("Write partition table, confirm ? (Y/N)")!=0
+#endif
+		)
 	      {
 		log_info("write!\n");
 		if(disk_car->arch->write_part(disk_car,list_part,RW,verbose,align))
