@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -35,12 +36,12 @@
 #include "fnctdsk.h"
 #include "log.h"
 
-static int set_HFSP_info(partition_t *partition, const struct hfsp_vh *vh);
+static int set_HFSP_info(partition_t *partition);
 
 int check_HFSP(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(HFSP_BOOT_SECTOR_SIZE);
-  if(disk_car->read(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer, partition->part_offset+0x400)!=0)
+  if(disk_car->pread(disk_car, buffer, HFSP_BOOT_SECTOR_SIZE, partition->part_offset + 0x400) != HFSP_BOOT_SECTOR_SIZE)
   {
     free(buffer);
     return 1;
@@ -50,7 +51,7 @@ int check_HFSP(disk_t *disk_car,partition_t *partition,const int verbose)
     free(buffer);
     return 1;
   }
-  set_HFSP_info(partition,(struct hfsp_vh *)buffer);
+  set_HFSP_info(partition);
   free(buffer);
   return 0;
 }
@@ -72,7 +73,7 @@ int recover_HFSP(disk_t *disk_car, const struct hfsp_vh *vh,partition_t *partiti
     partition->part_offset-=partition->sb_offset;
   }
   partition->part_size=part_size;
-  set_HFSP_info(partition,vh);
+  set_HFSP_info(partition);
   partition->part_type_i386=P_HFSP;
   partition->part_type_mac=PMAC_HFS;
   partition->part_type_gpt=GPT_ENT_TYPE_MAC_HFS;
@@ -85,50 +86,44 @@ int recover_HFSP(disk_t *disk_car, const struct hfsp_vh *vh,partition_t *partiti
 
 int test_HFSP(disk_t *disk_car, const struct hfsp_vh *vh,partition_t *partition,const int verbose, const int dump_ind)
 {
-  if (vh->signature==be16(HFSP_VOLHEAD_SIG) && be32(vh->blocksize)%512==0 && be32(vh->blocksize)!=0 && be32(vh->free_blocks)<=be32(vh->total_blocks))
+  if (!(be32(vh->blocksize)%512==0 && be32(vh->blocksize)!=0 && be32(vh->free_blocks)<=be32(vh->total_blocks)))
+    return 1;
+  /* http://developer.apple.com/technotes/tn/tn1150.html */
+  if (be16(vh->version)==4 && vh->signature==be16(HFSP_VOLHEAD_SIG))
   {
+    partition->upart_type=UP_HFSP;
     if(verbose>0 || dump_ind!=0)
     {
       log_info("\nHFS+ magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
     }
-    if(dump_ind!=0)
-    {
-      /* There is a little offset ... */
-      dump_log(vh,DEFAULT_SECTOR_SIZE);
-    }
-    if(verbose>1)
-    {
-      log_info("blocksize %u\n",(unsigned) be32(vh->blocksize));
-      log_info("total_blocks %u\n",(unsigned) be32(vh->total_blocks));
-      log_info("free_blocks  %u\n",(unsigned) be32(vh->free_blocks));
-    }
-    partition->upart_type=UP_HFSP;
-    return 0;
   }
-  if (vh->signature==be16(HFSX_VOLHEAD_SIG) && be32(vh->blocksize)%512==0 && be32(vh->blocksize)!=0 && be32(vh->free_blocks)<=be32(vh->total_blocks))
+  else if (be16(vh->version)==5 && vh->signature==be16(HFSX_VOLHEAD_SIG))
   {
+    partition->upart_type=UP_HFSX;
     if(verbose>0 || dump_ind!=0)
     {
       log_info("\nHFSX magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
     }
-    if(dump_ind!=0)
-    {
-      /* There is a little offset ... */
-      dump_log(vh,DEFAULT_SECTOR_SIZE);
-    }
-    if(verbose>1)
-    {
-      log_info("blocksize %u\n",(unsigned) be32(vh->blocksize));
-      log_info("total_blocks %u\n",(unsigned) be32(vh->total_blocks));
-      log_info("free_blocks  %u\n",(unsigned) be32(vh->free_blocks));
-    }
-    partition->upart_type=UP_HFSX;
-    return 0;
   }
-  return 1;
+  else
+  {
+    return 1;
+  }
+  if(dump_ind!=0)
+  {
+    /* There is a little offset ... */
+    dump_log(vh,DEFAULT_SECTOR_SIZE);
+  }
+  if(verbose>1)
+  {
+    log_info("blocksize %u\n",(unsigned) be32(vh->blocksize));
+    log_info("total_blocks %u\n",(unsigned) be32(vh->total_blocks));
+    log_info("free_blocks  %u\n",(unsigned) be32(vh->free_blocks));
+  }
+  return 0;
 }
 
-static int set_HFSP_info(partition_t *partition, const struct hfsp_vh *vh)
+static int set_HFSP_info(partition_t *partition)
 {
   partition->fsname[0]='\0';
   if(partition->upart_type==UP_HFSP)
