@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
 #ifdef DJGPP
+#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -51,9 +52,9 @@ static int hd_identify_enh_bios(disk_t *param_disk,const int verbose);
 static int check_enh_bios(const unsigned int disk, const int verbose);
 static int hd_report_error(disk_t *disk_car, const uint64_t hd_offset, const unsigned int count, const int rc);
 static const char *disk_description_short(disk_t *disk_car);
-static int disk_read(disk_t *disk_car, const unsigned int count, void *buf, const uint64_t hd_offset);
-static int disk_write(disk_t *disk_car, const unsigned int count, const void *buf, const uint64_t hd_offset);
-static int disk_nowrite(disk_t *disk_car, const unsigned int count, const void *buf, const uint64_t offset);
+static int disk_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t hd_offset);
+static int disk_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t hd_offset);
+static int disk_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int disk_sync(disk_t *disk_car);
 static int disk_clean(disk_t *disk_car);
 
@@ -87,7 +88,7 @@ static void disk_reset_error(disk_t *disk_car)
   biosdisk(0, data->disk, 0, 0, 1, 1, NULL);
 }
 
-static int hd_read(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
+static int hd_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
 {
   __dpmi_regs r;
   unsigned char buf_cmd[HD_RW_BUF_SIZ];
@@ -100,24 +101,27 @@ static int hd_read(disk_t *disk_car, void *buf, const unsigned int count, const
   if(data->mode_enh==0)
   { /* Limite CHS = 1023,255,63 = 8,064Mo ~= 7.8 Go */
     int head, track, sector;
-    if(data->CHSR.sector==0)
+    if(data->geo_phys.sectors_per_head==0)
     {
-      log_critical("hd_read: BUG CHSR.sector=0 !\n");
-      return 1;
+      log_critical("hd_pread: BUG geo_phys.sectors_per_head=0 !\n");
+      return -1;
     }
-    sector=(hd_offset%data->CHSR.sector)+1;
-    hd_offset/=data->CHSR.sector;
-    head=hd_offset%(data->CHSR.head+1);
-    track=hd_offset/(data->CHSR.head+1);
+    sector=(hd_offset%data->geo_phys.sectors_per_head)+1;
+    hd_offset/=data->geo_phys.sectors_per_head;
+    head=hd_offset%data->geo_phys.heads_per_cylinder;
+    track=hd_offset/data->geo_phys.heads_per_cylinder;
     if(track<1024)
-      return biosdisk(2, data->disk, head, track, sector, nsects, buf);
-    return 1;
+    {
+      const int res=biosdisk(2, data->disk, head, track, sector, nsects, buf);
+      return (res!=0 ? -res: count);
+    }
+    return -1;
   }
   if(cmd_dos_segment==0)
     if(alloc_cmd_dos_buffer())
-      return 1;
+      return -1;
   if ( (xfer_dos_segment=__dpmi_allocate_dos_memory((count + 15) >> 4, &xfer_dos_selector)) == -1 )
-    return 1;
+    return -1;
   *(uint16_t*)&buf_cmd[0]=HD_RW_BUF_SIZ;
   *(uint16_t*)&buf_cmd[2]=nsects;
   *(uint32_t*)&buf_cmd[0x4]=xfer_dos_segment<<16;
@@ -132,10 +136,10 @@ static int hd_read(disk_t *disk_car, void *buf, const unsigned int count, const
   __dpmi_int(0x13, &r);
   dosmemget(xfer_dos_segment<<4, count, buf);
   __dpmi_free_dos_memory(xfer_dos_selector);
-  return r.h.ah;
+  return (r.h.ah!=0 ? -r.h.ah : count);
 }
 
-static int hd_write(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
+static int hd_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   __dpmi_regs r;
   unsigned char buf_cmd[HD_RW_BUF_SIZ];
@@ -149,24 +153,27 @@ static int hd_write(disk_t *disk_car, const void *buf, const unsigned int count,
   if(data->mode_enh==0)
   { /* Limite CHS = 1023,255,63 = 8,064Mo ~= 7.8 Go */
     int head, track, sector;
-    if(data->CHSR.sector==0)
+    if(data->geo_phys.sectors_per_head==0)
     {
-      log_critical("hd_write: BUG CHSR.sector=0 !\n");
-      return 1;
+      log_critical("hd_pwrite: BUG geo_phys.sectors_per_head=0 !\n");
+      return -1;
     }
-    sector=(hd_offset%data->CHSR.sector)+1;
-    hd_offset/=data->CHSR.sector;
-    head=hd_offset%(data->CHSR.head+1);
-    track=hd_offset/(data->CHSR.head+1);
+    sector=(hd_offset%data->geo_phys.sectors_per_head)+1;
+    hd_offset/=data->geo_phys.sectors_per_head;
+    head=hd_offset%data->geo_phys.heads_per_cylinder;
+    track=hd_offset/data->geo_phys.heads_per_cylinder;
     if(track<1024)
-      return biosdisk(3, data->disk, head, track, sector, nsects, buf);
-    return 1;
+    {
+      const int res=biosdisk(3, data->disk, head, track, sector, nsects, buf);
+      return (res!=0 ? -res : count);
+    }
+    return -1;
   }
   if(cmd_dos_segment==0)
     if(alloc_cmd_dos_buffer())
-      return 1;
+      return -1;
   if ( (xfer_dos_segment=__dpmi_allocate_dos_memory((count + 15) >> 4, &xfer_dos_selector)) == -1 )
-    return 1;
+    return -1;
   *(uint16_t*)&buf_cmd[0]=HD_RW_BUF_SIZ;
   *(uint16_t*)&buf_cmd[2]=nsects;
   *(uint32_t*)&buf_cmd[0x4]=xfer_dos_segment<<16;
@@ -181,7 +188,7 @@ static int hd_write(disk_t *disk_car, const void *buf, const unsigned int count,
   dosmemput(&buf_cmd, HD_RW_BUF_SIZ, cmd_dos_segment<<4);
   __dpmi_int(0x13, &r);
   __dpmi_free_dos_memory(xfer_dos_selector);
-  return r.h.ah;
+  return (r.h.ah!=0 ? -r.h.ah : count);
 }
 
 static int check_enh_bios(const unsigned int disk, const int verbose)
@@ -229,6 +236,7 @@ static int check_enh_bios(const unsigned int disk, const int verbose)
 
 static int hd_identify_enh_bios(disk_t *disk_car,const int verbose)
 {
+  uint64_t computed_size;
   int compute_LBA=0;
   __dpmi_regs r;
   unsigned char buf[0x200];	/* Don't change it! */
@@ -247,13 +255,13 @@ static int hd_identify_enh_bios(disk_t *disk_car,const int verbose)
   dosmemget(cmd_dos_segment<<4, HDPARM_BUF_SIZ, &buf);
   if(r.h.ah)
     return 1;
-  disk_car->CHS.cylinder=*(uint16_t*)&buf[0x04];
-  disk_car->CHS.head=*(uint16_t*)&buf[0x08];
-  disk_car->CHS.sector=*(uint16_t*)&buf[0x0C];
+  disk_car->geom.cylinders=*(uint16_t*)&buf[0x04];
+  disk_car->geom.heads_per_cylinder=*(uint16_t*)&buf[0x08];
+  disk_car->geom.sectors_per_head=*(uint16_t*)&buf[0x0C];
   disk_car->disk_size=(*(uint32_t*)&buf[0x10])*(uint64_t)disk_car->sector_size;
   if(disk_car->disk_size==0)
   {
-    if(disk_car->CHS.cylinder==0 || disk_car->CHS.head==0 || disk_car->CHS.sector==0)
+    if(disk_car->geom.cylinders==0 || disk_car->geom.heads_per_cylinder==0 || disk_car->geom.sectors_per_head==0)
     {
       if(verbose>0)
 	log_warning("hd_identify_enh_bios: No size returned by BIOS.\n");
@@ -261,24 +269,23 @@ static int hd_identify_enh_bios(disk_t *disk_car,const int verbose)
     }
     else
     {
-      disk_car->CHS.cylinder--;
-      disk_car->CHS.head--;
       compute_LBA=1;
-      disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+      disk_car->disk_size=(uint64_t)disk_car->geom.cylinders*disk_car->geom.heads_per_cylinder*disk_car->geom.sectors_per_head*disk_car->sector_size;
       if(verbose>0)
         log_verbose("Computes LBA from CHS\n");
     }
   }
   else
   {
-    if(disk_car->CHS.cylinder>0 && disk_car->CHS.head>0 && disk_car->CHS.sector>0)
+    if(disk_car->geom.cylinders>0 && disk_car->geom.heads_per_cylinder>0 && disk_car->geom.sectors_per_head>0)
     {
-      disk_car->CHS.cylinder--;
-      disk_car->CHS.head--;
       /* Some bios are buggy */
-      if(disk_car->disk_size>(uint64_t)(disk_car->CHS.cylinder+2)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size)
+      if(disk_car->disk_size>(uint64_t)(disk_car->geom.cylinders+1)*disk_car->geom.heads_per_cylinder*disk_car->geom.sectors_per_head*disk_car->sector_size)
       {
-        disk_car->CHS.cylinder=(disk_car->disk_size/(disk_car->CHS.head+1))/disk_car->CHS.sector/disk_car->sector_size-1;
+        disk_car->geom.cylinders=disk_car->disk_size /
+	  disk_car->geom.heads_per_cylinder /
+	  disk_car->geom.sectors_per_head /
+	  disk_car->sector_size;
         if(verbose>0)
           log_verbose("Computes C from number of sectors\n");
       }
@@ -287,43 +294,52 @@ static int hd_identify_enh_bios(disk_t *disk_car,const int verbose)
     {
       if(verbose>0)
         log_verbose("Computes CHS from number of sectors\n");
-      disk_car->CHS.head=255-1;
-      disk_car->CHS.sector=63;
-      disk_car->CHS.cylinder=(disk_car->disk_size/(disk_car->CHS.head+1))/disk_car->CHS.sector/disk_car->sector_size-1;
+      disk_car->geom.heads_per_cylinder=255;
+      disk_car->geom.sectors_per_head=63;
+      disk_car->geom.cylinders=disk_car->disk_size /
+	disk_car->geom.heads_per_cylinder /
+	disk_car->geom.sectors_per_head /
+	disk_car->sector_size;
     }
   }
-  if(disk_car->CHS.sector==0)
+  if(disk_car->geom.sectors_per_head==0)
   {
     data->bad_geometry=1;
-    disk_car->CHS.sector=1;
+    disk_car->geom.sectors_per_head=1;
     log_critical("Incorrect number of sector\n");
   }
-  if(disk_car->CHS.sector>63)
+  if(disk_car->geom.sectors_per_head>63)
   {
 /*    data->bad_geometry=1; */
     log_critical("Incorrect number of sector\n");
   }
-  if(disk_car->CHS.head>255-1)
+  if(disk_car->geom.heads_per_cylinder>255)
   {
     data->bad_geometry=1;
     log_critical("Incorrect number of head\n");
   }
+  computed_size=(uint64_t)disk_car->geom.cylinders*disk_car->geom.heads_per_cylinder*disk_car->geom.sectors_per_head*disk_car->sector_size;
   if(verbose>0 || data->bad_geometry!=0)
-    log_info("LBA %lu, computed %u (CHS=%u,%u,%u)\n",(long unsigned)(disk_car->disk_size/disk_car->sector_size), (disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector,disk_car->CHS.cylinder,disk_car->CHS.head,disk_car->CHS.sector);
+    log_info("LBA %lu, computed %lu (CHS=%lu,%u,%u)\n",
+	(long unsigned)(disk_car->disk_size/disk_car->sector_size),
+	(long unsigned)(computed_size/disk_car->sector_size),
+	disk_car->geom.cylinders,
+	disk_car->geom.heads_per_cylinder,
+	disk_car->geom.sectors_per_head);
   if(compute_LBA)
-    disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+    disk_car->disk_size=computed_size;
   else
   {
-    if(disk_car->disk_size < (uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector/disk_car->sector_size)
+    if(disk_car->disk_size < computed_size)
     {
       log_info("Computes LBA from CHS, previous value may be false.\n");
-      disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+      disk_car->disk_size=computed_size;
     }
   }
   disk_car->disk_real_size=disk_car->disk_size;
-  data->CHSR.cylinder=disk_car->CHS.cylinder;
-  data->CHSR.head=disk_car->CHS.head;
-  data->CHSR.sector=disk_car->CHS.sector;
+  data->geo_phys.cylinders=disk_car->geom.cylinders;
+  data->geo_phys.heads_per_cylinder=disk_car->geom.heads_per_cylinder;
+  data->geo_phys.sectors_per_head=disk_car->geom.sectors_per_head;
   if(verbose>0)
   {
     log_info("hd_identify_enh_bios\n");
@@ -360,32 +376,32 @@ disk_t *hd_identify(const int verbose, const unsigned int disk, const arch_fnct_
     disk_car->sector_size=DEFAULT_SECTOR_SIZE;
     disk_car->description=disk_description;
     disk_car->description_short=disk_description_short;
-    disk_car->read=disk_read;
-    disk_car->write=((testdisk_mode&TESTDISK_O_RDWR)==TESTDISK_O_RDWR?disk_write:disk_nowrite);
+    disk_car->pread=disk_pread;
+    disk_car->pwrite=((testdisk_mode&TESTDISK_O_RDWR)==TESTDISK_O_RDWR?disk_pwrite:disk_nopwrite);
     disk_car->sync=disk_sync;
     disk_car->access_mode=testdisk_mode;
     disk_car->clean=disk_clean;
     disk_car->data=data;
-    disk_car->CHS.cylinder=((buf[0] & 0x0C0)<<2)|buf[1];
-    disk_car->CHS.head=buf[3];
-    disk_car->CHS.sector=buf[0] & 0x3F;
-    if(disk_car->CHS.head>=255)
+    disk_car->geom.cylinders=1+(((buf[0] & 0x0C0)<<2)|buf[1]);
+    disk_car->geom.heads_per_cylinder=1+buf[3];
+    disk_car->geom.sectors_per_head=buf[0] & 0x3F;
+    if(disk_car->geom.heads_per_cylinder>255)
     { /* Problem found by G Rowe */
       log_critical("BIOS reports an invalid heads number\n");
       data->bad_geometry=1;
-      disk_car->CHS.head=254;
+      disk_car->geom.heads_per_cylinder=255;
     }
-    if(disk_car->CHS.sector==0)
+    if(disk_car->geom.sectors_per_head==0)
     { /* Problem found by Brian Barrett */
       log_critical("BIOS reports an invalid number of sector per head\n");
       data->bad_geometry=1;
-      disk_car->CHS.sector=1;
+      disk_car->geom.sectors_per_head=1;
     }
-    disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+    disk_car->disk_size=(uint64_t)disk_car->geom.cylinders*disk_car->geom.heads_per_cylinder*disk_car->geom.sectors_per_head*disk_car->sector_size;
     disk_car->disk_real_size=disk_car->disk_size;
-    data->CHSR.cylinder=disk_car->CHS.cylinder;
-    data->CHSR.head=disk_car->CHS.head;
-    data->CHSR.sector=disk_car->CHS.sector;
+    data->geo_phys.cylinders=disk_car->geom.cylinders;
+    data->geo_phys.heads_per_cylinder=disk_car->geom.heads_per_cylinder;
+    data->geo_phys.sectors_per_head=disk_car->geom.sectors_per_head;
     if(verbose>0)
       log_info("%s\n",disk_description(disk_car));
     if(check_enh_bios(disk,verbose))
@@ -401,7 +417,7 @@ disk_t *hd_identify(const int verbose, const unsigned int disk, const arch_fnct_
 	/* standard geometry H,S, compute C from LBA */
 	disk_car->disk_size=param_disk_enh->disk_size;
 	disk_car->disk_real_size=disk_car->disk_size;
-	disk_car->CHS.cylinder=(disk_car->disk_size/(disk_car->CHS.head+1))/disk_car->CHS.sector/disk_car->sector_size-1;
+	disk_car->geom.cylinders=(disk_car->disk_size/disk_car->geom.heads_per_cylinder)/disk_car->geom.sectors_per_head/disk_car->sector_size;
       }
       else
 	data->mode_enh=0;
@@ -416,9 +432,9 @@ const char *disk_description(disk_t *disk_car)
 {
   struct info_disk_struct*data=disk_car->data;
   char buffer_disk_size[100];
-  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %2x - %s - CHS %u %u %u%s",
+  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %2x - %s - CHS %lu %u %u%s",
       data->disk, size_to_unit(disk_car->disk_size,buffer_disk_size),
-      disk_car->CHS.cylinder+1, disk_car->CHS.head+1, disk_car->CHS.sector,
+      disk_car->geom.cylinders, disk_car->geom.heads_per_cylinder, disk_car->geom.sectors_per_head,
       data->bad_geometry!=0?" (Buggy BIOS)":"");
   return disk_car->description_txt;
 }
@@ -432,12 +448,12 @@ static const char *disk_description_short(disk_t *disk_car)
   return disk_car->description_short_txt;
 }
 
-static int disk_read_aux(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
+static int disk_pread_aux(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
 {
   struct info_disk_struct*data=disk_car->data;
-  if(data->CHSR.cylinder>0 && offset+count>disk_car->disk_size)
+  if(data->geo_phys.cylinders>0 && offset+count>disk_car->disk_size)
   {
-    log_error("disk_read_aux: Don't read after the end of the disk\n");
+    log_error("disk_pread_aux: Don't read after the end of the disk\n");
     return -1;
   }
   {
@@ -450,60 +466,58 @@ static int disk_read_aux(disk_t *disk_car, void *buf, const unsigned int count,
       read_size=count-read_offset>16*512?16*512:count-read_offset;
       do
       {
-        rc=hd_read(disk_car, (char*)buf+read_offset, read_size, offset+read_offset);
-        if(rc!=0)
+        rc=hd_pread(disk_car, (char*)buf+read_offset, read_size, offset+read_offset);
+        if(rc < 0)
           disk_reset_error(disk_car);
-      } while(rc!=0 && rc!=1 && ++i<MAX_IO_NBR);
-      // 0=successful completion
-      // 1=invalid function in AH or invalid parameter
-      if(rc!=0)
+      } while(rc!=read_size && rc!=-1 && ++i<MAX_IO_NBR);
+      // <0 invalid function in AH or invalid parameter
+      if(rc < 0)
       {
-        log_error("disk_read_aux failed ");
-        hd_report_error(disk_car, offset, count, rc);
-        return -rc;
+        log_error("disk_pread_aux failed ");
+        hd_report_error(disk_car, offset, count, -rc);
       }
+      if(rc != read_size)
+	return (read_offset==0 ? rc : read_offset);
       read_offset+=read_size;
     } while(read_offset<count);
   }
-  return 0;
+  return count;
 }
 
-static int disk_read(disk_t *disk_car, const unsigned int count, void *buf, const uint64_t offset)
+static int disk_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
 {
-  return align_read(&disk_read_aux, disk_car, buf, count, offset);
+  return align_pread(&disk_pread_aux, disk_car, buf, count, offset);
 }
 
-static int disk_write_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t hd_offset)
+static int disk_pwrite_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t hd_offset)
 {
-
   struct info_disk_struct*data=disk_car->data;
   int i=0;
   int rc;
   disk_car->write_used=1;
   {
-    rc=hd_write(disk_car, buf, count, hd_offset);
-    if(rc!=0)
+    rc=hd_pwrite(disk_car, buf, count, hd_offset);
+    if(rc < 0)
       disk_reset_error(disk_car);
-  } while(rc==4 && ++i<MAX_IO_NBR);
+  } while(rc==-4 && ++i<MAX_IO_NBR);
   /* 4=sector not found/read error */
-  if(rc!=0)
+  if(rc < 0)
   {
-    log_error("disk_write error\n");
-    hd_report_error(disk_car, hd_offset, count, rc);
-    return -rc;
+    log_error("disk_pwrite error\n");
+    hd_report_error(disk_car, hd_offset, count, -rc);
   }
-  return 0;
+  return rc;
 }
 
-static int disk_write(disk_t *disk_car, const unsigned int count, const void *buf, const uint64_t offset)
+static int disk_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
-  return align_write(&disk_read_aux, &disk_write_aux, disk_car, buf, count, offset);
+  return align_pwrite(&disk_pread_aux, &disk_pwrite_aux, disk_car, buf, count, offset);
 }
 
-static int disk_nowrite(disk_t *disk_car,const unsigned int count, const void *buf, const uint64_t offset)
+static int disk_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   struct info_disk_struct *data=disk_car->data;
-  log_warning("disk_nowrite(%d,%u,buffer,%lu(%u/%u/%u)) write refused\n", data->disk,
+  log_warning("disk_nopwrite(%d,%u,buffer,%lu(%u/%u/%u)) write refused\n", data->disk,
       (unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
       offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset));
   return -1;
