@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -36,13 +37,15 @@
 #include "intrfn.h"
 #include "dirpart.h"
 #include "ntfs.h"
-#include "fnctdsk.h"
 #include "lang.h"
 #include "io_redir.h"
 #include "log.h"
+#include "log_part.h"
 
 #define INTER_NTFS_X 0
 #define INTER_NTFS_Y 23
+#define INTER_NTFSBS_X		0
+#define INTER_NTFSBS_Y		22
 
 #define MAX_INFO_MFT 10
 #define NTFS_SECTOR_SIZE 0x200
@@ -131,15 +134,19 @@ static void menu_write_ntfs_boot_sector_cli(disk_t *disk_car, partition_t *parti
     else if(strncmp(*current_cmd,"write",5)==0)
     {
       (*current_cmd)+=5;
-      if(no_confirm!=0 || ask_confirmation("Write new NTFS boot sector, confirm ? (Y/N)")!=0)
+      if(no_confirm!=0
+#ifdef HAVE_NCURSES
+	|| ask_confirmation("Write new NTFS boot sector, confirm ? (Y/N)")!=0
+#endif
+	)
       {
 	log_info("Write new boot!\n");
 	/* Write boot sector and backup boot sector */
-	if(disk_car->write(disk_car,NTFS_SECTOR_SIZE, newboot, partition->part_offset))
+	if(disk_car->pwrite(disk_car, newboot, NTFS_SECTOR_SIZE, partition->part_offset) != NTFS_SECTOR_SIZE)
 	{
 	  display_message("Write error: Can't write new NTFS boot sector\n");
 	}
-	if(disk_car->write(disk_car,NTFS_SECTOR_SIZE, newboot, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
+	if(disk_car->pwrite(disk_car, newboot, NTFS_SECTOR_SIZE, partition->part_offset + partition->part_size - disk_car->sector_size) != NTFS_SECTOR_SIZE)
 	{
 	  display_message("Write error: Can't write new NTFS backup boot sector\n");
 	}
@@ -194,25 +201,27 @@ static void menu_write_ntfs_boot_sector_ncurses(disk_t *disk_car, partition_t *p
       ncurses_ntfs_info(ntfs_header);
       wprintw(stdscr,"Extrapolated boot sector and current boot sector are identical.\n");
     }
-    command=wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuSaveBoot,8,options,MENU_HORIZ | MENU_BUTTON, 1);
+    command=wmenuSelect(stdscr, INTER_NTFSBS_Y+1, INTER_NTFSBS_Y, INTER_NTFSBS_X, menuSaveBoot,8,options,MENU_HORIZ | MENU_BUTTON, 1);
     switch(command)
     {
       case 'w':
       case 'W':
+#ifdef HAVE_NCURSES
 	if(strchr(options,'W')!=NULL && ask_confirmation("Write new NTFS boot sector, confirm ? (Y/N)")!=0)
 	{
 	  log_info("Write new boot!\n");
 	  /* Write boot sector and backup boot sector */
-	  if(disk_car->write(disk_car,NTFS_SECTOR_SIZE, newboot, partition->part_offset))
+	  if(disk_car->pwrite(disk_car, newboot, NTFS_SECTOR_SIZE, partition->part_offset) != NTFS_SECTOR_SIZE)
 	  {
 	    display_message("Write error: Can't write new NTFS boot sector\n");
 	  }
-	  if(disk_car->write(disk_car,NTFS_SECTOR_SIZE, newboot, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
+	  if(disk_car->pwrite(disk_car, newboot, NTFS_SECTOR_SIZE, partition->part_offset + partition->part_size - disk_car->sector_size) != NTFS_SECTOR_SIZE)
 	  {
 	    display_message("Write error: Can't write new NTFS backup boot sector\n");
 	  }
           disk_car->sync(disk_car);
 	}
+#endif
 	return;
       case 'd':
       case 'D':
@@ -269,7 +278,7 @@ static int create_ntfs_boot_sector(disk_t *disk_car, partition_t *partition, con
   struct ntfs_boot_sector *org_ntfs_header=(struct ntfs_boot_sector *)&orgboot;
   struct ntfs_boot_sector *ntfs_header=(struct ntfs_boot_sector *)&newboot;
   int error=0;
-  if(disk_car->read(disk_car,NTFS_SECTOR_SIZE, &orgboot, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, &orgboot, NTFS_SECTOR_SIZE, partition->part_offset) != NTFS_SECTOR_SIZE)
   {
     log_error("create_ntfs_boot_sector: Can't read boot sector.\n");
     memset(&orgboot,0,NTFS_SECTOR_SIZE);
@@ -296,18 +305,18 @@ static int create_ntfs_boot_sector(disk_t *disk_car, partition_t *partition, con
   ntfs_header->sectors[1]=0;
   ntfs_header->media=0xF8;
   ntfs_header->fat_length=le16(0);
-  ntfs_header->secs_track=le16(disk_car->CHS.sector);
-  ntfs_header->heads=le16(disk_car->CHS.head+1);
+  ntfs_header->secs_track=le16(disk_car->geom.sectors_per_head);
+  ntfs_header->heads=le16(disk_car->geom.heads_per_cylinder);
   /* absolute sector address from the beginning of the disk (!= FAT) */
   ntfs_header->hidden=le32(partition->part_offset/disk_car->sector_size);
   ntfs_header->total_sect=le32(0);
   ntfs_header->sectors_nbr=le64(partition->part_size/disk_car->sector_size-1);
   ntfs_header->mft_lcn=le64(mft_lcn);
   ntfs_header->mftmirr_lcn=le64(mftmirr_lcn);
-  ntfs_header->clusters_per_mft_record=(mft_record_size >= cluster_size?mft_record_size / cluster_size:
-      -(testdisk_ffs(mft_record_size) - 1));
-  ntfs_header->clusters_per_index_record =(index_block_size >= cluster_size?index_block_size / cluster_size:
-      -(testdisk_ffs(index_block_size) - 1));
+  ntfs_header->clusters_per_mft_record=(mft_record_size >= cluster_size ?
+      mft_record_size / cluster_size : -(testdisk_ffs(mft_record_size) - 1));
+  ntfs_header->clusters_per_index_record =(index_block_size >= cluster_size ?
+      index_block_size / cluster_size : -(testdisk_ffs(index_block_size) - 1));
   ntfs_header->reserved0[0]=0;
   ntfs_header->reserved0[1]=0;
   ntfs_header->reserved0[2]=0;
@@ -347,12 +356,12 @@ static int read_mft_info(disk_t *disk_car, partition_t *partition, const uint64_
 {
   char buffer[8*DEFAULT_SECTOR_SIZE];
   const char *attr=buffer;
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)mft_sector*disk_car->sector_size)!=0)
+  if(disk_car->pread(disk_car, &buffer, sizeof(buffer), partition->part_offset + (uint64_t)mft_sector * disk_car->sector_size) != sizeof(buffer))
   {
     display_message("NTFS: Can't read mft_sector\n");
     return 1;
   }
-  *mft_lcn=ntfs_get_attr(attr,0x80,partition,buffer+8*DEFAULT_SECTOR_SIZE,verbose,0,NULL);
+  *mft_lcn=ntfs_get_attr(attr, 0x80, partition, buffer+8*DEFAULT_SECTOR_SIZE, verbose, NULL);
   *mft_record_size=NTFS_GETU32(attr + 0x1C);
   if(*mft_record_size==0)
   {
@@ -361,7 +370,7 @@ static int read_mft_info(disk_t *disk_car, partition_t *partition, const uint64_
     return 2;
   }
   attr+= NTFS_GETU32(attr + 0x1C);
-  *mftmirr_lcn=ntfs_get_attr(attr,0x80,partition,buffer+8*DEFAULT_SECTOR_SIZE,verbose,0,NULL);
+  *mftmirr_lcn=ntfs_get_attr(attr, 0x80, partition,buffer+8*DEFAULT_SECTOR_SIZE, verbose, NULL);
   /* Try to divide by the biggest number first */
   if(*mft_lcn<*mftmirr_lcn)
   {
@@ -426,7 +435,7 @@ static int read_mft_info(disk_t *disk_car, partition_t *partition, const uint64_
   return 3;
 }
 
-int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind,const int interface, const unsigned int expert, char **current_cmd)
+int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose, const int interface, const unsigned int expert, char **current_cmd)
 {
   uint64_t sector;
   char buffer[8*DEFAULT_SECTOR_SIZE];
@@ -456,13 +465,13 @@ int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose,
   /* try to find MFT Backup first */
   for(sector=(partition->part_size/disk_car->sector_size/2-20>0?partition->part_size/disk_car->sector_size/2-20:1);(sector<partition->part_size/disk_car->sector_size)&&(sector<=partition->part_size/disk_car->sector_size/2+20)&&(ind_stop==0);sector++)
   {
-    if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, &buffer, partition->part_offset+sector*(uint64_t)disk_car->sector_size)==0)
+    if(disk_car->pread(disk_car, &buffer, 2 * DEFAULT_SECTOR_SIZE, partition->part_offset + sector * (uint64_t)disk_car->sector_size) == 2 * DEFAULT_SECTOR_SIZE)
     {
       if(memcmp(buffer,"FILE",4)==0 && (NTFS_GETU16(buffer+ 0x14)%8==0) && (NTFS_GETU16(buffer+ 0x14)>=42)
 	  &&(NTFS_GETU16(buffer+22)==1))	/* MFT_RECORD_IN_USE */
       {
 	int res;
-	res=ntfs_get_attr(buffer,0x30,partition,buffer+2*DEFAULT_SECTOR_SIZE,verbose,0,"$MFT");
+	res=ntfs_get_attr(buffer, 0x30, partition, buffer+2*DEFAULT_SECTOR_SIZE, verbose, "$MFT");
 	if(res==1)
 	{
 	  int tmp;
@@ -472,7 +481,11 @@ int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose,
 	  {
 	    log_info("ntfs_find_mft: mft_lcn             %lu\n",(long unsigned int)mft_lcn);
 	    log_info("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)mftmirr_lcn);
-	    if(expert==0 || ask_confirmation("Use MFT from %lu, confirm ? (Y/N)",(long unsigned int)mft_lcn)!=0)
+	    if(expert==0
+#ifdef HAVE_NCURSES
+		|| ask_confirmation("Use MFT from %lu, confirm ? (Y/N)",(long unsigned int)mft_lcn)!=0
+#endif
+	      )
 	      ind_stop=1;
 	  }
 	  else if(tmp==3)
@@ -507,12 +520,12 @@ int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose,
       }
     }
 #endif
-    if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, &buffer, partition->part_offset+sector*(uint64_t)disk_car->sector_size)==0)
+    if(disk_car->pread(disk_car, &buffer, 2 * DEFAULT_SECTOR_SIZE, partition->part_offset + sector * (uint64_t)disk_car->sector_size) == 2 * DEFAULT_SECTOR_SIZE)
     {
       if(memcmp(buffer,"FILE",4)==0 && (NTFS_GETU16(buffer+ 0x14)%8==0) && (NTFS_GETU16(buffer+ 0x14)>=42))
       {
 	int res;
-	res=ntfs_get_attr(buffer,0x30,partition,buffer+2*DEFAULT_SECTOR_SIZE,verbose,0,"$MFT");
+	res=ntfs_get_attr(buffer, 0x30, partition, buffer+2*DEFAULT_SECTOR_SIZE, verbose, "$MFT");
 	if(res==1)
 	{
 	  int tmp;
@@ -522,7 +535,11 @@ int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose,
 	  {
 	    log_info("ntfs_find_mft: mft_lcn             %lu\n",(long unsigned int)mft_lcn);
 	    log_info("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)mftmirr_lcn);
-	    if(expert==0 || ask_confirmation("Use MFT from %lu, confirm ? (Y/N)",(long unsigned int)mft_lcn)!=0)
+	    if(expert==0
+#ifdef HAVE_NCURSES
+	      || ask_confirmation("Use MFT from %lu, confirm ? (Y/N)",(long unsigned int)mft_lcn)!=0
+#endif
+	      )
 	      ind_stop=1;
 	  }
 	  else if(tmp==3)
@@ -611,19 +628,19 @@ int rebuild_NTFS_BS(disk_t *disk_car, partition_t *partition, const int verbose,
   if(sectors_per_cluster>0 && mft_record_size>0)
   {
     unsigned int index_block_size=4096;
-    /* Read "root directory" in MFT */
-    if(disk_car->read(disk_car,mft_record_size, &buffer, partition->part_offset+(uint64_t)mft_lcn*sectors_per_cluster*disk_car->sector_size+5*(uint64_t)mft_record_size)!=0)
-    {
-      display_message("NTFS Can't read \"root directory\" in MFT\n");
-      return 1;
-    }
-    index_block_size=ntfs_get_attr(buffer,0x90,partition,buffer+mft_record_size,verbose,0,NULL);
-    if(index_block_size%512!=0)
-      index_block_size=4096;
     log_info("ntfs_find_mft: sectors_per_cluster %u\n",sectors_per_cluster);
     log_info("ntfs_find_mft: mft_lcn             %lu\n",(long unsigned int)mft_lcn);
     log_info("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)mftmirr_lcn);
     log_info("ntfs_find_mft: mft_record_size     %u\n",mft_record_size);
+    /* Read "root directory" in MFT */
+    if((unsigned)disk_car->pread(disk_car, &buffer, mft_record_size, partition->part_offset + (uint64_t)mft_lcn * sectors_per_cluster * disk_car->sector_size + 5 * (uint64_t)mft_record_size) != mft_record_size)
+    {
+      display_message("NTFS Can't read \"root directory\" in MFT\n");
+      return 1;
+    }
+    index_block_size=ntfs_get_attr(buffer, 0x90, partition, buffer+mft_record_size, verbose, NULL);
+    if(index_block_size%512!=0)
+      index_block_size=4096;
     log_info("ntfs_find_mft: index_block_size    %u\n",index_block_size);
     create_ntfs_boot_sector(disk_car,partition, interface, sectors_per_cluster*disk_car->sector_size, mft_lcn, mftmirr_lcn, mft_record_size, index_block_size,current_cmd);
     /* TODO: ask if the user want to continue the search of MFT */
@@ -658,7 +675,7 @@ static int testdisk_ffs(int x)
     r += 2;
   }
   if (!(x & 1)) {
-    x >>= 1;
+//  x >>= 1;
     r += 1;
   }
   return r;
