@@ -1,7 +1,7 @@
 /**
  * ntfs_dir.c - Part of the TestDisk project.
  *
- * Copyright (c) 2004-2007 Christophe Grenier
+ * Copyright (c) 2004-2008 Christophe Grenier
  *
  * Original version comes from the Linux-NTFS project.
  * Copyright (c) 2003 Lode Leroy
@@ -79,6 +79,9 @@
 #define NTFS_DT_DIR               4
 #define NTFS_DT_REG             8
 #define NTFS_TIME_OFFSET ((s64)(369 * 365 + 89) * 24 * 3600 * 10000000)
+#ifndef FILE_first_user
+#define FILE_first_user 16
+#endif
 
 /*
  * This is the "ntfs_filldir" function type, used by ntfs_readdir() to let
@@ -155,7 +158,7 @@ static int ntfs_ucstoutf8(iconv_t cd, const ntfschar *ins, int ins_len, char **o
     inb_left = ins_len << 1;    // ntfschar is 16-bit
     outb_left = outs_len - 1;   // reserve 1 byte for NUL
 
-    if (iconv(cd, &inp, &inb_left, &outp, &outb_left) == (size_t)(-1))
+    if (iconv(cd, (char**)&inp, &inb_left, &outp, &outb_left) == (size_t)(-1))
     {
       // Regardless of the value of errno
       log_error("ntfs_ucstoutf8: iconv failed\n");
@@ -176,7 +179,7 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
 {
   char *filename = NULL;
   int result = 0;
-  filename = calloc (1, MAX_PATH);
+  filename = (char *)calloc (1, MAX_PATH);
   if (!filename)
   {
     log_critical("ntfs_td_list_entry calloc failed\n");
@@ -197,8 +200,8 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
 #endif
 
   result = 0;					/* These are successful */
-  if (filename[0] == '$')			/* system */
-    goto free;
+  if (MREF(mref) < FILE_first_user && filename[0] == '$')	/* Hide system file */
+      goto free;
   /* Keep FILE_NAME_WIN32 and FILE_NAME_POSIX */
   if ((name_type & FILE_NAME_WIN32_AND_DOS) == FILE_NAME_DOS)
     goto free;
@@ -238,28 +241,29 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
     }
 
     {
-      file_data_t *new_file=MALLOC(sizeof(*new_file));
+      file_data_t *new_file=(file_data_t *)MALLOC(sizeof(*new_file));
       memcpy(new_file->name,filename,(MAX_PATH<sizeof(new_file->name)?MAX_PATH:sizeof(new_file->name)));
+      new_file->status=0;
       new_file->prev=ls->current_file;
       new_file->next=NULL;
-      new_file->filestat.st_dev=0;
-      new_file->filestat.st_ino=MREF(mref);
-      new_file->filestat.st_mode = (dt_type == NTFS_DT_DIR?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
-      new_file->filestat.st_nlink=1;
-      new_file->filestat.st_uid=0;
-      new_file->filestat.st_gid=0;
-      new_file->filestat.st_rdev=0;
-      new_file->filestat.st_size=filesize;
-      new_file->filestat.st_blksize=DEFAULT_SECTOR_SIZE;
+      new_file->stat.st_dev=0;
+      new_file->stat.st_ino=MREF(mref);
+      new_file->stat.st_mode = (dt_type == NTFS_DT_DIR?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
+      new_file->stat.st_nlink=1;
+      new_file->stat.st_uid=0;
+      new_file->stat.st_gid=0;
+      new_file->stat.st_rdev=0;
+      new_file->stat.st_size=filesize;
+      new_file->stat.st_blksize=DEFAULT_SECTOR_SIZE;
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-      if(new_file->filestat.st_blksize!=0)
+      if(new_file->stat.st_blksize!=0)
       {
-	new_file->filestat.st_blocks=(new_file->filestat.st_size+new_file->filestat.st_blksize-1)/new_file->filestat.st_blksize;
+	new_file->stat.st_blocks=(new_file->stat.st_size+new_file->stat.st_blksize-1)/new_file->stat.st_blksize;
       }
 #endif
-      new_file->filestat.st_atime=ntfs2utc(sle64_to_cpu(file_name_attr->last_access_time));
-      new_file->filestat.st_ctime=ntfs2utc(sle64_to_cpu(file_name_attr->creation_time));
-      new_file->filestat.st_mtime=ntfs2utc(sle64_to_cpu(file_name_attr->last_data_change_time));
+      new_file->stat.st_atime=ntfs2utc(sle64_to_cpu(file_name_attr->last_access_time));
+      new_file->stat.st_ctime=ntfs2utc(sle64_to_cpu(file_name_attr->creation_time));
+      new_file->stat.st_mtime=ntfs2utc(sle64_to_cpu(file_name_attr->last_data_change_time));
       new_file->prev=ls->current_file;
       new_file->next=NULL;
       /* log_debug("fat: new file %s de=%p size=%u\n",new_file->name,de,de->size); */
@@ -315,9 +319,89 @@ static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir
   return ls->dir_list;
 }
 
+#if 0
+static int ntfs_copy_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_data_t *dir)
+{
+  file_data_t *dir_list;
+  struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
+  const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
+  file_data_t *current_file;
+  char *dir_name;
+  int copy_bad=0;
+  int copy_ok=0;
+  dir_name=mkdir_local(dir_data->local_dir, dir_data->current_directory);
+  {
+    ntfs_inode *inode;
+    s64 pos;
+    ls->dir_list=NULL;
+    ls->current_file=NULL;
+
+    inode = ntfs_inode_open (ls->vol, dir->stat.st_ino);
+    if (!inode) {
+      log_error("ntfs_dir: ntfs_inode_open failed\n");
+      free(dir_name);
+      return -2;
+    }
+
+    /*
+     * We now are at the final path component.  If it is a file just
+     * list it.  If it is a directory, list its contents.
+     */
+    pos = 0;
+    if (inode->mrec->flags & MFT_RECORD_IS_DIRECTORY) {
+      if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)ntfs_td_list_entry)<0)
+      {
+	log_error("ntfs_readdir failed\n");
+      }
+    }
+    else
+      log_critical("ntfs_readdir BUG not MFT_RECORD_IS_DIRECTORY\n");
+    /* Finished with the inode; release it. */
+    ntfs_inode_close(inode);
+  }
+  for(current_file=ls->dir_list;current_file!=NULL;current_file=current_file->next)
+  {
+    if(current_directory_namelength+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
+    {
+      if(strcmp(dir_data->current_directory,"/"))
+	strcat(dir_data->current_directory,"/");
+      strcat(dir_data->current_directory,current_file->name);
+      if(LINUX_S_ISDIR(current_file->stat.st_mode)!=0)
+      {
+	int tmp=0;
+	if(current_file->stat.st_ino != dir->stat.st_ino &&
+	    strcmp(current_file->name,"..")!=0 && strcmp(current_file->name,".")!=0)
+	  tmp=ntfs_copy_dir(disk, partition, dir_data, current_file);
+	if(tmp>=-1)
+	  copy_ok=1;
+	if(tmp<0)
+	  copy_bad=1;
+      }
+      else if(LINUX_S_ISREG(current_file->stat.st_mode)!=0)
+      {
+//	log_trace("copy_file %s\n",dir_data->current_directory);
+	int tmp;
+	tmp=ntfs_copy(disk, partition, dir_data, current_file);
+	if(tmp==0)
+	  copy_ok=1;
+	else
+	  copy_bad=1;
+      }
+      dir_data->current_directory[current_directory_namelength]='\0';
+    }
+  }
+  delete_list_file(ls->dir_list);
+  set_date(dir_name, dir->stat.st_atime, dir->stat.st_mtime);
+  free(dir_name);
+  return (copy_bad>0?(copy_ok>0?-1:-2):0);
+}
+#endif
+
+enum { bufsize = 4096 };
+
 static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file)
 {
-  const unsigned long int first_inode=file->filestat.st_ino;
+  const unsigned long int first_inode=file->stat.st_ino;
   ntfs_inode *inode;
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
   inode = ntfs_inode_open (ls->vol, first_inode);
@@ -326,7 +410,6 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     return -1;
   }
   {
-    const int bufsize = 4096;
     char *buffer;
     char *new_file;
     ntfs_attr *attr;
@@ -334,14 +417,18 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     s64 bytes_read, written;
     s64 offset;
     u32 block_size;
-    buffer = MALLOC(bufsize);
+    buffer = (char *)MALLOC(bufsize);
     if (!buffer)
+    {
+      ntfs_inode_close(inode);
       return -2;
+    }
     attr = ntfs_attr_open(inode, AT_DATA, NULL, 0);
     if (!attr)
     {
       log_error("Cannot find attribute type 0x%lx.\n", (long) AT_DATA);
       free(buffer);
+      ntfs_inode_close(inode);
       return -3;
     }
     if ((inode->mft_no < 2) && (attr->type == AT_DATA))
@@ -350,18 +437,14 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       block_size = index_get_size(inode);
     else
       block_size = 0;
-    {
-      int l1=strlen(dir_data->local_dir);
-      int l2=strlen(dir_data->current_directory);
-      new_file=MALLOC(l1+l2+1);
-      l1=filename_convert(new_file,dir_data->local_dir,l1+1);
-      filename_convert(new_file+l1,dir_data->current_directory,l2+1);
-    }
-    f_out=create_file(new_file);
+    f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
     if(!f_out)
     {
       log_critical("Can't create file %s: %s\n",new_file, strerror(errno));
       free(new_file);
+      ntfs_attr_close(attr);
+      free(buffer);
+      ntfs_inode_close(inode);
       return -4;
     }
     offset = 0;
@@ -391,9 +474,9 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       offset += bytes_read;
     }
     fclose(f_out);
-    set_date(new_file, file->filestat.st_atime, file->filestat.st_mtime);
-    ntfs_attr_close(attr);
+    set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
     free(new_file);
+    ntfs_attr_close(attr);
     free(buffer);
   }
   /* Finished with the inode; release it. */
@@ -403,12 +486,8 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
 
 static void dir_partition_ntfs_close(dir_data_t *dir_data)
 {
-  const partition_t *partition;
-  disk_t *disk_car;
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
   /* ntfs_umount() will invoke ntfs_device_free() for us. */
-  disk_car=ls->my_data->disk_car;
-  partition=ls->my_data->partition;
   ntfs_umount(ls->vol, FALSE);
   free(ls->my_data);
 #ifdef HAVE_ICONV
@@ -433,7 +512,7 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
   dev = ntfs_device_alloc("/", 0, &ntfs_device_testdisk_io_ops, NULL);
   if (dev)
   {
-    my_data=MALLOC(sizeof(*my_data));
+    my_data=(my_data_t *)MALLOC(sizeof(*my_data));
     my_data->partition=partition;
     my_data->disk_car=disk_car;
     my_data->offset=0;
@@ -480,7 +559,9 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
 #endif
     strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
     dir_data->current_inode=FILE_root;
+    dir_data->param=0;
     dir_data->verbose=verbose;
+    dir_data->capabilities=0;
     dir_data->get_dir=ntfs_dir;
     dir_data->copy_file=ntfs_copy;
     dir_data->close=&dir_partition_ntfs_close;
