@@ -49,14 +49,13 @@
 #define INTER_MFT_X		0
 #define INTER_MFT_Y  		18
 
-int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, const unsigned int expert, char **current_cmd)
+int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, const unsigned int expert)
 {
   struct ntfs_boot_sector *ntfs_header;
   unsigned char *buffer_mft;
   unsigned char *buffer_mftmirr;
   unsigned int cluster_size;
   unsigned int mft_record_size;
-  unsigned int mftmirr_size;
   unsigned int mftmirr_size_bytes;
   unsigned int use_MFT=0;
   /* 0: do nothing
@@ -71,7 +70,7 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     return -1;
   }
   ntfs_header=(struct ntfs_boot_sector *)MALLOC(DEFAULT_SECTOR_SIZE);
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, ntfs_header, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, ntfs_header, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
   {
     free(ntfs_header);
     display_message("Can't read NTFS boot sector.\n");
@@ -86,15 +85,10 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
 
   cluster_size=ntfs_header->sectors_per_cluster;
 
-  if (cluster_size <= 4 * mft_record_size)
-    mftmirr_size = 4;
-  else
-    mftmirr_size = cluster_size / mft_record_size;
-  mftmirr_size_bytes=mftmirr_size * mft_record_size * ntfs_sector_size(ntfs_header);
+  mftmirr_size_bytes = (cluster_size <= 4 * mft_record_size ? 4 * mft_record_size : cluster_size) * ntfs_sector_size(ntfs_header);
 #ifdef DEBUG_REPAIR_MFT
   log_debug("mft_pos %lu\n",(unsigned long)(mft_pos/disk_car->sector_size));
   log_debug("mftmirr_pos %lu\n",(unsigned long)(mftmirr_pos/disk_car->sector_size));
-  log_debug("mftmirr_size %u\n", mftmirr_size);
   log_debug("cluster_size %u\n", cluster_size);
   log_debug("mft_record_size    %u\n", mft_record_size);
   log_debug("ntfs_sector_size   %u\n", ntfs_sector_size(ntfs_header));
@@ -109,7 +103,7 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
   }
   /* Check if MFT mirror is identical to the beginning of MFT */
   buffer_mft=(unsigned char *)MALLOC(mftmirr_size_bytes);
-  if(disk_car->read(disk_car, mftmirr_size_bytes, buffer_mft, mft_pos)!=0)
+  if((unsigned)disk_car->pread(disk_car, buffer_mft, mftmirr_size_bytes, mft_pos) != mftmirr_size_bytes)
   {
     display_message("Can't read NTFS MFT.\n");
     log_error("Can't read NTFS MFT.\n");
@@ -118,7 +112,7 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     return -1;
   }
   buffer_mftmirr=(unsigned char *)MALLOC(mftmirr_size_bytes);
-  if(disk_car->read(disk_car, mftmirr_size_bytes, buffer_mftmirr, mftmirr_pos)!=0)
+  if((unsigned)disk_car->pread(disk_car, buffer_mftmirr, mftmirr_size_bytes, mftmirr_pos) != mftmirr_size_bytes)
   {
     display_message("Can't read NTFS MFT mirror.\n");
     log_error("Can't read NTFS MFT mirror.\n");
@@ -136,6 +130,15 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     free(ntfs_header);
     return 0;
   }
+  if(partition->sb_offset!=0)
+  {
+    log_info("Please quit TestDisk and reboot your computer before trying to fix the MFT.\n");
+    display_message("Please quit TestDisk and reboot your computer before trying to fix the MFT.\n");
+    free(buffer_mftmirr);
+    free(buffer_mft);
+    free(ntfs_header);
+    return -1;
+  }
 /*
   log_debug("MFT\n");
   dump_log(buffer_mft, mftmirr_size_bytes);
@@ -171,9 +174,9 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
       if(dir_list!=NULL)
       {
 	log_info("NTFS listing using MFT:\n");
-	dir_aff_log(disk_car, partition, &dir_data, dir_list);
-	delete_list_file(dir_list);
-	res1++;
+	dir_aff_log(&dir_data, dir_list);
+	if(delete_list_file(dir_list)>2)
+	  res1++;
       }
       dir_data.close(&dir_data);
     }
@@ -188,42 +191,46 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
       if(dir_list!=NULL)
       {
 	log_info("NTFS listing using MFT mirror:\n");
-	dir_aff_log(disk_car, partition, &dir_data, dir_list);
-	delete_list_file(dir_list);
-	res2++;
+	dir_aff_log(&dir_data, dir_list);
+	if(delete_list_file(dir_list)>2)
+	  res2++;
       }
       dir_data.close(&dir_data);
     }
     io_redir_del_redir(disk_car,mft_pos);
     /* */
-    if(res1>res2)
+    if(res1>res2 && res1>0)
     {
       /* Use MFT */
-      if(ask_confirmation("Fix MFT mirror ? (Y/N)")!=0)
+#ifdef HAVE_NCURSES
+      if(ask_confirmation("Fix MFT mirror using MFT ? (Y/N)")!=0)
 	use_MFT=1;
       else
+#endif
 	log_info("Don't fix MFT mirror.\n");
     }
-    else if(res1<res2)
+    else if(res1<res2 && res2>0)
     {
       /* Use MFT mirror */
-      if(ask_confirmation("Fix MFT ? (Y/N)")!=0)
+#ifdef HAVE_NCURSES
+      if(ask_confirmation("Fix MFT using its mirror ? (Y/N)")!=0)
 	use_MFT=2;
       else
+#endif
 	log_info("Don't fix MFT.\n");
     }
     else
     { /* res1==res2 */
-      if(res1<0)
-	log_error("MFT and MFT mirror are bad. Failed to repair them.\n");
-      else
+      if(res1>0 && res2>0)
 	log_error("Both MFT seems ok but they don't match, use chkdsk.\n");
+      else
+	log_error("MFT and MFT mirror are bad. Failed to repair them.\n");
       if(expert==0)
       {
-	if(res1<0)
-	  display_message("MFT and MFT mirror are bad. Failed to repair them.\n");
-	else
+	if(res1>0 && res2>0)
 	  display_message("Both MFT seems ok but they don't match, use chkdsk.\n");
+	else
+	  display_message("MFT and MFT mirror are bad. Failed to repair them.\n");
       }
       else
       {
@@ -245,11 +252,11 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
 	wmove(stdscr,6,0);
 	aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
 	wmove(stdscr,8,0);
-	if(res1<0)
-	  wprintw(stdscr, "MFT and MFT mirror are bad.\n");
-	else
+	if(res1>0 && res2>0)
 	  wprintw(stdscr, "Both MFT seems ok but they don't match.\n");
-	command=wmenuSelect_ext(stdscr,INTER_MFT_Y, INTER_MFT_X, menuMFT, 10, "MBQ",
+	else
+	  wprintw(stdscr, "MFT and MFT mirror are bad.\n");
+	command=wmenuSelect_ext(stdscr, 23, INTER_MFT_Y, INTER_MFT_X, menuMFT, 10, "MBQ",
 	    MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu, &real_key);
 	switch(command)
 	{
@@ -271,29 +278,25 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
   }
   if(use_MFT==2)
   {
-    if(disk_car->write(disk_car, mftmirr_size_bytes, buffer_mftmirr, mft_pos)!=0)
+    if((unsigned)disk_car->pwrite(disk_car, buffer_mftmirr, mftmirr_size_bytes, mft_pos) != mftmirr_size_bytes)
     {
-      log_error("Failed to fix MFT: write error.\n");
       display_message("Failed to fix MFT: write error.\n");
     }
     else
     {
       disk_car->sync(disk_car);
-      log_info("MFT fixed.\n");
       display_message("MFT fixed.\n");
     }
   }
   else if(use_MFT==1)
   {
-    if(disk_car->write(disk_car, mftmirr_size_bytes, buffer_mft, mftmirr_pos)!=0)
+    if((unsigned)disk_car->pwrite(disk_car, buffer_mft, mftmirr_size_bytes, mftmirr_pos) != mftmirr_size_bytes)
     {
-      log_error("Failed to fix MFT mirror: write error.\n");
       display_message("Failed to fix MFT mirror: write error.\n");
     }
     else
     {
       disk_car->sync(disk_car);
-      log_info("MFT mirror fixed.\n");
       display_message("MFT mirror fixed.\n");
     }
   }
