@@ -20,7 +20,13 @@
 
  */
 #ifdef HAVE_LIBNTFS
+#ifdef __cplusplus
+extern "C" {
+#endif
 ATTR_RECORD * find_attribute(const ATTR_TYPES type, ntfs_attr_search_ctx *ctx);
 ATTR_RECORD * find_first_attribute(const ATTR_TYPES type, MFT_RECORD *mft);
 int utils_cluster_in_use(ntfs_volume *vol, long long lcn);
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
 #endif
