@@ -2,7 +2,7 @@
 
     File: ntfsp.c
 
-    Copyright (C) 2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2007-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,12 +42,12 @@
 #include "ntfsp.h"
 #endif
 #include "intrf.h"
-#include "intrfn.h"
 #include "dir.h"
 #include "ntfs.h"
 #include "ntfs_dir.h"
 #include "ntfs_inc.h"
 #include "log.h"
+#include "log_part.h"
 
 #ifdef HAVE_LIBNTFS
 #define SIZEOF_BUFFER ((const unsigned int)512)
@@ -55,32 +55,12 @@
 unsigned int ntfs_remove_used_space(disk_t *disk_car,const partition_t *partition, alloc_data_t *list_search_space)
 {
   dir_data_t dir_data;
-  unsigned int res;
   switch(dir_partition_ntfs_init(disk_car,partition,&dir_data,0))
   {
     case -2:
     case -1:
-      aff_buffer(BUFFER_RESET,"Q");
-      {
-#ifdef HAVE_NCURSES
-	WINDOW *window;
-	window=newwin(0,0,0,0);	/* full screen */
-	aff_copy(window);
-	wmove(window,4,0);
-	aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
-#endif
 	log_partition(disk_car,partition);
-	aff_buffer(BUFFER_ADD,"Can't open filesystem. Filesystem seems damaged.\n");
-	screen_buffer_to_log();
-#ifdef HAVE_NCURSES
-	screen_buffer_display(window,"",NULL);
-	delwin(window);
-	(void) clearok(stdscr, TRUE);
-#ifdef HAVE_TOUCHWIN
-	touchwin(stdscr);
-#endif
-#endif
-      }
+	log_error("Can't open filesystem. Filesystem seems damaged.\n");
       return 0;
   }
   {
@@ -90,45 +70,54 @@ unsigned int ntfs_remove_used_space(disk_t *disk_car,const partition_t *partitio
     uint64_t end_free=0;
     unsigned long int lcn;
     unsigned long int no_of_cluster;
-    unsigned int cluster_size;
-    unsigned int sector_size;
+    unsigned int cluster_size;	/* size in bytes */
     log_trace("ntfs_remove_used_space\n");
     buffer=(unsigned char *)MALLOC(SIZEOF_BUFFER);
     {
       const struct ntfs_boot_sector*ntfs_header=(const struct ntfs_boot_sector*)buffer;
-      if(disk_car->read(disk_car,512, buffer, partition->part_offset)!=0)
+      if(disk_car->pread(disk_car, buffer, 512, partition->part_offset) != 512)
       {
 	free(buffer);
+	dir_data.close(&dir_data);
 	return 0;
       }
-      no_of_cluster=le64(ntfs_header->sectors_nbr)/ntfs_header->sectors_per_cluster;
-      cluster_size=ntfs_header->sectors_per_cluster;
-      sector_size=ntfs_sector_size(ntfs_header);
-      res=cluster_size * sector_size;
+      cluster_size=ntfs_header->sectors_per_cluster*ntfs_sector_size(ntfs_header);
+      if(cluster_size==0)
+      {
+	free(buffer);
+	dir_data.close(&dir_data);
+	return 0;
+      }
+      no_of_cluster=(le64(ntfs_header->sectors_nbr) < partition->part_size ? le64(ntfs_header->sectors_nbr) : partition->part_size);
+      no_of_cluster/=ntfs_header->sectors_per_cluster;
     }
     for(lcn=0;lcn<no_of_cluster;lcn++)
     {
-      static long long int bmplcn = -SIZEOF_BUFFER - 1;	/* Which bit of $Bitmap is in the buffer */
+      static long long int bmplcn = - (SIZEOF_BUFFER << 3);	/* Which bit of $Bitmap is in the buffer */
       int byte, bit;
-      if ((lcn < bmplcn) || (lcn >= (bmplcn + (SIZEOF_BUFFER << 3))))
+      if ((bmplcn < 0) || (lcn < (unsigned)bmplcn) || (lcn >= ((unsigned)bmplcn + (SIZEOF_BUFFER << 3))))
       {
 	ntfs_attr *attr;
 	/* Mark the buffer as not in use, in case the read is shorter. */
 	memset(buffer, 0x00, SIZEOF_BUFFER);
 	bmplcn = lcn & (~((SIZEOF_BUFFER << 3) - 1));
 	attr = ntfs_attr_open(ls->vol->lcnbmp_ni, AT_DATA, AT_UNNAMED, 0);
-	if(attr)
-	{
-	  if (ntfs_attr_pread(attr, (bmplcn>>3), SIZEOF_BUFFER, buffer) < 0)
-	  {
-	    log_error("Couldn't read $Bitmap\n");
-	  }
-	  ntfs_attr_close(attr);
-	}
-	else
+	if(attr==NULL)
 	{
 	  log_error("Couldn't open $Bitmap\n");
+	  free(buffer);
+	  dir_data.close(&dir_data);
+	  return 0;
 	}
+	if (ntfs_attr_pread(attr, (bmplcn>>3), SIZEOF_BUFFER, buffer) < 0)
+	{
+	  log_error("Couldn't read $Bitmap\n");
+	  ntfs_attr_close(attr);
+	  free(buffer);
+	  dir_data.close(&dir_data);
+	  return 0;
+	}
+	ntfs_attr_close(attr);
       }
 
       bit  = 1 << (lcn & 7);
@@ -136,22 +125,22 @@ unsigned int ntfs_remove_used_space(disk_t *disk_car,const partition_t *partitio
       if((buffer[byte] & bit)!=0)
       {
 	/* Not free */
-	if(end_free+1==partition->part_offset+(uint64_t)lcn*cluster_size*sector_size)
-	  end_free+=cluster_size*sector_size;
+	if(end_free+1==partition->part_offset+(uint64_t)lcn*cluster_size)
+	  end_free+=cluster_size;
 	else
 	{
-	  if(start_free != end_free)
+	  if(start_free < end_free)
 	    del_search_space(list_search_space, start_free, end_free);
-	  start_free=partition->part_offset+(uint64_t)lcn*cluster_size*sector_size;
-	  end_free=start_free+(uint64_t)cluster_size*sector_size-1;
+	  start_free=partition->part_offset+(uint64_t)lcn*cluster_size;
+	  end_free=start_free+(uint64_t)cluster_size-1;
 	}
       }
     }
     free(buffer);
-    if(start_free != end_free)
+    if(start_free < end_free)
       del_search_space(list_search_space, start_free, end_free);
     dir_data.close(&dir_data);
-    return cluster_size * sector_size;
+    return cluster_size;
   }
 }
 #endif
