@@ -19,6 +19,12 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 void autodetect_arch(disk_t *disk);
 
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
