@@ -24,7 +24,8 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
- 
+
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -33,60 +34,52 @@
 #endif
 #include <ctype.h>      /* tolower */
 #include "types.h"
-#ifdef HAVE_UUID_UUID_H
+#if defined(HAVE_UUID_H)
+#include <uuid.h>
+#elif defined(HAVE_UUID_UUID_H)
 #include <uuid/uuid.h>
-#elif defined(HAVE_SYS_UUID_H)
+#endif
+#if defined(HAVE_SYS_UUID_H)
 #include <sys/uuid.h>
 #endif
 #include "common.h"
-#include "testdisk.h"
 #include "fnctdsk.h"
 #include "lang.h"
 #include "intrf.h"
-#include "intrfn.h"
 #include "chgtype.h"
 #include "partgpt.h"
 #include "savehdr.h"
 #include "cramfs.h"
+#include "exfat.h"
 #include "ext2.h"
 #include "fat.h"
-#include "hfs.h"
-#include "hfsp.h"
 #include "jfs_superblock.h"
 #include "jfs.h"
 #include "ntfs.h"
 #include "rfs.h"
 #include "xfs.h"
 #include "log.h"
+#include "log_part.h"
 #include "guid_cmp.h"
 #include "guid_cpy.h"
 #include "unicode.h"
 #include "crc.h"
-/* #include "partnone.h" */
+
 extern const arch_fnct_t arch_i386;
 
 static int check_part_gpt(disk_t *disk_car, const int verbose,partition_t *partition,const int saveheader);
 static list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int saveheader);
 static int write_part_gpt(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose, const int align);
 static list_part_t *init_part_order_gpt(const disk_t *disk_car, list_part_t *list_part);
-static list_part_t *add_partition_gpt(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd);
 static void set_next_status_gpt(const disk_t *disk_car, partition_t *partition);
 static int test_structure_gpt(list_part_t *list_part);
 static int is_part_known_gpt(const partition_t *partition);
 static void init_structure_gpt(const disk_t *disk_car,list_part_t *list_part, const int verbose);
 static const char *get_partition_typename_gpt(const partition_t *partition);
 static const char *get_gpt_typename(const efi_guid_t part_type_gpt);
-#if 0
-static int set_part_type_gpt(partition_t *partition, efi_guid_t part_type_gpt);
-static efi_guid_t get_part_type_gpt(const partition_t *partition);
-#endif
+static void efi_generate_uuid(efi_guid_t *ent_uuid);
 
-struct systypes_gtp {
-  const efi_guid_t part_type;
-  const char *name;
-};
-
-static const struct systypes_gtp gpt_sys_types[] = {
+const struct systypes_gtp gpt_sys_types[] = {
   { GPT_ENT_TYPE_EFI, 			"EFI System"		},
   { GPT_ENT_TYPE_MBR,			"MBR"			},
   { GPT_ENT_TYPE_FREEBSD,		"FreeBSD"		},
@@ -139,19 +132,15 @@ arch_fnct_t arch_gpt=
   .get_geometry_from_mbr=NULL,
   .check_part=check_part_gpt,
   .write_MBR_code=NULL,
-  .add_partition=add_partition_gpt,
   .set_prev_status=set_next_status_gpt,
   .set_next_status=set_next_status_gpt,
   .test_structure=test_structure_gpt,
+  .get_part_type=NULL,
   .set_part_type=NULL,
-//  .set_part_type=set_part_type_none,
-  .is_part_known=is_part_known_gpt,
   .init_structure=init_structure_gpt,
   .erase_list_part=NULL,
   .get_partition_typename=get_partition_typename_gpt,
-//  .get_part_type=get_part_type_gpt
-//  .get_part_type=get_part_type_none
-  .get_part_type=NULL
+  .is_part_known=&is_part_known_gpt
 };
 
 static void swap_uuid_and_efi_guid(efi_guid_t *guid)
@@ -171,33 +160,36 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
   uint64_t gpt_entries_offset;
 
   gpt=(struct gpt_hdr*)MALLOC(disk_car->sector_size);
-  aff_buffer(BUFFER_RESET,"Q");
-  if(disk_car->read(disk_car, disk_car->sector_size, gpt, disk_car->sector_size)!=0)
+  screen_buffer_reset();
+  if((unsigned)disk_car->pread(disk_car, gpt, disk_car->sector_size, disk_car->sector_size) != disk_car->sector_size)
   {
     free(gpt);
     return NULL;
   }
   if(memcmp(gpt->hdr_sig, GPT_HDR_SIG, 8)!=0)
   {
-    aff_buffer(BUFFER_ADD,"Bad GPT partition, invalid signature.\n");
+    screen_buffer_add("Bad GPT partition, invalid signature.\n");
     free(gpt);
     return NULL;
   }
-  log_info("hdr_size=%llu\n", (long long unsigned)le32(gpt->hdr_size));
-  log_info("hdr_lba_self=%llu\n", (long long unsigned)le64(gpt->hdr_lba_self));
-  log_info("hdr_lba_alt=%llu (expected %llu)\n",
-      (long long unsigned)le64(gpt->hdr_lba_alt),
-      (long long unsigned)((disk_car->disk_size-1)/disk_car->sector_size));
-  log_info("hdr_lba_start=%llu\n", (long long unsigned)le64(gpt->hdr_lba_start));
-  log_info("hdr_lba_end=%llu\n", (long long unsigned)le64(gpt->hdr_lba_end));
-  log_info("hdr_lba_table=%llu\n",
-      (long long unsigned)le64(gpt->hdr_lba_table));
-  log_info("hdr_entries=%llu\n", (long long unsigned)le32(gpt->hdr_entries));
-  log_info("hdr_entsz=%llu\n", (long long unsigned)le32(gpt->hdr_entsz));
+  if(verbose>0)
+  {
+    log_info("hdr_size=%llu\n", (long long unsigned)le32(gpt->hdr_size));
+    log_info("hdr_lba_self=%llu\n", (long long unsigned)le64(gpt->hdr_lba_self));
+    log_info("hdr_lba_alt=%llu (expected %llu)\n",
+	(long long unsigned)le64(gpt->hdr_lba_alt),
+	(long long unsigned)((disk_car->disk_size-1)/disk_car->sector_size));
+    log_info("hdr_lba_start=%llu\n", (long long unsigned)le64(gpt->hdr_lba_start));
+    log_info("hdr_lba_end=%llu\n", (long long unsigned)le64(gpt->hdr_lba_end));
+    log_info("hdr_lba_table=%llu\n",
+	(long long unsigned)le64(gpt->hdr_lba_table));
+    log_info("hdr_entries=%llu\n", (long long unsigned)le32(gpt->hdr_entries));
+    log_info("hdr_entsz=%llu\n", (long long unsigned)le32(gpt->hdr_entsz));
+  }
   /* Check header size */
   if(le32(gpt->hdr_size)<92 || le32(gpt->hdr_size) > disk_car->sector_size)
   {
-    aff_buffer(BUFFER_ADD,"GPT: invalid header size.\n");
+    screen_buffer_add("GPT: invalid header size.\n");
     free(gpt);
     return NULL;
   }
@@ -209,7 +201,7 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
     crc=get_crc32(gpt, le32(gpt->hdr_size), 0xFFFFFFFF)^0xFFFFFFFF;
     if(crc!=origcrc)
     {
-      aff_buffer(BUFFER_ADD,"Bad GPT partition, invalid header checksum.\n");
+      screen_buffer_add("Bad GPT partition, invalid header checksum.\n");
       free(gpt);
       return NULL;
     }
@@ -217,27 +209,27 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
   }
   if(le64(gpt->hdr_lba_self)!=1)
   {
-    aff_buffer(BUFFER_ADD,"Bad GPT partition, invalid LBA self location.\n");
+    screen_buffer_add("Bad GPT partition, invalid LBA self location.\n");
     free(gpt);
     return NULL;
   }
   if(le64(gpt->hdr_lba_start) >= le64(gpt->hdr_lba_end))
   {
-    aff_buffer(BUFFER_ADD,"Bad GPT partition, invalid LBA start/end location.\n");
+    screen_buffer_add("Bad GPT partition, invalid LBA start/end location.\n");
     free(gpt);
     return NULL;
   }
   if(le32(gpt->hdr_revision)!=GPT_HDR_REVISION)
   {
-    aff_buffer(BUFFER_ADD,"GPT: Warning - not revision 1.0\n");
+    screen_buffer_add("GPT: Warning - not revision 1.0\n");
   }
   if(le32(gpt->__reserved)!=0)
   {
-    aff_buffer(BUFFER_ADD,"GPT: Warning - __reserved!=0\n");
+    screen_buffer_add("GPT: Warning - __reserved!=0\n");
   }
   if(le32(gpt->hdr_entries)==0 || le32(gpt->hdr_entries)>4096)
   {
-    aff_buffer(BUFFER_ADD,"GPT: invalid number (%u) of partition entries.\n",
+    screen_buffer_add("GPT: invalid number (%u) of partition entries.\n",
         (unsigned int)le32(gpt->hdr_entries));
     free(gpt);
     return NULL;
@@ -245,7 +237,7 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
   /* le32(gpt->hdr_entsz)==128 */
   if(le32(gpt->hdr_entsz)%8!=0 || le32(gpt->hdr_entsz)<128 || le32(gpt->hdr_entsz)>4096)
   {
-    aff_buffer(BUFFER_ADD,"GPT: invalid partition entry size.\n");
+    screen_buffer_add("GPT: invalid partition entry size.\n");
     free(gpt);
     return NULL;
   }
@@ -253,7 +245,7 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
   gpt_entries_size=le32(gpt->hdr_entries) * le32(gpt->hdr_entsz);
   if(gpt_entries_size<16384)
   {
-    aff_buffer(BUFFER_ADD,"GPT: A minimum of 16,384 bytes of space must be reserved for the GUID Partition Entry array.\n");
+    screen_buffer_add("GPT: A minimum of 16,384 bytes of space must be reserved for the GUID Partition Entry array.\n");
     free(gpt);
     return NULL;
   }
@@ -261,13 +253,13 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
   if((uint64_t) le64(gpt->hdr_lba_self) + le32(gpt->hdr_size) - 1 >= gpt_entries_offset ||
       gpt_entries_offset >= le64(gpt->hdr_lba_start) * disk_car->sector_size)
   {
-    aff_buffer(BUFFER_ADD, "GPT: The primary GUID Partition Entry array must be located after the primary GUID Partition Table Header and end before the FirstUsableLBA.\n");
+    screen_buffer_add( "GPT: The primary GUID Partition Entry array must be located after the primary GUID Partition Table Header and end before the FirstUsableLBA.\n");
     free(gpt);
     return NULL;
   }
 
   gpt_entries=(struct gpt_ent*)MALLOC(gpt_entries_size);
-  if(disk_car->read(disk_car, gpt_entries_size, gpt_entries, gpt_entries_offset)!=0)
+  if((unsigned)disk_car->pread(disk_car, gpt_entries, gpt_entries_size, gpt_entries_offset) != gpt_entries_size)
   {
     free(gpt_entries);
     free(gpt);
@@ -278,7 +270,7 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
     crc=get_crc32(gpt_entries, gpt_entries_size, 0xFFFFFFFF)^0xFFFFFFFF;
     if(crc!=le32(gpt->hdr_crc_table))
     {
-      aff_buffer(BUFFER_ADD,"Bad GPT partition entries, invalid checksum.\n");
+      screen_buffer_add("Bad GPT partition entries, invalid checksum.\n");
       free(gpt_entries);
       free(gpt);
       return NULL;
@@ -317,6 +309,26 @@ list_part_t *read_part_gpt(disk_t *disk_car, const int verbose, const int savehe
   return new_list_part;
 }
 
+static void efi_generate_uuid(efi_guid_t *ent_uuid)
+{
+#ifdef HAVE_UUID_GENERATE
+  uuid_generate((unsigned char*)ent_uuid);
+#elif defined HAVE_UUIDGEN
+  uuidgen((struct uuid*)ent_uuid,1);
+#elif defined HAVE_UUID_CREATE
+  uuid_t *uuid;
+  char *data_ptr=(char*)&ent_uuid;
+  size_t data_len=sizeof(ent_uuid);;
+  uuid_create(&uuid);
+  uuid_make(uuid, UUID_MAKE_V1);
+  uuid_export(uuid, UUID_FMT_BIN, (void **)&data_ptr, &data_len);
+  uuid_destroy(uuid);
+#else
+#warning You need a uuid_generate, uuidgen or uuid_create function
+#endif
+  swap_uuid_and_efi_guid(ent_uuid);
+}
+
 static void partition_generate_gpt_entry(struct gpt_ent* gpt_entry, const partition_t *partition, const disk_t *disk_car)
 {
   guid_cpy(&gpt_entry->ent_type, &partition->part_type_gpt);
@@ -326,16 +338,7 @@ static void partition_generate_gpt_entry(struct gpt_ent* gpt_entry, const partit
   if(guid_cmp(partition->part_uuid, GPT_ENT_TYPE_UNUSED)!=0)
     guid_cpy(&gpt_entry->ent_uuid, &partition->part_uuid);
   else
-  {
-#ifdef HAVE_UUID_GENERATE
-    uuid_generate((unsigned char*)(&gpt_entry->ent_uuid));
-#elif defined HAVE_UUIDGEN
-    uuidgen((struct uuid*)(&gpt_entry->ent_uuid),1);
-#else
-#warn You need a uuid_generate or uuidgen function
-#endif
-    swap_uuid_and_efi_guid((efi_guid_t *)(&gpt_entry->ent_uuid));
-  }
+    efi_generate_uuid(&gpt_entry->ent_uuid);
   gpt_entry->ent_attr=le64(0);  /* May need fixing */
 }
 
@@ -467,26 +470,19 @@ static int write_part_gpt(disk_t *disk_car, const list_part_t *list_part, const
   gpt->__reserved=le32(0);
   gpt->hdr_lba_start=le64(1 + gpt_entries_size/disk_car->sector_size + 1);
   gpt->hdr_lba_end=le64((disk_car->disk_size-1 - gpt_entries_size)/disk_car->sector_size - 1);
-#ifdef HAVE_UUID_GENERATE
-    uuid_generate((unsigned char*)(&gpt->hdr_guid));
-#elif defined HAVE_UUIDGEN
-    uuidgen((struct uuid*)(&gpt->hdr_guid),1);
-#else
-#warn You need a uuid_generate or uuidgen function
-#endif
-  swap_uuid_and_efi_guid((efi_guid_t *)(&gpt->hdr_guid));
+  efi_generate_uuid(&gpt->hdr_guid);
   gpt->hdr_crc_table=le32(get_crc32(gpt_entries, gpt_entries_size, 0xFFFFFFFF)^0xFFFFFFFF);
   gpt->hdr_lba_self=le64(1);
   gpt->hdr_lba_alt=le64((disk_car->disk_size-1)/disk_car->sector_size);
   gpt->hdr_lba_table=le64(1+1);
   gpt->hdr_crc_self=le32(get_crc32(gpt, le32(gpt->hdr_size), 0xFFFFFFFF)^0xFFFFFFFF);
-  if(disk_car->write(disk_car, gpt_entries_size, gpt_entries, le64(gpt->hdr_lba_table) * disk_car->sector_size))
+  if((unsigned)disk_car->pwrite(disk_car, gpt_entries, gpt_entries_size, le64(gpt->hdr_lba_table) * disk_car->sector_size) != gpt_entries_size)
   {
     free(gpt);
     free(gpt_entries);
     return 1;
   }
-  if(disk_car->write(disk_car, disk_car->sector_size, gpt, le64(gpt->hdr_lba_self) * disk_car->sector_size))
+  if((unsigned)disk_car->pwrite(disk_car, gpt, disk_car->sector_size, le64(gpt->hdr_lba_self) * disk_car->sector_size) != disk_car->sector_size)
   {
     free(gpt);
     free(gpt_entries);
@@ -496,13 +492,13 @@ static int write_part_gpt(disk_t *disk_car, const list_part_t *list_part, const
   gpt->hdr_lba_alt=le64(1);
   gpt->hdr_lba_table=le64((disk_car->disk_size-1 - gpt_entries_size)/disk_car->sector_size);
   gpt->hdr_crc_self=le32(get_crc32(gpt, le32(gpt->hdr_size), 0xFFFFFFFF)^0xFFFFFFFF);
-  if(disk_car->write(disk_car, gpt_entries_size, gpt_entries, le64(gpt->hdr_lba_table) * disk_car->sector_size))
+  if((unsigned)disk_car->pwrite(disk_car, gpt_entries, gpt_entries_size, le64(gpt->hdr_lba_table) * disk_car->sector_size) != gpt_entries_size)
   {
     free(gpt);
     free(gpt_entries);
     return 1;
   }
-  if(disk_car->write(disk_car, disk_car->sector_size, gpt, le64(gpt->hdr_lba_self) * disk_car->sector_size))
+  if((unsigned)disk_car->pwrite(disk_car, gpt, disk_car->sector_size, le64(gpt->hdr_lba_self) * disk_car->sector_size) != disk_car->sector_size)
   {
     free(gpt);
     free(gpt_entries);
@@ -528,7 +524,7 @@ static list_part_t *init_part_order_gpt(const disk_t *disk_car, list_part_t *lis
   return list_part;
 }
 
-static list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
+list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   partition_t *new_partition=partition_new(&arch_gpt);
   new_partition->part_offset=disk_car->sector_size;
@@ -566,7 +562,7 @@ static list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_par
     else if(strncmp(*current_cmd,"T,",2)==0)
     {
       (*current_cmd)+=2;
-      change_part_type(disk_car,new_partition,current_cmd);
+      change_part_type_cli(disk_car,new_partition,current_cmd);
     }
     else if(new_partition->part_size>0 && guid_cmp(new_partition->part_type_gpt, GPT_ENT_TYPE_UNUSED)!=0)
     {
@@ -590,106 +586,6 @@ static list_part_t *add_partition_gpt_cli(disk_t *disk_car,list_part_t *list_par
   }
 }
 
-#ifdef HAVE_NCURSES
-static list_part_t *add_partition_gpt_ncurses(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
-{
-  int position=0;
-  int done = FALSE;
-  partition_t *new_partition=partition_new(&arch_gpt);
-  new_partition->part_offset=disk_car->sector_size;
-  new_partition->part_size=disk_car->disk_size-disk_car->sector_size;
-  while (done==FALSE)
-  {
-    int command;
-    static struct MenuItem menuGeometry[]=
-    {
-      { 's', "Sector", 	"Change starting sector" },
-      { 'S', "Sector", 	"Change ending sector" },
-      { 'T' ,"Type",	"Change partition type"},
-      { 'd', "Done", "" },
-      { 0, NULL, NULL }
-    };
-    aff_copy(stdscr);
-    wmove(stdscr,4,0);
-    wprintw(stdscr,"%s",disk_car->description(disk_car));
-    wmove(stdscr,10, 0);
-    wclrtoeol(stdscr);
-    aff_part(stdscr, AFF_PART_BASE, disk_car, new_partition);
-    wmove(stdscr,INTER_GEOM_Y, INTER_GEOM_X);
-    wclrtoeol(stdscr);
-    wrefresh(stdscr);
-    command=wmenuSimple(stdscr,menuGeometry, position);
-    switch (command) {
-      case 's':
-        {
-          uint64_t part_offset;
-          part_offset=new_partition->part_offset;
-          wmove(stdscr, INTER_GEOM_Y, INTER_GEOM_X);
-          new_partition->part_offset=(uint64_t)ask_number(
-              new_partition->part_offset/disk_car->sector_size,
-              1,
-              (disk_car->disk_size-1)/disk_car->sector_size,
-              "Enter the starting sector ") *
-            (uint64_t)disk_car->sector_size;
-          new_partition->part_size=new_partition->part_size + part_offset - new_partition->part_offset;
-          position=1;
-        }
-        break;
-      case 'S':
-        wmove(stdscr, INTER_GEOM_Y, INTER_GEOM_X);
-        new_partition->part_size=(uint64_t)ask_number(
-            (new_partition->part_offset+new_partition->part_size-1)/disk_car->sector_size,
-            new_partition->part_offset/disk_car->sector_size,
-            (disk_car->disk_size-1)/disk_car->sector_size,
-            "Enter the ending sector ") *
-          (uint64_t)disk_car->sector_size +
-          disk_car->sector_size - new_partition->part_offset;
-        position=2;
-        break;
-      case 'T':
-      case 't':
-        change_part_type(disk_car,new_partition, current_cmd);
-        position=3;
-        break;
-      case key_ESC:
-      case 'd':
-      case 'D':
-      case 'q':
-      case 'Q':
-        done = TRUE;
-        break;
-    }
-  }
-  if(new_partition->part_size>0 && guid_cmp(new_partition->part_type_gpt, GPT_ENT_TYPE_UNUSED)!=0)
-  {
-    int insert_error=0;
-    list_part_t *new_list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
-    if(insert_error>0)
-    {
-      free(new_partition);
-      return new_list_part;
-    }
-    new_partition->status=STATUS_PRIM;
-    if(test_structure_gpt(list_part)!=0)
-      new_partition->status=STATUS_DELETED;
-    return new_list_part;
-  }
-  free(new_partition);
-  return list_part;
-}
-#endif
-
-static list_part_t *add_partition_gpt(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
-{
-  if(*current_cmd!=NULL)
-    return add_partition_gpt_cli(disk_car, list_part, verbose, current_cmd);
-#ifdef HAVE_NCURSES
-  return add_partition_gpt_ncurses(disk_car, list_part, verbose, current_cmd);
-#else
-  return list_part;
-#endif
-}
-
 static void set_next_status_gpt(const disk_t *disk_car, partition_t *partition)
 {
   if(partition->status==STATUS_DELETED)
@@ -708,23 +604,6 @@ static int test_structure_gpt(list_part_t *list_part)
   return res;
 }
 
-#if 0
-static efi_guid_t get_part_type_gpt(const partition_t *partition)
-{
-  return partition->part_type_gpt;
-}
-
-static int set_part_type_gpt(partition_t *partition, efi_guid_t part_type_gpt)
-{
-  if(guid_cmp(partition->part_type_gpt, GPT_ENT_TYPE_UNUSED)!=0)
-  {
-    guid_cpy(&partition->part_type_gpt, &part_type_gpt);
-    return 0;
-  }
-  return 1;
-}
-#endif
-
 static int is_part_known_gpt(const partition_t *partition)
 {
   return (guid_cmp(partition->part_type_gpt, GPT_ENT_TYPE_UNUSED)!=0);
@@ -752,9 +631,6 @@ static void init_structure_gpt(const disk_t *disk_car,list_part_t *list_part, co
     if(element->to_be_removed==0)
       new_list_part=insert_new_partition(new_list_part, element->part, 0, &insert_error);
   }
-#ifdef DEBUG
-  check_list_part(new_list_part);
-#endif
     for(element=new_list_part;element!=NULL;element=element->next)
       element->part->status=STATUS_PRIM;
   if(disk_car->arch->test_structure(new_list_part))
@@ -763,9 +639,6 @@ static void init_structure_gpt(const disk_t *disk_car,list_part_t *list_part, co
       element->part->status=STATUS_DELETED;
   }
   part_free_list_only(new_list_part);
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
 }
 
 static int check_part_gpt(disk_t *disk_car,const int verbose,partition_t *partition, const int saveheader)
@@ -777,10 +650,12 @@ static int check_part_gpt(disk_t *disk_car,const int verbose,partition_t *partit
       guid_cmp(partition->part_type_gpt, GPT_ENT_TYPE_MS_RESERVED)==0)
   {
     ret=check_FAT(disk_car,partition,verbose);
+    if(ret!=0)
+      ret=check_EXFAT(disk_car, partition);
     if(ret!=0)
       ret=check_NTFS(disk_car,partition,verbose,0);
     if(ret!=0)
-      ret=check_JFS(disk_car,partition,verbose);
+      ret=check_JFS(disk_car, partition);
     if(ret!=0)
       ret=check_rfs(disk_car,partition,verbose);
     if(ret!=0)
@@ -791,7 +666,7 @@ static int check_part_gpt(disk_t *disk_car,const int verbose,partition_t *partit
       ret=check_xfs(disk_car,partition,verbose);
     if(ret!=0)
     {
-      aff_buffer(BUFFER_ADD,"No FAT, NTFS, EXT2, JFS, Reiser, cramfs or XFS marker\n"); 
+      screen_buffer_add("No FAT, NTFS, EXT2, JFS, Reiser, cramfs or XFS marker\n"); 
     }
   }
   /* TODO: complete me */
@@ -799,6 +674,7 @@ static int check_part_gpt(disk_t *disk_car,const int verbose,partition_t *partit
   if(ret!=0)
   {
     log_error("check_part_gpt failed for partition\n");
+    log_partition(disk_car, partition);
     aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
     if(saveheader>0)
     {
