@@ -2,7 +2,7 @@
 
     File: partsun.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -34,12 +35,9 @@
 #include <ctype.h>      /* tolower */
 #include "types.h"
 #include "common.h"
-#include "testdisk.h"
 #include "fnctdsk.h"
-//#include "analyse.h"
 #include "lang.h"
 #include "intrf.h"
-#include "intrfn.h"
 #include "chgtype.h"
 #include "sun.h"
 #include "swap.h"
@@ -57,20 +55,20 @@
 #include "xfs.h"
 #include "ufs.h"
 #include "log.h"
+#include "partsun.h"
 
-static int check_part_sun(disk_t *disk_car, const int debug,partition_t *partition,const int saveheader);
-static int get_geometry_from_sunmbr(const unsigned char *buffer, const int debug, CHS_t *geometry);
-static list_part_t *read_part_sun(disk_t *disk_car, const int debug, const int saveheader);
-static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro , const int debug, const int align);
+static int check_part_sun(disk_t *disk_car, const int verbose,partition_t *partition,const int saveheader);
+static int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry);
+static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int saveheader);
+static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose, const int align);
 static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *list_part);
-static list_part_t *add_partition_sun(disk_t *disk_car,list_part_t *list_part, const int debug, char **current_cmd);
 static void set_next_status_sun(const disk_t *disk_car, partition_t *partition);
 static int test_structure_sun(list_part_t *list_part);
 static int set_part_type_sun(partition_t *partition, unsigned int part_type_sun);
 static int is_part_known_sun(const partition_t *partition);
-static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, const int debug);
-static const char *get_partition_name_sun(const partition_t *partition);
-static const char *get_partition_name_sun_aux(const unsigned int part_type_sun);
+static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, const int verbose);
+static const char *get_partition_typename_sun(const partition_t *partition);
+static const char *get_partition_typename_sun_aux(const unsigned int part_type_sun);
 static unsigned int get_part_type_sun(const partition_t *partition);
 
 static const struct systypes sun_sys_types[] = {
@@ -103,16 +101,15 @@ arch_fnct_t arch_sun=
   .get_geometry_from_mbr=get_geometry_from_sunmbr,
   .check_part=check_part_sun,
   .write_MBR_code=NULL,
-  .add_partition=add_partition_sun,
   .set_prev_status=set_next_status_sun,
   .set_next_status=set_next_status_sun,
   .test_structure=test_structure_sun,
+  .get_part_type=get_part_type_sun,
   .set_part_type=set_part_type_sun,
-  .is_part_known=is_part_known_sun,
   .init_structure=init_structure_sun,
   .erase_list_part=NULL,
-  .get_partition_name=get_partition_name_sun,
-  .get_part_type=get_part_type_sun
+  .get_partition_typename=get_partition_typename_sun,
+  .is_part_known=is_part_known_sun
 };
 
 static unsigned int get_part_type_sun(const partition_t *partition)
@@ -120,38 +117,42 @@ static unsigned int get_part_type_sun(const partition_t *partition)
   return partition->part_type_sun;
 }
 
-int get_geometry_from_sunmbr(const unsigned char *buffer, const int debug, CHS_t *geometry)
+int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry)
 {
   const sun_partition *sunlabel=(const sun_partition*)buffer;
-  if(debug>1)
+  if(verbose>1)
   {
     log_trace("get_geometry_from_sunmbr\n");
   }
-  geometry->cylinder=0;
-  geometry->head=be16(sunlabel->ntrks)-1;
-  geometry->sector=be16(sunlabel->nsect);
-  if(geometry->sector>0)
+  geometry->cylinders=0;
+  geometry->heads_per_cylinder=be16(sunlabel->ntrks);
+  geometry->sectors_per_head=be16(sunlabel->nsect);
+  if(geometry->sectors_per_head>0)
   {
-    log_info("Geometry from SUN MBR: head=%u sector=%u\n",geometry->head+1,geometry->sector);
+    log_info("Geometry from SUN MBR: head=%u sector=%u\n",
+	geometry->heads_per_cylinder, geometry->sectors_per_head);
   }
   return 0;
 }
 
-list_part_t *read_part_sun(disk_t *disk_car, const int debug, const int saveheader)
+list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int saveheader)
 {
-  unsigned char buffer[DEFAULT_SECTOR_SIZE];
-  sun_partition *sunlabel=(sun_partition*)&buffer;
-  list_part_t *new_list_part=NULL;
   unsigned int i;
-  aff_buffer(BUFFER_RESET,"Q");
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  sun_partition *sunlabel;
+  list_part_t *new_list_part=NULL;
+  unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
+  screen_buffer_reset();
+  sunlabel=(sun_partition*)buffer;
+  if(disk_car->pread(disk_car, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
   {
-    wdoprintf(stdscr,msg_PART_RD_ERR);
+    screen_buffer_add( msg_PART_RD_ERR);
+    free(buffer);
     return NULL;
   }
   if (be16(sunlabel->magic) != SUN_LABEL_MAGIC)
   {
-    aff_buffer(BUFFER_ADD,"Bad SUN partition\n");
+    screen_buffer_add("Bad SUN partition\n");
+    free(buffer);
     return NULL;
   }
   for(i=0;i<8;i++)
@@ -160,32 +161,35 @@ list_part_t *read_part_sun(disk_t *disk_car, const int debug, const int savehead
 	 && sunlabel->infos[i].id > 0
 	 && sunlabel->infos[i].id != PSUN_WHOLE_DISK)
      {
-       partition_t *new_partition=partition_new();
-       partition_reset(new_partition);
+       int insert_error=0;
+       partition_t *new_partition=partition_new(&arch_sun);
        new_partition->order=i;
        new_partition->part_type_sun=sunlabel->infos[i].id;
        new_partition->part_offset=be32(sunlabel->partitions[i].start_cylinder)*be16(sunlabel->ntrks)*be16(sunlabel->nsect)*disk_car->sector_size;
        new_partition->part_size=(uint64_t)be32(sunlabel->partitions[i].num_sectors)*disk_car->sector_size;
        new_partition->status=STATUS_PRIM;
-       disk_car->arch->check_part(disk_car,debug,new_partition,saveheader);
-       new_list_part=insert_new_partition(new_list_part,new_partition);
-       aff_part_buffer(AFF_PART_ORDER,disk_car,new_partition);
+       disk_car->arch->check_part(disk_car,verbose,new_partition,saveheader);
+       aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
+       new_list_part=insert_new_partition(new_list_part, new_partition, 0, &insert_error);
+       if(insert_error>0)
+	 free(new_partition);
      }
   }
+  free(buffer);
   return new_list_part;
 }
 
-static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro, const int debug, const int align)
-{ /* TODO: Implement it */
+static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align)
+{
+  /* TODO: Implement it */
   if(ro==0)
-  {
-    not_implemented("write_part_sun");
-  }
+    return -1;
   return 0;
 }
 
 static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *list_part)
 {
+  int insert_error=0;
   int nbr_prim=0;
   partition_t *new_partition;
   list_part_t *element;
@@ -204,88 +208,68 @@ static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *lis
 	break;
     }
   }
-  new_partition=partition_new();
+  new_partition=partition_new(&arch_sun);
   new_partition->part_offset=0;
   new_partition->part_size=disk_car->disk_size;
   new_partition->status=STATUS_PRIM;
   new_partition->part_type_sun=PSUN_WHOLE_DISK;
   new_partition->order=2;
-  return insert_new_partition(list_part, new_partition);
+  list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
+  if(insert_error>0)
+    free(new_partition);
+  return list_part;
 }
 
-static list_part_t *add_partition_sun(disk_t *disk_car,list_part_t *list_part, const int debug, char **current_cmd)
+list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   CHS_t start,end;
-  partition_t *new_partition=partition_new();
-  int position=0;
+  partition_t *new_partition=partition_new(&arch_sun);
   start.cylinder=0;
   start.head=0;
   start.sector=1;
-  end.cylinder=disk_car->CHS.cylinder;
-  end.head=disk_car->CHS.head;
-  end.sector=disk_car->CHS.sector;
+  end.cylinder=disk_car->geom.cylinders-1;
+  end.head=disk_car->geom.heads_per_cylinder-1;
+  end.sector=disk_car->geom.sectors_per_head;
+  while(*current_cmd[0]==',')
+    (*current_cmd)++;
+  while(1)
   {
-    int done = FALSE;
-    while (done==FALSE) {
-      int command;
-      static struct MenuItem menuGeometry[]=
-      {
-	{ 'c', "Cylinders", 	"Change starting cylinder" },
-	{ 'C', "Cylinders", 	"Change ending cylinder" },
-	{ 'T' ,"Type",		"Change partition type"},
-	{ 'd', "Done", "" },
-	{ 0, NULL, NULL }
-      };
-      aff_copy(stdscr);
-      wmove(stdscr,4,0);
-      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
-      new_partition->part_offset=CHS2offset(disk_car,&start);
-      new_partition->part_size=CHS2offset(disk_car,&end) - new_partition->part_offset + disk_car->sector_size;
-      wmove(stdscr,10, 0);
-      wclrtoeol(stdscr);
-      aff_part(stdscr,AFF_PART_SHORT,disk_car,new_partition);
-      wmove(stdscr,INTER_GEOM_Y, INTER_GEOM_X);
-      wclrtoeol(stdscr);
-      wrefresh(stdscr);
-      command=wmenuSimple(stdscr,menuGeometry, position);
-      switch (command) {
-	case 'c':
-	  wmove(stdscr, INTER_GEOM_Y, INTER_GEOM_X);
-	  start.cylinder=ask_number(start.cylinder,0,disk_car->CHS.cylinder,"Enter the starting cylinder ");
-	  position=1;
-	  break;
-	case 'C':
-	  wmove(stdscr, INTER_GEOM_Y, INTER_GEOM_X);
-	  end.cylinder=ask_number(end.cylinder,start.cylinder,disk_car->CHS.cylinder,"Enter the ending cylinder ");
-	  position=2;
-	  break;
-	case 'T':
-	case 't':
-	  change_part_type(disk_car,new_partition,current_cmd);
-	  position=3;
-	  break;
-	case key_ESC:
-	case 'd':
-	case 'D':
-	case 'q':
-	case 'Q':
-	  done = TRUE;
-	  break;
-      }
-    }
-  }
-  if((CHS2offset(disk_car,&end)>new_partition->part_offset)&& new_partition->part_type_sun>0)
-  {
-    list_part_t *new_list_part=insert_new_partition(list_part, new_partition);
-    new_partition->status=STATUS_PRIM;
-    if(test_structure_sun(list_part)!=0)
+    if(strncmp(*current_cmd,"c,",2)==0)
     {
-      new_partition->status=STATUS_DELETED;
+      (*current_cmd)+=2;
+      start.cylinder=ask_number_cli(current_cmd, start.cylinder,0,disk_car->geom.cylinders-1,"Enter the starting cylinder ");
+    }
+    else if(strncmp(*current_cmd,"C,",2)==0)
+    {
+      (*current_cmd)+=2;
+      end.cylinder=ask_number_cli(current_cmd, end.cylinder,start.cylinder,disk_car->geom.cylinders-1,"Enter the ending cylinder ");
+    }
+    else if(strncmp(*current_cmd,"T,",2)==0)
+    {
+      (*current_cmd)+=2;
+      change_part_type_cli(disk_car,new_partition,current_cmd);
+    }
+    else if((CHS2offset(disk_car,&end)>new_partition->part_offset) &&
+      new_partition->part_type_sun>0)
+    {
+      int insert_error=0;
+      list_part_t *new_list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
+      if(insert_error>0)
+      {
+	free(new_partition);
+	return new_list_part;
+      }
+      new_partition->status=STATUS_PRIM;
+      if(test_structure_sun(list_part)!=0)
+	new_partition->status=STATUS_DELETED;
+      return new_list_part;
+    }
+    else
+    {
+      free(new_partition);
+      return list_part;
     }
-    return new_list_part;
   }
-  free(new_partition);
-  return list_part;
 }
 
 static void set_next_status_sun(const disk_t *disk_car, partition_t *partition)
@@ -299,29 +283,10 @@ static void set_next_status_sun(const disk_t *disk_car, partition_t *partition)
 static int test_structure_sun(list_part_t *list_part)
 { /* Return 1 if bad*/
   list_part_t *new_list_part=NULL;
-  list_part_t *element;
-  list_part_t *new_element;
-  int res=0;
-  /* Sort list_part in new_list_part */
-  for(element=list_part;element!=NULL;element=element->next)
-  {
-    if(element->part->status!=STATUS_DELETED)
-      new_list_part=insert_new_partition(new_list_part,element->part);
-  }
-  /* Test overlapping */
-  for(element=new_list_part;element!=NULL;element=element->next)
-  {
-    if(((element->prev!=NULL) && (element->part->part_offset<=element->prev->part->part_offset+element->prev->part->part_size-1)) ||
-	((element->next!=NULL) && (element->part->part_offset+element->part->part_size-1>=element->next->part->part_offset)))
-    {
-      res=1;
-    }
-  }
-  for(element=new_list_part;element!=NULL;element=new_element)
-  {
-    new_element=element->next;
-    free(element);
-  }
+  int res;
+  new_list_part=gen_sorted_partition_list(list_part);
+  res=is_part_overlapping(new_list_part);
+  part_free_list_only(new_list_part);
   return res;
 }
 
@@ -340,7 +305,7 @@ static int is_part_known_sun(const partition_t *partition)
   return (partition->part_type_sun!=PSUN_UNK);
 }
 
-static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, const int debug)
+static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, const int verbose)
 {
   list_part_t *element;
   list_part_t *new_list_part=NULL;
@@ -349,44 +314,32 @@ static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, co
     element->to_be_removed=0;
   for(element=list_part;element!=NULL;element=element->next)
   {
-    int to_be_removed=0;
     list_part_t *element2;
     for(element2=element->next;element2!=NULL;element2=element2->next)
+    {
       if(element->part->part_offset+element->part->part_size-1 >= element2->part->part_offset)
       {
-	to_be_removed=1;
+	element->to_be_removed=1;
 	element2->to_be_removed=1;
       }
-    if(to_be_removed)
-      element->to_be_removed=1;
+    }
     if(element->to_be_removed==0)
-      new_list_part=insert_new_partition_aux(new_list_part,element_new(element->part),1);
+    {
+      int insert_error=0;
+      new_list_part=insert_new_partition(new_list_part, element->part, 0, &insert_error);
+    }
   }
-#ifdef DEBUG
-  check_list_part(new_list_part);
-#endif
-    for(element=new_list_part;element!=NULL;element=element->next)
-      element->part->status=STATUS_PRIM;
+  for(element=new_list_part;element!=NULL;element=element->next)
+    element->part->status=STATUS_PRIM;
   if(disk_car->arch->test_structure(new_list_part))
   {
     for(element=new_list_part;element!=NULL;element=element->next)
       element->part->status=STATUS_DELETED;
   }
-  {
-    /* free */
-    list_part_t *new_element;
-    for(element=new_list_part;element!=NULL;element=new_element)
-    {
-      new_element=element->next;
-      free(element);
-    }
-  }
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
+  part_free_list_only(new_list_part);
 }
 
-static int check_part_sun(disk_t *disk_car,const int debug,partition_t *partition, const int saveheader)
+static int check_part_sun(disk_t *disk_car,const int verbose,partition_t *partition, const int saveheader)
 {
   int ret=0;
   switch(partition->part_type_sun)
@@ -398,35 +351,35 @@ static int check_part_sun(disk_t *disk_car,const int debug,partition_t *partitio
     case PSUN_VAR:
     case PSUN_HOME:
     case PSUN_ALT:
-      ret=check_ufs(disk_car,partition,debug);
+      ret=check_ufs(disk_car,partition,verbose);
       break;
     case PSUN_LINUX:
-      ret=check_JFS(disk_car,partition,debug);
+      ret=check_JFS(disk_car, partition);
       if(ret!=0)
       {
-	ret=check_rfs(disk_car,partition,debug);
+	ret=check_rfs(disk_car,partition,verbose);
       }
       if(ret!=0)
       {
-	ret=check_EXT2(disk_car,partition,debug);
+	ret=check_EXT2(disk_car,partition,verbose);
       }
       if(ret!=0)
       {
-	ret=check_cramfs(disk_car,partition,debug);
+	ret=check_cramfs(disk_car,partition,verbose);
       }
       if(ret!=0)
       {
-	ret=check_xfs(disk_car,partition,debug);
+	ret=check_xfs(disk_car,partition,verbose);
       }
       if(ret!=0)
-      { aff_buffer(BUFFER_ADD,"No EXT2, JFS, Reiser, cramfs or XFS marker\n"); }
+      { screen_buffer_add("No EXT2, JFS, Reiser, cramfs or XFS marker\n"); }
       break;
 
     case PSUN_LINSWAP:
-      ret=check_Linux_SWAP(disk_car,partition,debug);
+      ret=check_Linux_SWAP(disk_car, partition);
       break;
     default:
-      if(debug>0)
+      if(verbose>0)
       {
 	log_info("check_part_sun %u type %02X: no test\n",partition->order,partition->part_type_sun);
       }
@@ -435,16 +388,16 @@ static int check_part_sun(disk_t *disk_car,const int debug,partition_t *partitio
   if(ret!=0)
   {
     log_error("check_part_sun failed for partition type %02X\n", partition->part_type_sun);
-    aff_part_buffer(AFF_PART_ORDER,disk_car,partition);
+    aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
     if(saveheader>0)
     {
-      save_header(disk_car,partition,debug);
+      save_header(disk_car,partition,verbose);
     }
   }
   return ret;
 }
 
-static const char *get_partition_name_sun_aux(const unsigned int part_type_sun)
+static const char *get_partition_typename_sun_aux(const unsigned int part_type_sun)
 {
   int i;
   for (i=0; sun_sys_types[i].name!=NULL; i++)
@@ -453,7 +406,7 @@ static const char *get_partition_name_sun_aux(const unsigned int part_type_sun)
   return NULL;
 }
 
-static const char *get_partition_name_sun(const partition_t *partition)
+static const char *get_partition_typename_sun(const partition_t *partition)
 {
-  return get_partition_name_sun_aux(partition->part_type_sun);
+  return get_partition_typename_sun_aux(partition->part_type_sun);
 }
