@@ -119,10 +119,11 @@ char *dal_error(dal_t *dal) {
 static int file_read(dal_t *dal, void *buff, blk_t block, blk_t count) {
   uint64_t off;
   unsigned int blocklen;
-  my_data_t *my_data=(my_data_t*)dal->data;
+  my_data_t *my_data;
 /* log_trace("reiser file_read(dal=%p,buff=%p,block=%ld, count=%ld)\n",dal,buff,block,count); */
   if (!dal || !buff)
     return 0;
+  my_data=(my_data_t*)dal->data;
 #ifdef HAVE_DAL_T_BLOCK_SIZE
   off = (uint64_t)block * (uint64_t)dal->block_size;
   blocklen = count * dal->block_size;
@@ -131,27 +132,29 @@ static int file_read(dal_t *dal, void *buff, blk_t block, blk_t count) {
   blocklen = count * dal->blocksize;
 #endif
 /* log_debug("blocklen=%ld\n",blocklen); */
-  if(my_data->disk_car->read(my_data->disk_car,blocklen,buff,my_data->partition->part_offset+off))
+  if(my_data->disk_car->pread(my_data->disk_car, buff, blocklen, my_data->partition->part_offset + off) != blocklen)
     return 0;
   return 1;
 }
 
 static int file_write(dal_t *dal, void *buff, blk_t block, blk_t count)
 {
-  uint64_t off, blocklen;
   if (!dal || !buff)
     return 0;
-#ifdef HAVE_DAL_T_BLOCK_SIZE
-  off = (uint64_t)block * (uint64_t)dal->block_size;
-  blocklen = (uint64_t)count * (uint64_t)dal->block_size;
-#else
-  off = (uint64_t)block * (uint64_t)dal->blocksize;
-  blocklen = (uint64_t)count * (uint64_t)dal->blocksize;
-#endif
 #ifdef ENABLE_REISERFS_WRITE
-  if(my_data->disk_car->write(my_data->disk_car,blocklen,buff,my_data->partition->part_offset+off))
-    return 0;
-  return 1;
+  {
+    uint64_t off, blocklen;
+#ifdef HAVE_DAL_T_BLOCK_SIZE
+    off = (uint64_t)block * (uint64_t)dal->block_size;
+    blocklen = (uint64_t)count * (uint64_t)dal->block_size;
+#else
+    off = (uint64_t)block * (uint64_t)dal->blocksize;
+    blocklen = (uint64_t)count * (uint64_t)dal->blocksize;
+#endif
+    if(my_data->disk_car->pwrite(my_data->disk_car, buff, blocklen, my_data->partition->part_offset + off) != blocklen)
+      return 0;
+    return 1;
+  }
 #else
   log_info("reiser file_write not implemented\n");
   return 0;
@@ -210,9 +213,10 @@ static blk_t   file_len(dal_t *dal) {
 #else
 static count_t file_len(dal_t *dal) {
 #endif
-  my_data_t *my_data=(my_data_t*)dal->data;
+  my_data_t *my_data;
 /* log_trace("reiser file_len\n"); */
   if (!dal) return 0;
+  my_data=(my_data_t*)dal->data;
 #ifdef HAVE_DAL_T_BLOCK_SIZE
   return my_data->partition->part_size / dal->block_size;
 #else
@@ -441,8 +445,8 @@ static file_data_t *reiser_dir(disk_t *disk_car, const partition_t *partition, d
       new_file->status=0;
       new_file->prev=ls->current_file;
       new_file->next=NULL;
-      new_file->filestat.st_size=entity->stat.st_size;
-      memcpy(&new_file->filestat,&entity->stat,sizeof(new_file->filestat));
+      new_file->stat.st_size=entity->stat.st_size;
+      memcpy(&new_file->stat,&entity->stat,sizeof(new_file->stat));
       reiserfs_object_free(entity);
       if(ls->current_file)
 	ls->current_file->next=new_file;
