@@ -40,17 +40,16 @@
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h" /* get_LBA_part */
-#include "analyse.h"
 #include "savehdr.h"
 #include "log.h"
 #define BACKUP_MAXSIZE 5120
 
-int save_header(disk_t *disk_car,partition_t *partition, const int debug)
+int save_header(disk_t *disk_car,partition_t *partition, const int verbose)
 {
   unsigned char *buffer;
   FILE *f_backup;
   int res=0;
-  if(debug>1)
+  if(verbose>1)
   {
     log_trace("save_header\n");
   }
@@ -60,7 +59,7 @@ int save_header(disk_t *disk_car,partition_t *partition, const int debug)
     log_critical("Can't create header.log file: %s\n",strerror(errno));
     return -1;
   }
-  buffer=MALLOC(256*DEFAULT_SECTOR_SIZE);
+  buffer=(unsigned char *)MALLOC(256*DEFAULT_SECTOR_SIZE);
   memset(buffer,0,DEFAULT_SECTOR_SIZE);
   {
     char status='D';
@@ -80,17 +79,17 @@ int save_header(disk_t *disk_car,partition_t *partition, const int debug)
 	(unsigned long)(partition->part_size/disk_car->sector_size));
   }
   if(fwrite(buffer,DEFAULT_SECTOR_SIZE,1,f_backup)!=1)
-    res=-1;
-  if(res>=0 && disk_car->read(disk_car,256*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
-    res=-1;
+    res = -1;
+  if(res>=0 && disk_car->pread(disk_car, buffer, 256 * DEFAULT_SECTOR_SIZE, partition->part_offset) != 256 * DEFAULT_SECTOR_SIZE)
+    res = -1;
   if(res>=0 && fwrite(buffer,DEFAULT_SECTOR_SIZE,256,f_backup)!=256)
-    res=-1;
+    res = -1;
   fclose(f_backup);
   free(buffer);
   return res;
 }
 
-backup_disk_t *partition_load(const disk_t *disk_car, const int debug)
+backup_disk_t *partition_load(const disk_t *disk_car, const int verbose)
 {
   FILE *f_backup;
   char *buffer;
@@ -102,7 +101,7 @@ backup_disk_t *partition_load(const disk_t *disk_car, const int debug)
   list_backup->list.prev= &list_backup->list;
   list_backup->list.next = &list_backup->list;
 
-  if(debug>1)
+  if(verbose>1)
   {
     log_trace("partition_load\n");
   }
@@ -112,10 +111,10 @@ backup_disk_t *partition_load(const disk_t *disk_car, const int debug)
     log_error("Can't open backup.log file: %s\n",strerror(errno));
     return list_backup;
   }
-  buffer=MALLOC(BACKUP_MAXSIZE);
+  buffer=(char *)MALLOC(BACKUP_MAXSIZE);
   taille=fread(buffer,1,BACKUP_MAXSIZE,f_backup);
   buffer[(taille<BACKUP_MAXSIZE?taille:BACKUP_MAXSIZE-1)]='\0';
-  if(debug>1)
+  if(verbose>1)
   {
     log_info("partition_load backup.log size=%d\n",taille);
   }
@@ -132,66 +131,70 @@ backup_disk_t *partition_load(const disk_t *disk_car, const int debug)
     if(*pos=='#')
     {
       pos++;
-      if(debug>1)
+      if(verbose>1)
       {
-	log_debug("new disk: %s\n",pos);
+        log_verbose("new disk: %s\n",pos);
       }
       if(new_backup!=NULL)
-	td_list_add_tail(&new_backup->list,&list_backup->list);
+        td_list_add_tail(&new_backup->list,&list_backup->list);
       new_backup=(backup_disk_t*)MALLOC(sizeof(*new_backup));
       new_backup->description[0]='\0';
       new_backup->list_part=NULL;
       new_backup->my_time=strtol(pos,&pos,10);
       if(pos!=NULL)
       {
-	strncpy(new_backup->description,++pos,sizeof(new_backup->description));
+        strncpy(new_backup->description,++pos,sizeof(new_backup->description));
       }
     }
     else if(new_backup!=NULL)
     {
-      partition_t *new_partition=partition_new();
+      partition_t *new_partition=partition_new(disk_car->arch);
       char status;
       unsigned int part_type;
       unsigned long part_size;
       unsigned long part_offset;
-      if(debug>1)
+      if(verbose>1)
       {
-	log_debug("new partition\n");
+        log_verbose("new partition\n");
       }
       if(sscanf(pos,"%2u : start=%10lu, size=%10lu, Id=%02X, %c\n",
-	  &new_partition->order, &part_offset,
-	  &part_size,&part_type,&status)==5)
+            &new_partition->order, &part_offset,
+            &part_size,&part_type,&status)==5)
       {
-	new_partition->part_offset=(uint64_t)part_offset*disk_car->sector_size;
-	new_partition->part_size=(uint64_t)part_size*disk_car->sector_size;
-	disk_car->arch->set_part_type(new_partition,part_type);
-	switch(status)
-	{
-	  case 'P':	new_partition->status=STATUS_PRIM; break;
-	  case '*':	new_partition->status=STATUS_PRIM_BOOT; break;
-	  case 'L':	new_partition->status=STATUS_LOG; break;
-	  default:	new_partition->status=STATUS_DELETED; break;
-	}
-	if(new_partition->status!=STATUS_DELETED)
-	{
-	  new_backup->list_part=insert_new_partition(new_backup->list_part,new_partition);
-	}
-	else
-	{
-	  free(new_partition);
-	}
+        new_partition->part_offset=(uint64_t)part_offset*disk_car->sector_size;
+        new_partition->part_size=(uint64_t)part_size*disk_car->sector_size;
+        if(disk_car->arch->set_part_type!=NULL)
+          disk_car->arch->set_part_type(new_partition,part_type);
+        switch(status)
+        {
+          case 'P':	new_partition->status=STATUS_PRIM; break;
+          case '*':	new_partition->status=STATUS_PRIM_BOOT; break;
+          case 'L':	new_partition->status=STATUS_LOG; break;
+          default:	new_partition->status=STATUS_DELETED; break;
+        }
+        if(new_partition->status!=STATUS_DELETED)
+        {
+          int insert_error=0;
+          new_backup->list_part=insert_new_partition(new_backup->list_part, new_partition, 0, &insert_error);
+          if(insert_error>0)
+            free(new_partition);
+        }
+        else
+        {
+          free(new_partition);
+        }
       }
       else
       {
-	log_critical("partition_load: sscanf failed\n");
-	free(new_partition);
-	pos=NULL;
+        log_critical("partition_load: sscanf failed\n");
+        free(new_partition);
+        pos=NULL;
       }
     }
     if(pos!=NULL)
     {
       while(*pos!='\0' && pos<buffer+taille)
-	pos++;
+        pos++;
       pos++;
     }
   }
@@ -202,11 +205,11 @@ backup_disk_t *partition_load(const disk_t *disk_car, const int debug)
   return list_backup;
 }
 
-int partition_save(disk_t *disk_car, list_part_t *list_part, const int debug)
+int partition_save(disk_t *disk_car, list_part_t *list_part, const int verbose)
 {
   list_part_t *parts;
   FILE *f_backup;
-  if(debug>0)
+  if(verbose>0)
   {
     log_trace("partition_save\n");
   }
