@@ -2,7 +2,7 @@
 
     File: testdisk.c
 
-    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,37 +26,30 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include <stdarg.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>	/* dup2 */
-#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#include <ctype.h>      /* tolower */
 #ifdef HAVE_LOCALE_H
 #include <locale.h>	/* setlocale */
 #endif
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
+#ifdef HAVE_TIME_H
+#include <time.h>
 #endif
-#include <errno.h>
 #include "types.h"
 #include "common.h"
-#include "testdisk.h"
-#include "lang.h"
 #include "intrf.h"
+#ifdef HAVE_NCURSES
+#include "intrfn.h"
+#else
+#include <stdio.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "intrface.h"
-#include "godmode.h"
 #include "fnctdsk.h"
 #include "dir.h"
 #include "ext2_dir.h"
@@ -64,140 +57,143 @@
 #include "ntfs_dir.h"
 #include "hdcache.h"
 #include "ewf.h"
-extern const t_arch_fnct arch_i386;
-extern const t_arch_fnct arch_mac;
-extern const t_arch_fnct arch_sun;
+#include "log.h"
+#include "hdaccess.h"
+#include "sudo.h"
+#include "partauto.h"
+#include "misc.h"
+#include "tdisksel.h"
+#include "tlog.h"
 
-static FILE *f_rapport=NULL;
-static int f_status=0;
-
-void ecrit_rapport_string(const char *string,const int max_length)
-{
-  int i;
-  for(i=0;(string[i]!='\0')&&(i<max_length);i++)
-    ecrit_rapport("%c",string[i]);
-}
-
-int ecrit_rapport(const char *_format, ...)
-{
-  int res=0;
-  if(f_rapport!=NULL)
-  {
-    va_list ap;
-    va_start(ap,_format);
-    res=vfprintf(f_rapport,_format,ap);
-    va_end(ap);
-    if(res<0)
-    {
-      f_status=1;
-    }
-    fflush(stderr);
-    if(fflush(f_rapport))
-    {
-      f_status=1;
-    }
-  }
-  return res;
-}
+extern const arch_fnct_t arch_i386;
+extern const arch_fnct_t arch_mac;
+extern const arch_fnct_t arch_sun;
 
 #ifdef HAVE_SIGACTION
-void sighup_hdlr(int shup)
+struct sigaction action;
+void sighup_hdlr(int sig);
+
+void sighup_hdlr(int sig)
 {
-  ecrit_rapport("SIGHUP detected! TestDisk has been killed.\n");
-  if(f_rapport!=NULL)
-  {
-    fclose(f_rapport);
-  }
-  exit(1);
+  if(sig == SIGINT)
+    log_critical("SIGINT detected! TestDisk has been killed.\n");
+  else
+    log_critical("SIGHUP detected! TestDisk has been killed.\n");
+  log_flush();
+  action.sa_handler=SIG_DFL;
+  sigaction(sig,&action,NULL);
+  kill(0, sig);
 }
 #endif
 
-void aff_copy(WINDOW *window)
-{
-  wclear(window);
-  keypad(window, TRUE); /* Need it to get arrow key */
-  wmove(window,0,0);
-  wdoprintf(window, "TestDisk %s, Data Recovery Utility, %s",VERSION,TESTDISKDATE);
-  wmove(window,1,0);
-  wdoprintf(window,"Christophe GRENIER <grenier@cgsecurity.org>");
-  wmove(window,2,0);
-  wdoprintf(window,"http://www.cgsecurity.org");
-}
-
 int main( int argc, char **argv )
 {
   int i;
-  int help=0, create_log=0, debug=0, dump_ind=0;
+  int use_sudo=0;
+  int help=0, version=0, verbose=0, dump_ind=0;
+  int create_log=TD_LOG_NONE;
   int do_list=0;
+  int unit=UNIT_DEFAULT;
   int write_used;
   int saveheader=0;
   int create_backup=0;
   int run_setlocale=1;
   int done=0;
+  int safe=0;
   int testdisk_mode=TESTDISK_O_RDWR|TESTDISK_O_READAHEAD_8K;
-  t_list_disk *list_disk=NULL;
-  t_list_disk *element_disk;
+  list_disk_t *list_disk=NULL;
+  list_disk_t *element_disk;
   const char *cmd_device=NULL;
   char *cmd_run=NULL;
+  const char *logfile="testdisk.log";
 #ifdef TARGET_SOLARIS
-  const t_arch_fnct *arch=&arch_sun;
+  const arch_fnct_t *arch=&arch_sun;
 #elif defined __APPLE__
-  const t_arch_fnct *arch=&arch_mac;
+  const arch_fnct_t *arch=&arch_mac;
 #else
-  const t_arch_fnct *arch=&arch_i386;
+  const arch_fnct_t *arch=&arch_i386;
 #endif
+  FILE *log_handle=NULL;
+  /* srand needed for GPT creation (weak is ok) */
+  srand(time(NULL));
 #ifdef HAVE_SIGACTION
-  struct sigaction action;
-#endif
-#ifdef TESTING
-  srand(1);
-#endif
-#ifdef HAVE_SIGACTION
-  /* set up the signal handler for SIGHUP */
+  /* set up the signal handler for SIGINT & SIGHUP */
+  sigemptyset(&action.sa_mask);
+  sigaddset(&action.sa_mask, SIGINT);
+  sigaddset(&action.sa_mask, SIGHUP);
   action.sa_handler  = sighup_hdlr;
   action.sa_flags = 0;
+  if(sigaction(SIGINT, &action, NULL)==-1)
+  {
+    printf("Error on SIGACTION call\n");
+    return -1;
+  }
   if(sigaction(SIGHUP, &action, NULL)==-1)
   {
     printf("Error on SIGACTION call\n");
     return -1;
   }
 #endif
+  printf("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
   for(i=1;i<argc;i++)
   {
     if((strcmp(argv[i],"/dump")==0) || (strcmp(argv[i],"-dump")==0))
       dump_ind=1;
+    else if((strcmp(argv[i],"/logname")==0) ||(strcmp(argv[i],"-logname")==0))
+    {
+      if(i+2>=argc)
+	help=1;
+      else
+	logfile=argv[++i];
+    }
     else if((strcmp(argv[i],"/log")==0) ||(strcmp(argv[i],"-log")==0))
     {
-      create_log=1;
+      if(create_log==TD_LOG_NONE)
+        create_log=TD_LOG_APPEND;
     }
     else if((strcmp(argv[i],"/debug")==0) || (strcmp(argv[i],"-debug")==0))
     {
-      debug++;
-      create_log=1;
+      verbose++;
+      if(create_log==TD_LOG_NONE)
+        create_log=TD_LOG_APPEND;
     }
-    else if((strcmp(argv[i],"/list")==0) || (strcmp(argv[i],"-list")==0))
-      do_list=1;
-    else if((strcmp(argv[i],"/saveheader")==0) || (strcmp(argv[i],"-saveheader")==0))
-      saveheader=1;
+    else if((strcmp(argv[i],"/all")==0) || (strcmp(argv[i],"-all")==0))
+      testdisk_mode|=TESTDISK_O_ALL;
     else if((strcmp(argv[i],"/backup")==0) || (strcmp(argv[i],"-backup")==0))
       create_backup=1;
     else if((strcmp(argv[i],"/direct")==0) || (strcmp(argv[i],"-direct")==0))
       testdisk_mode|=TESTDISK_O_DIRECT;
-    else if((strcmp(argv[i],"/nosetlocale")==0) || (strcmp(argv[i],"-nosetlocale")==0))
-      run_setlocale=0;
     else if((strcmp(argv[i],"/help")==0) || (strcmp(argv[i],"-help")==0) || (strcmp(argv[i],"--help")==0) ||
       (strcmp(argv[i],"/h")==0) || (strcmp(argv[i],"-h")==0))
       help=1;
+    else if((strcmp(argv[i],"/version")==0) || (strcmp(argv[i],"-version")==0) || (strcmp(argv[i],"--version")==0) ||
+      (strcmp(argv[i],"/v")==0) || (strcmp(argv[i],"-v")==0))
+      version=1;
+    else if(strcmp(argv[i],"/list")==0 || strcmp(argv[i],"-list")==0 || strcmp(argv[i],"-l")==0)
+    {
+      do_list=1;
+    }
+    else if(strcmp(argv[i],"-lu")==0)
+    {
+      do_list=1;
+      unit=UNIT_SECTOR;
+    }
+    else if((strcmp(argv[i],"/nosetlocale")==0) || (strcmp(argv[i],"-nosetlocale")==0))
+      run_setlocale=0;
+    else if((strcmp(argv[i],"/safe")==0) || (strcmp(argv[i],"-safe")==0))
+      safe=1;
+    else if((strcmp(argv[i],"/saveheader")==0) || (strcmp(argv[i],"-saveheader")==0))
+      saveheader=1;
     else if(strcmp(argv[i],"/cmd")==0)
     {
       if(i+2>=argc)
 	help=1;
       else
       {
-	t_param_disk *disk_car;
+	disk_t *disk_car;
 	cmd_device=argv[++i];
 	cmd_run=argv[++i];
-	disk_car=file_test_availability(cmd_device,debug,arch,testdisk_mode);
+	disk_car=file_test_availability(cmd_device,verbose,arch,testdisk_mode);
 	if(disk_car==NULL)
 	{
 	  printf("\nUnable to open file or device %s\n",cmd_device);
@@ -209,7 +205,7 @@ int main( int argc, char **argv )
     }
     else
     {
-      t_param_disk *disk_car=file_test_availability(argv[i],debug,arch,testdisk_mode);
+      disk_t *disk_car=file_test_availability(argv[i],verbose,arch,testdisk_mode);
       if(disk_car==NULL)
       {
 	printf("\nUnable to open file or device %s\n",argv[i]);
@@ -219,12 +215,22 @@ int main( int argc, char **argv )
 	list_disk=insert_new_disk(list_disk,disk_car);
     }
   }
-  printf("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
+  if(version!=0)
+  {
+    printf("\n");
+    printf("Version: %s\n", VERSION);
+    printf("Compiler: %s\n", get_compiler());
+    printf("ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s\n",
+	td_ext2fs_version(), td_ntfs_version(), td_reiserfs_version(), td_ewf_version());
+    printf("OS: %s\n" , get_os());
+    return 0;
+  }
   if(help!=0)
   {
     printf("\n" \
-	"Usage: testdisk [/log] [/debug] [file or device]\n"\
-	"       testdisk /list  [/log]   [file or device]\n" \
+	"Usage: testdisk [/log] [/debug] [file.dd|file.e01|device]\n"\
+	"       testdisk /list  [/log]   [file.dd|file.e01|device]\n" \
+	"       testdisk /version\n" \
 	"\n" \
 	"/log          : create a testdisk.log file\n" \
 	"/debug        : add debug information\n" \
@@ -244,13 +250,13 @@ int main( int argc, char **argv )
 	"If you have problems with TestDisk or bug reports, please contact me.\n");
     return 0;
   }
-  aff_buffer(BUFFER_RESET,"Q");
+  screen_buffer_reset();
   if(do_list!=0)
   {
     printf("Please wait...\n");
     /* Scan for available device only if no device or image has been supplied in parameter */
     if(list_disk==NULL)
-      list_disk=hd_parse(list_disk,debug,arch,testdisk_mode);
+      list_disk=hd_parse(list_disk,verbose,arch,testdisk_mode);
     /* Activate the cache */
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
       element_disk->disk=new_diskcache(element_disk->disk,testdisk_mode);
@@ -260,7 +266,8 @@ int main( int argc, char **argv )
       printf("%s\n",element_disk->disk->description(element_disk->disk));
     }
 #endif
-    hd_update_all_geometry(list_disk,0,debug);
+    if(safe==0)
+      hd_update_all_geometry(list_disk,0,verbose);
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
       printf("%s, sector size=%u\n",element_disk->disk->description(element_disk->disk),element_disk->disk->sector_size);
@@ -269,26 +276,18 @@ int main( int argc, char **argv )
 
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
-      interface_list(element_disk->disk,debug,0,saveheader,create_backup);
+      autodetect_arch(element_disk->disk);
+      if(unit==UNIT_DEFAULT)
+	autoset_unit(element_disk->disk);
+      else
+	element_disk->disk->unit=unit;
+      interface_list(element_disk->disk, verbose, saveheader, create_backup);
     }
     delete_list_disk(list_disk);
     return 0;
   }
-  if(start_ncurses("TestDisk",argv[0]))
-    return 1;
-  if(argc==1)
-  {
-    debug=1;
-    create_log=ask_log_creation();
-  }
-  if(create_log>0)
-    f_rapport=log_open("testdisk.log",(create_log==1?"a":"w"),"TestDisk",argc,argv);
-  if(f_rapport!=NULL)
-  {
-    ecrit_rapport("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
-    ecrit_rapport(TESTDISK_OS);
-    ecrit_rapport(" (ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s)\n",td_ext2fs_version(),td_ntfs_version(),td_reiserfs_version(), td_ewf_version());
-  }
+  if(create_log!=TD_LOG_NONE)
+    log_handle=log_open(logfile, create_log);
 #ifdef HAVE_SETLOCALE
   if(run_setlocale>0)
   {
@@ -296,36 +295,105 @@ int main( int argc, char **argv )
     locale = setlocale (LC_ALL, "");
     if (locale==NULL) {
       locale = setlocale (LC_ALL, NULL);
-      ecrit_rapport("Failed to set locale, using default '%s'.\n", locale);
+      log_error("Failed to set locale, using default '%s'.\n", locale);
     } else {
-      ecrit_rapport("Using locale '%s'.\n", locale);
+      log_info("Using locale '%s'.\n", locale);
     }
   }
 #endif
+  if(create_log!=TD_LOG_NONE && log_handle==NULL)
+    log_handle=log_open_default(logfile, create_log);
+#ifdef HAVE_NCURSES
+  /* ncurses need locale for correct unicode support */
+  if(start_ncurses("TestDisk",argv[0]))
+    return 1;
+  if(argc==1 && create_log==TD_LOG_NONE)
+  {
+    verbose=1;
+    create_log=ask_testdisk_log_creation();
+    if(create_log==TD_LOG_CREATE || create_log==TD_LOG_APPEND)
+      log_handle=log_open(logfile, create_log);
+  }
+  {
+    const char*filename=logfile;
+    while(create_log!=TD_LOG_NONE && log_handle==NULL)
+    {
+      filename=ask_log_location(filename);
+      if(filename!=NULL)
+	log_handle=log_open(filename, create_log);
+      else
+	create_log=TD_LOG_NONE;
+    }
+  }
+#endif
+  if(log_handle!=NULL)
+  {
+    time_t my_time;
+#ifdef HAVE_DUP2
+    dup2(fileno(log_handle),2);
+#endif
+    my_time=time(NULL);
+    log_info("\n\n%s",ctime(&my_time));
+    log_info("Command line: TestDisk");
+    for(i=1;i<argc;i++)
+      log_info(" %s", argv[i]);
+    log_info("\n\n");
+    log_flush();
+  }
+  log_info("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
+  log_info("OS: %s\n" , get_os());
+  log_info("Compiler: %s\n", get_compiler());
+  log_info("ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s, ewf lib: %s\n",
+      td_ext2fs_version(), td_ntfs_version(), td_reiserfs_version(), td_ewf_version());
+#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(DJGPP)
+#else
+#ifdef HAVE_GETEUID
+  if(geteuid()!=0)
+  {
+    log_warning("User is not root!\n");
+  }
+#endif
+#endif
+#ifdef HAVE_NCURSES
   aff_copy(stdscr);
   wmove(stdscr,5,0);
-  wdoprintf(stdscr, "Please wait...\n");
+  wprintw(stdscr, "Please wait...\n");
+  wrefresh(stdscr);
+#endif
   /* Scan for available device only if no device or image has been supplied in parameter */
   if(list_disk==NULL)
-    list_disk=hd_parse(list_disk,debug,arch,testdisk_mode);
+    list_disk=hd_parse(list_disk,verbose,arch,testdisk_mode);
   /* Activate the cache */
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     element_disk->disk=new_diskcache(element_disk->disk,testdisk_mode);
+#ifdef HAVE_NCURSES
   wmove(stdscr,6,0);
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
   {
-    wdoprintf(stdscr,"%s\n",element_disk->disk->description(element_disk->disk));
+    wprintw(stdscr,"%s\n",element_disk->disk->description(element_disk->disk));
   }
   wrefresh(stdscr);
-  hd_update_all_geometry(list_disk,0,debug);
+#endif
+  if(safe==0)
+    hd_update_all_geometry(list_disk,0,verbose);
   /* save disk parameters to rapport */
-  ecrit_rapport("Hard disk list\n");
+  log_info("Hard disk list\n");
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
-    ecrit_rapport("%s, sector size=%u\n",element_disk->disk->description(element_disk->disk),element_disk->disk->sector_size);
-  ecrit_rapport("\n");
-  do_curses_testdisk(debug,dump_ind,list_disk,saveheader,cmd_device,&cmd_run);
+  {
+    disk_t *disk=element_disk->disk;
+    if(disk->model==NULL)
+      log_info("%s, sector size=%u\n",
+	  disk->description(disk), disk->sector_size);
+    else
+      log_info("%s, sector size=%u - %s\n",
+	  disk->description(disk), disk->sector_size, disk->model);
+  }
+  log_info("\n");
+  use_sudo=do_curses_testdisk(verbose,dump_ind,list_disk,saveheader,cmd_device,&cmd_run);
+#ifdef HAVE_NCURSES
   end_ncurses();
-  ecrit_rapport("\n");
+#endif
+  log_info("\n");
   while(done==0)
   {
     int command='Q';
@@ -340,14 +408,15 @@ int main( int argc, char **argv )
       }
       else if(cmd_run[0]!='\0')
       {
-	ecrit_rapport("error >%s<\n",cmd_run);
+	log_critical("error in command line: %s\n",cmd_run);
+	printf("error in command line: %s\n",cmd_run);
       }
     }
     switch(command)
     {
       case 'L':
 	for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
-	  interface_list(element_disk->disk,debug,0, saveheader, create_backup);
+	  interface_list(element_disk->disk, verbose, saveheader, create_backup);
 	break;
       case 'q':
       case 'Q':
@@ -358,15 +427,8 @@ int main( int argc, char **argv )
   cmd_device=NULL;
   cmd_run=NULL;
   write_used=delete_list_disk(list_disk);
-  if(f_rapport!=NULL)
-  {
-    ecrit_rapport("TestDisk exited normally.\n");
-    if(fclose(f_rapport))
-    {
-      f_status=1;
-    }
-  }
-  if(f_status!=0)
+  log_info("TestDisk exited normally.\n");
+  if(log_close()!=0)
   {
     printf("TestDisk: Log file corrupted!\n");
   }
@@ -378,5 +440,9 @@ int main( int argc, char **argv )
   {
     printf("You have to reboot for the change to take effect.\n");
   }
+#ifdef SUDO_BIN
+  if(use_sudo>0)
+    run_sudo(argc, argv);
+#endif
   return 0;
 }
